/*
 * $Id$
 * 
 * Created on 25.10.2005
 *
 * Written by Tobias Sager (tsa) in 2005
 * 
 * See LICENSE for more information about licensing and warranties.
 */
package ch.toe.famix;

import ch.toe.famix.model.Access;
import ch.toe.famix.model.AccessArgument;
import ch.toe.famix.model.Attribute;
import ch.toe.famix.model.Class;
import ch.toe.famix.model.ExpressionArgument;
import ch.toe.famix.model.FormalParameter;
import ch.toe.famix.model.Function;
import ch.toe.famix.model.GlobalVariable;
import ch.toe.famix.model.ImplicitVariable;
import ch.toe.famix.model.InheritanceDefinition;
import ch.toe.famix.model.Invocation;
import ch.toe.famix.model.LocalVariable;
import ch.toe.famix.model.Method;
import ch.toe.famix.model.Model;
import ch.toe.famix.model.Package;
import ch.toe.famix.model.SourceAnchor;
import ch.toe.famix.model.TypeCast;

/**
 * This defines constants for the FAMIX model.
 * 
 * @author tsa
 */
public final class FAMIXConstants {
	
	/**
	 * A constant for each FAMIX object type. Negative for discriminating
	 * from isomorphism codes.
	 */
	public static final int ACCESS 					= -1;
	public static final int ACCESS_ARGUMENT			= -2;
	public static final int ATTRIBUTE				= -3;
	public static final int CLASS					= -4;
	public static final int EXPRESSION_ARGUMENT		= -5;
	public static final int FAMIX_INSTANCE			= -6;
	public static final int FORMAL_PARAMETER		= -7;
	public static final int FUNCTION				= -8;
	public static final int GLOBAL_VARIABLE			= -9;
	public static final int IMPLICIT_VARIABLE		= -10;
	public static final int INHERITANCE_DEFINITON	= -11;
	public static final int INVOCATION				= -12;
	public static final int LOCAL_VARIABLE			= -13;
	public static final int METHOD					= -14;
	public static final int MODEL					= -15;
	public static final int PACKAGE					= -16;
	public static final int SOURCE_ANCHOR			= -17; 
	public static final int TYPE_CAST				= -18;

	public static final int UNKNOWN					= -100;


	/**
	 * Get the type constant for the given Object (of the FAMIX model).
	 * 
	 * @param o
	 * @return a constant as defined in FAMIXConstants
	 */
	public static int getType(java.lang.Object o) {
		if (o == null)
			return UNKNOWN;
		
		if (o instanceof Access)
			return ACCESS;
		if (o instanceof AccessArgument)
			return ACCESS_ARGUMENT;
		if (o instanceof Attribute)
			return ATTRIBUTE;
		if (o instanceof Class)
			return CLASS;
		if (o instanceof ExpressionArgument)
			return EXPRESSION_ARGUMENT;
		if (o instanceof FAMIXInstance)
			return FAMIX_INSTANCE;
		if (o instanceof FormalParameter)
			return FORMAL_PARAMETER;
		if (o instanceof Function)
			return FUNCTION;
		if (o instanceof GlobalVariable)
			return GLOBAL_VARIABLE;
		if (o instanceof ImplicitVariable)
			return IMPLICIT_VARIABLE;
		if (o instanceof InheritanceDefinition)
			return INHERITANCE_DEFINITON;
		if (o instanceof Invocation)
			return INVOCATION;
		if (o instanceof LocalVariable)
			return LOCAL_VARIABLE;
		if (o instanceof Method)
			return METHOD;
		if (o instanceof Model)
			return MODEL;
		if (o instanceof Package)
			return PACKAGE;
		if (o instanceof SourceAnchor)
			return SOURCE_ANCHOR;
		if (o instanceof TypeCast)
			return TYPE_CAST;
		
		return UNKNOWN;
	}
}
