/*
 * $Id$
 * 
 * Created on 30.09.2005
 *
 * Written by Tobias Sager (tsa) in 2005
 * 
 * See LICENSE for more information about licensing and warranties.
 */
package ch.toe.famix;

import java.util.Stack;

import javax.swing.tree.DefaultMutableTreeNode;

import ch.toe.famix.model.Attribute;
import ch.toe.famix.model.BehaviouralEntity;
import ch.toe.famix.model.FormalParameter;
import ch.toe.famix.model.GlobalVariable;
import ch.toe.famix.model.ImplicitVariable;
import ch.toe.famix.model.Invocation;
import ch.toe.famix.model.LocalVariable;
import ch.toe.famix.model.Model;

public class TreeBuildVisitor implements Visitor {

	/**
	 * Stack used while tree visiting
	 */
	private Stack<DefaultMutableTreeNode> stack = new Stack<DefaultMutableTreeNode>();

	/**
	 * The root object of the tree
	 */
	private DefaultMutableTreeNode root = null;

	/* (non-Javadoc)
	 * @see ch.toe.famix.Visitor#visit(java.lang.Object)
	 */
	public void visit(Object o) {
		if (isTreeRelevantElement(o))
			preVisit(o);
	}
	
	/* (non-Javadoc)
	 * @see ch.toe.famix.Visitor#endVisit(java.lang.Object)
	 */
	public void endVisit(Object o) {
		if (isTreeRelevantElement(o))
			postVisit();
	}

	/**
	 * Checks if the given Object is a desired object in the tree graph
	 * 
	 * @param o
	 * @return true if passed Object is one of the elements to visit, false otherwise
	 */
	private boolean isTreeRelevantElement(Object o) {
		if (
				o instanceof FAMIXInstance ||
				o instanceof Model ||
				o instanceof ch.toe.famix.model.Package ||
				o instanceof ch.toe.famix.model.Class ||
				o instanceof GlobalVariable ||
				o instanceof Attribute ||
				o instanceof BehaviouralEntity ||
				o instanceof FormalParameter ||
				o instanceof ImplicitVariable ||
				o instanceof LocalVariable ||
				o instanceof Invocation
			)
			return true;
		return false;
	}
	
	private void preVisit(Object o) {
		DefaultMutableTreeNode node = new DefaultMutableTreeNode(o); 
		if (root == null)
			root = node; 
		stack.push(node);
	}
	
	private void postVisit() {
		// child is the current visited object
		DefaultMutableTreeNode child = stack.pop();
		if (!stack.isEmpty()) {
			DefaultMutableTreeNode node = stack.pop();
			node.add(child);
			stack.push(node);
		}
		else
			stack.push(child);
	}

	/**
	 * @return Returns the root.
	 */
	public DefaultMutableTreeNode getRoot() {
		return root;
	}
}
