/*
 * $Id$
 * 
 * Created on 27.07.2005
 *
 * Written by Tobias Sager (tsa) in 2005
 * 
 * See LICENSE for more information about licensing and warranties.
 */
package ch.toe.famix.model;


/**
 * See FAMIX specification for more information about this object.
 * 
 * @author tsa
 */
public class Access extends Association {

	/**
	 * Mandatory FAMIX attribute
	 */
	private StructuralEntity accesses;

	/**
	 * Reference for unresolved accesses
	 */
	private String accessesString = null;
	
	/**
	 * Mandatory FAMIX attribute
	 */
	private BehaviouralEntity accessedIn;

	/**
	 * Optional FAMIX attribute
	 */
	private boolean isAccessLValue;
	
	/**
	 * @param accesses the object that is being accessed
	 * @param accessedIn the accessor
	 */
	public Access(StructuralEntity accesses, BehaviouralEntity accessedIn) {
		super();
		this.accesses = accesses;
		this.accessedIn = accessedIn;
	}
	
	/**
	 * @param uniqueName exact name of object being accessed
	 * @param accessedIn the accessor
	 */
	public Access(String uniqueName, BehaviouralEntity accessedIn) {
		super();
		this.accessesString = uniqueName;
		this.accessedIn = accessedIn;
	}

	/**
	 * Return my name
	 * 
	 * @return name of object
	 */
	public String toString() {
		return accessedIn.getUniqueName()+
				"->"+
				(hasUnresolvedAccess() ? accessesString : accesses.getUniqueName());
	}
	
	/**
	 * @return true if has unresolved access
	 */
	public boolean hasUnresolvedAccess() {
		if (accesses == null && accessesString != null)
			return true;
		return false;
	}

	/**
	 * @return Returns the isAccessLValue.
	 */
	public boolean isAccessLValue() {
		return isAccessLValue;
	}
	/**
	 * @param isAccessLValue The isAccessLValue to set.
	 */
	public void setAccessLValue(boolean isAccessLValue) {
		this.isAccessLValue = isAccessLValue;
	}
	/**
	 * @return Returns the accessedIn.
	 */
	public BehaviouralEntity getAccessedIn() {
		return accessedIn;
	}
	/**
	 * @return Returns the accesses.
	 */
	public StructuralEntity getAccesses() {
		return accesses;
	}

	/**
	 * @param accesses The accesses to set.
	 */
	public void setAccesses(StructuralEntity accesses) {
		this.accesses = accesses;
	}

	/**
	 * @return Returns the accessesString.
	 */
	public String getAccessesString() {
		return accessesString;
	}
}
