/*
 * $Id$
 * 
 * Created on 26.07.2005
 *
 * Written by Tobias Sager (tsa) in 2005
 * 
 * See LICENSE for more information about licensing and warranties.
 */
package ch.toe.famix.model;


/**
 * See FAMIX specification for more information about this object.
 * 
 * @author tsa
 */
public class Attribute extends StructuralEntity {

	/**
	 * Mandatory FAMIX attribute
	 */
	private Class belongsToClass;
	
	/**
	 * Optional FAMIX attribute
	 * valid contents: ["", "public", "protected", "private"]
	 * "" equals default visibility which denotes package visibility
	 */
	private String accessControlQualifier = "";

	/**
	 * Optional FAMIX attribute
	 * True if the attribute is static.
	 */
	private boolean hasClassScope = false;

	/**
	 * Optional FAMIX attribute
	 */
	private boolean isFinal = false;
	
	/**
	 * Optional FAMIX attribute
	 */
	private boolean isTransient = false;
	
	/**
	 * Optional FAMIX attribute
	 */
	private boolean isVolatile = false;
	
	/**
	 * @param name
	 */
	public Attribute(String name, Class belongsToClass) {
		super(name);
		this.belongsToClass = belongsToClass;
	}
		
	/**
	 * @return Returns the uniqueName. 
	 */
	public String getUniqueName() {
		return belongsToClass.getUniqueName() + "." + super.getName();
	}

	/**
	 * Return my name
	 * 
	 * @return name of object
	 */
	public String toString() {
		return getUniqueName();
	}
	
	/**
	 * @return true if instance is valid
	 */
	public boolean validate() {
		return super.validate() && (belongsToClass != null);
	}
	/**
	 * @return Returns the accessControlQualifier.
	 */
	public String getAccessControlQualifier() {
		return accessControlQualifier;
	}
	/**
	 * @param accessControlQualifier The accessControlQualifier to set.
	 */
	public void setAccessControlQualifier(String accessControlQualifier) {
		this.accessControlQualifier = accessControlQualifier;
	}
	/**
	 * @return Returns the hasClassScope.
	 */
	public boolean isHasClassScope() {
		return hasClassScope;
	}
	/**
	 * @param hasClassScope The hasClassScope to set.
	 */
	public void setHasClassScope(boolean hasClassScope) {
		this.hasClassScope = hasClassScope;
	}
	/**
	 * @return Returns the belongsToClass.
	 */
	public Class getBelongsToClass() {
		return belongsToClass;
	}
	/**
	 * @return Returns the isFinal.
	 */
	public boolean isFinal() {
		return isFinal;
	}
	/**
	 * @param isFinal The isFinal to set.
	 */
	public void setFinal(boolean isFinal) {
		this.isFinal = isFinal;
	}
	/**
	 * @return Returns the isTransient.
	 */
	public boolean isTransient() {
		return isTransient;
	}
	/**
	 * @param isTransient The isTransient to set.
	 */
	public void setTransient(boolean isTransient) {
		this.isTransient = isTransient;
	}
	/**
	 * @return Returns the isVolatile.
	 */
	public boolean isVolatile() {
		return isVolatile;
	}
	/**
	 * @param isVolatile The isVolatile to set.
	 */
	public void setVolatile(boolean isVolatile) {
		this.isVolatile = isVolatile;
	}
}
