/*
 * $Id$
 * 
 * Created on 26.07.2005
 *
 * Written by Tobias Sager (tsa) in 2005
 * 
 * See LICENSE for more information about licensing and warranties.
 */
package ch.toe.famix.model;


/**
 * See FAMIX specification for more information about this object.
 * 
 * @author tsa
 */
public class FormalParameter extends StructuralEntity {

	/**
	 * Mandatory FAMIX attribute
	 */
	private BehaviouralEntity belongsToBehaviour;

	/**
	 * Mandatory FAMIX attribute
	 */
	private Integer position;

	/**
	 * Optional FAMIX attribute
	 */
	private boolean isFinal = false;

	/**
	 * @param name
	 * @param belongsToBehaviour
	 * @param position
	 */
	public FormalParameter(String name, BehaviouralEntity belongsToBehaviour, Integer position) {
		super(name);
		this.belongsToBehaviour = belongsToBehaviour;
		this.position = position;
	}
	
	/**
	 * @return Returns the uniqueName. 
	 */
	public String getUniqueName() {
		if (belongsToBehaviour == null)
			return super.getName();
		else
			return belongsToBehaviour.getUniqueName() + "." + super.getName();
	}	

	/**
	 * Return my name
	 * 
	 * @return name of object
	 */
	public String toString() {
		return getUniqueName();
	}

	/**
	 * @return true if instance is valid
	 */
	public boolean validate() {
		// TODO this could check if position is a valid Argument attribute
		return super.validate();
	}
	
	/**
	 * @return Returns the position.
	 */
	public Integer getPosition() {
		return position;
	}
	
	/**
	 * @return Returns the belongsToBehaviour.
	 */
	public BehaviouralEntity getBelongsToBehaviour() {
		return belongsToBehaviour;
	}
	/**
	 * @return Returns the isFinal.
	 */
	public boolean isFinal() {
		return isFinal;
	}
	/**
	 * @param isFinal The isFinal to set.
	 */
	public void setFinal(boolean isFinal) {
		this.isFinal = isFinal;
	}
}
