/*
 * $Id$
 * 
 * Created on 27.07.2005
 *
 * Written by Tobias Sager (tsa) in 2005
 * 
 * See LICENSE for more information about licensing and warranties.
 */
package ch.toe.famix.model;

import java.util.Iterator;
import java.util.LinkedHashSet;
import java.util.Set;

import ch.toe.famix.Visitable;
import ch.toe.famix.Visitor;

/**
 * See FAMIX specification for more information about this object.
 * 
 * @author tsa
 */
public class Invocation extends Association implements Visitable {

	/**
	 * Mandatory FAMIX attribute
	 */
	private BehaviouralEntity invokedBy;
	
	/**
	 * Mandatory FAMIX attribute
	 */
	private BehaviouralEntity invokes;
	
	/**
	 * Replacement for non resolvable invokes reference.
	 */
	private String invokesString;

	/**
	 * Optional FAMIX attribute
	 */
	private String base;
	
	/**
	 * Optional FAMIX attribute
	 */
	private Set<String> candidates = new LinkedHashSet<String>();

	/**
	 * Optional FAMIX attribute
	 */
	private Set<Argument> arguments = new LinkedHashSet<Argument>();
	
	/**
	 * @param invokedBy the invoker BehaviouralEntity
	 * @param invokes reference to the invoked BehaviouralEntity
	 */
	public Invocation(BehaviouralEntity invokedBy, BehaviouralEntity invokes) {
		super();
		this.invokedBy = invokedBy;
		this.invokes = invokes;
	}

	/**
	 * @param invokedBy the invoker BehaviouralEntity
	 * @param invokesString reference to the invoked BehaviouralEntity
	 */
	public Invocation(BehaviouralEntity invokedBy, String invokesString) {
		super();
		this.invokedBy = invokedBy;
		this.invokesString = invokesString;
	}

	public void accept(Visitor v) {
		v.visit(this);
		Iterator<Argument> iterArgument = arguments.iterator();
		while (iterArgument.hasNext())
			(iterArgument.next()).accept(v);
		v.endVisit(this);
	}
	
	/**
	 * Return my name
	 * 
	 * @return name of object
	 */
	public String toString() {
		return invokedBy.getUniqueName()+
				"->"+
				(hasUnresolvedInvokes() ? invokesString : invokes.getUniqueName());
	}
	
	/**
	 * @return true if invocation is not resolvable
	 */
	public boolean hasUnresolvedInvokes() {
		if (invokes == null && invokesString != null)
			return true;
		return false;
	}
	
	/**
	 * @return Returns the base.
	 */
	public String getBase() {
		return base;
	}
	/**
	 * @return Returns the invokedBy.
	 */
	public BehaviouralEntity getInvokedBy() {
		return invokedBy;
	}
	/**
	 * @return Returns the invokes.
	 */
	public BehaviouralEntity getInvokes() {
		return invokes;
	}	

	/**
	 * @return Returns the arguments.
	 */
	public Set<Argument> getArguments() {
		return arguments;
	}

	/**
	 * @param arguments The arguments to set.
	 */
	public void setArguments(Set<Argument> arguments) {
		this.arguments = arguments;
	}

	/**
	 * @return Returns the candidates.
	 */
	public Set<String> getCandidates() {
		return candidates;
	}

	/**
	 * @param candidates The candidates to set.
	 */
	public void setCandidates(Set<String> candidates) {
		this.candidates = candidates;
	}

	/**
	 * @param invokes The invokes to set.
	 */
	public void setInvokes(BehaviouralEntity invokes) {
		this.invokes = invokes;
	}

	/**
	 * @return Returns the invokesString.
	 */
	public String getInvokesString() {
		return invokesString;
	}
}
