/*
 * $Id$
 * 
 * Created on 22.07.2005
 *
 * Written by Tobias Sager (tsa) in 2005
 * 
 * See LICENSE for more information about licensing and warranties.
 */
package ch.toe.famix.model;

import ch.toe.famix.Visitor;

/**
 * See FAMIX specification for more information about this object.
 * 
 * @author tsa
 */
public class Method extends BehaviouralEntity {

	/**
	 * Mandatory FAMIX attribute
	 */
	private Class belongsToClass;
	
	/**
	 * Optional FAMIX attribute
	 * This should be true if Method is static.
	 */
	private boolean hasClassScope = false;

	/**
	 * Optional FAMIX attribute
	 */
	private boolean isAbstract = false;

	/**
	 * Optional FAMIX attribute
	 */
	private boolean isConstructor = false;

	/**
	 * Optional FAMIX attribute
	 */
	private boolean isFinal = false;
	
	/**
	 * Optional FAMIX attribute
	 */
	private boolean isSynchronized = false;
	
	/**
	 * Optional FAMIX attribute
	 */
	private boolean isNative = false;
	
	/**
	 * @param name			name of the method
	 * @param signature		in the format "method(param1,param2)"
	 * @param belongsToClass class this method belongs to
	 */
	public Method(String name, String signature, Class belongsToClass) {
		super(name, signature);
		this.belongsToClass = belongsToClass;
	}
	
	/**
	 * @return Returns the uniqueName. 
	 */
	public String getUniqueName() {
		if (belongsToClass == null) {
			return super.getSignature();
		}
		else
			return belongsToClass.getUniqueName() + "." + super.getSignature();
	}

	/**
	 * Return my name
	 * 
	 * @return name of object
	 */
	public String toString() {
		return getUniqueName();
	}

	@Override
	public void accept(Visitor v) {
		super.accept(v);
	}
	
	/**
	 * @return Returns the hasClassScope.
	 */
	public boolean isHasClassScope() {
		return hasClassScope;
	}
	/**
	 * @param hasClassScope The hasClassScope to set.
	 */
	public void setHasClassScope(boolean hasClassScope) {
		this.hasClassScope = hasClassScope;
	}
	/**
	 * @return Returns the isAbstract.
	 */
	public boolean isAbstract() {
		return isAbstract;
	}
	/**
	 * @param isAbstract The isAbstract to set.
	 */
	public void setAbstract(boolean isAbstract) {
		this.isAbstract = isAbstract;
	}
	/**
	 * @return Returns the isConstructor.
	 */
	public boolean isConstructor() {
		return isConstructor;
	}
	/**
	 * @param isConstructor The isConstructor to set.
	 */
	public void setConstructor(boolean isConstructor) {
		this.isConstructor = isConstructor;
	}
	/**
	 * @return true if instance is valid
	 */
	public boolean validate() {
		return super.validate() && (belongsToClass != null);
	}
	/**
	 * @return Returns the isFinal.
	 */
	public boolean isFinal() {
		return isFinal;
	}
	/**
	 * @param isFinal The isFinal to set.
	 */
	public void setFinal(boolean isFinal) {
		this.isFinal = isFinal;
	}
	/**
	 * @return Returns the isNative.
	 */
	public boolean isNative() {
		return isNative;
	}
	/**
	 * @param isNative The isNative to set.
	 */
	public void setNative(boolean isNative) {
		this.isNative = isNative;
	}
	/**
	 * @return Returns the isSynchronized.
	 */
	public boolean isSynchronized() {
		return isSynchronized;
	}
	/**
	 * @param isSynchronized The isSynchronized to set.
	 */
	public void setSynchronized(boolean isSynchronized) {
		this.isSynchronized = isSynchronized;
	}
	/**
	 * @return Returns the belongsToClass.
	 */
	public Class getBelongsToClass() {
		return belongsToClass;
	}
}
