/*
 * $Id$
 * 
 * Created on 20.07.2005
 *
 * Written by Tobias Sager (tsa) in 2005
 * 
 * See LICENSE for more information about licensing and warranties.
 */
package ch.toe.famix.model;

import java.text.DateFormat;
import java.util.Date;

/**
 * See FAMIX specification for more information about this object.
 * 
 * @author tsa
 */
public class Model extends ch.toe.famix.model.Object {

	/**
	 * Class defaults
	 */
	private static final String DEFAULT_EXPORTER_NAME = "AST2FAMIX";
	private static final String DEFAULT_EXPORTER_VERSION = "1.0";
	private static final String DEFAULT_PUBLISHER_NAME = "";
	private static final String DEFAULT_EXTRACTION_LEVEL = "4";
	private static final String DEFAULT_SOURCE_LANGUAGE = "Java";
	
	/**
	 * Mandatory FAMIX attribute
	 */
	private final String exporterName = DEFAULT_EXPORTER_NAME;
	
	/**
	 * Mandatory FAMIX attribute
	 */
	private final String exporterVersion = DEFAULT_EXPORTER_VERSION;
	
	/**
	 * Mandatory FAMIX attribute
	 */
	private String exporterDate;
	
	/**
	 * Mandatory FAMIX attribute
	 */
	private String exporterTime;

	/**
	 * Mandatory FAMIX attribute
	 */
	private String publisherName = DEFAULT_PUBLISHER_NAME;

	/**
	 * Optional FAMIX attribute
	 */
	private String parsedSystemName;

	/**
	 * Mandatory FAMIX attribute
	 */
	private String extractionLevel = DEFAULT_EXTRACTION_LEVEL;
	
	/**
	 * Mandatory FAMIX attribute
	 */
	private final String sourceLanguage = DEFAULT_SOURCE_LANGUAGE;

	/**
	 * Optional FAMIX attribute
	 */
	private String sourceDialect;

	/**
	 * Default constructor
	 */
	public Model() {
		super();
		initializeDate();
	}

	/**
	 * @param publisherName
	 * @param parsedSystemName
	 * @param extractionLevel
	 * @param sourceDialect
	 */
	public Model(String publisherName, String parsedSystemName, String extractionLevel, String sourceDialect) {
		super();
		initializeDate();
		if (publisherName != null)
			this.publisherName = publisherName;
		this.parsedSystemName = parsedSystemName;
		if (extractionLevel != null && extractionLevel != "")
			this.extractionLevel = extractionLevel;
		this.sourceDialect = sourceDialect;
	}

	/**
	 * Initializes exporterDate and exporterTime
	 */
	private void initializeDate() {
		Date currentDate = new Date();
		DateFormat df = DateFormat.getDateInstance(DateFormat.SHORT);
		exporterDate = df.format(currentDate);
		df = DateFormat.getTimeInstance(DateFormat.SHORT);
		exporterTime = df.format(currentDate);		
	}

	/**
	 * Return my name
	 * 
	 * @return name of object
	 */
	public String toString() {
		return "Model";
	}
	
	/* (non-Javadoc)
	 * @see ch.toe.famix.model.Object#validate()
	 */
	public boolean validate() {
		// we should be allright
		return true;
	}

	/**
	 * @return Returns the extractionLevel.
	 */
	public String getExtractionLevel() {
		return extractionLevel;
	}
	/**
	 * @param extractionLevel The extractionLevel to set.
	 */
	public void setExtractionLevel(String extractionLevel) {
		this.extractionLevel = extractionLevel;
	}
	/**
	 * @return Returns the parsedSystemName.
	 */
	public String getParsedSystemName() {
		return parsedSystemName;
	}
	/**
	 * @param parsedSystemName The parsedSystemName to set.
	 */
	public void setParsedSystemName(String parsedSystemName) {
		this.parsedSystemName = parsedSystemName;
	}
	/**
	 * @return Returns the publisherName.
	 */
	public String getPublisherName() {
		return publisherName;
	}
	/**
	 * @param publisherName The publisherName to set.
	 */
	public void setPublisherName(String publisherName) {
		this.publisherName = publisherName;
	}
	/**
	 * @return Returns the sourceLanguage.
	 */
	public String getSourceLanguage() {
		return sourceLanguage;
	}
	/**
	 * @return Returns the sourceDialect.
	 */
	public String getSourceDialect() {
		return sourceDialect;
	}
	/**
	 * @param sourceDialect The sourceDialect to set.
	 */
	public void setSourceDialect(String sourceDialect) {
		this.sourceDialect = sourceDialect;
	}
	/**
	 * @return Returns the exporterDate.
	 */
	public String getExporterDate() {
		return exporterDate;
	}
	/**
	 * @return Returns the exporterName.
	 */
	public String getExporterName() {
		return exporterName;
	}
	/**
	 * @return Returns the exporterTime.
	 */
	public String getExporterTime() {
		return exporterTime;
	}
	/**
	 * @return Returns the exporterVersion.
	 */
	public String getExporterVersion() {
		return exporterVersion;
	}
}
