/*
 * $Id: Object.java 590 2005-11-13 23:12:52Z tsa $
 * 
 * Created on 20.07.2005
 *
 * Written by Tobias Sager (tsa) in 2005
 * 
 * See LICENSE for more information about licensing and warranties.
 */
package ch.toe.famix.model;

import java.util.LinkedHashSet;
import java.util.Set;

import ch.toe.famix.Visitable;
import ch.toe.famix.Visitor;

/**
 * See FAMIX specification for more information about this object.
 * 
 * @author tsa
 */
public abstract class Object implements Visitable {

	/**
	 * This is used for enumerating all the elements in the instance 
	 */
	private static int instanceElementCount = 0;
	
	/**
	 * Optional FAMIX attribute
	 */
	private SourceAnchor sourceAnchor; 
	
	/**
	 * Optional FAMIX attribute
	 */
	private Set<String> comments = new LinkedHashSet<String>(); 

	/**
	 * Property container
	 * Containing objects of type Property
	 * 
	 * @see Property
	 */
	private Set<String> properties = new LinkedHashSet<String>();
	
	/**
	 * This is the element's number in the instance
	 */
	protected int elementNumber = ++instanceElementCount;
	
	/**
	 * Constructor
	 */
	public Object() {
		super();
	}
	
	/**
	 * Return my name
	 * 
	 * @return name of object
	 */
	public String toString() {
		return "Object";
	}

	/**
	 * @return Returns the sourceAnchor.
	 */
	public SourceAnchor getSourceAnchor() {
		return sourceAnchor;
	}
	/**
	 * @param sourceAnchor The sourceAnchor to set.
	 */
	public void setSourceAnchor(SourceAnchor sourceAnchor) {
		this.sourceAnchor = sourceAnchor;
	}
	
	/**
	 * This method is to be overriden by subclasses. It should check if mandatory
	 * attributes are set and compliant.
	 *  
	 * @return true on successful validation
	 */
	public abstract boolean validate();

	/**
	 * Default accept implementation for Visitor pattern
	 */
	public void accept(Visitor v) {
		v.visit(this);
		v.endVisit(this);
	}

	/**
	 * @return Returns the comments.
	 */
	public Set<String> getComments() {
		return comments;
	}

	/**
	 * @param comments The comments to set.
	 */
	public void setComments(Set<String> comments) {
		this.comments = comments;
	}

	/**
	 * @return Returns the properties.
	 */
	public Set<String> getProperties() {
		return properties;
	}

	/**
	 * @param properties The properties to set.
	 */
	public void setProperties(Set<String> properties) {
		this.properties = properties;
	}
}
