/*
 * $Id: SourceAnchor.java 303 2005-10-07 16:52:59Z tsa $
 * 
 * Created on 20.07.2005
 *
 * Written by Tobias Sager (tsa) in 2005
 * 
 * See LICENSE for more information about licensing and warranties.
 */
package ch.toe.famix.model;

/**
 * See FAMIX specification for more information about this object.
 * 
 * @author tsa
 */
public final class SourceAnchor {

	private String file;
	
	private Integer start;
	
	private Integer end;
	
	/**
	 * Default constructor for Qualifiers.
	 * The FAMIX format for a qualifier is like this:
	 *   file "FILE" start START end END
	 * Where FILE denotes the source file and START/END mark the position for the anchor.
	 * 
	 * @param file		The source file in which the anchor is valid.
	 * @param start		Start position of anchor.
	 * @param end		End position of anchor.
	 */
	public SourceAnchor(String file, Integer start, Integer end) {
		this.setFile(file);
		this.setStart(start);
		this.setEnd(end);
	}

	/**
	 * Return my name
	 * 
	 * @return name of object
	 */
	public String toString() {
		return "SourceAnchor";
	}

	/**
	 * This validates the submitted data. Will check following things:
	 *  - all fields must nut be NULL
	 *  - start >= end
	 * 
	 * @return true upon valid instance
	 */
	public boolean validate() {
		// all fields must not be NULL
		if ((start != null) && (end != null) && (file != null)) {
			// start < end 
			if (start.compareTo(end) < 0)
				return false;
			// in all other cases
			return true;
		}
		else
			return false;
	}

	/**
	 * @return Returns the file.
	 */
	public String getFile() {
		return file;
	}
	/**
	 * @param file The file to set.
	 */
	public void setFile(String file) {
		this.file = file;
	}
	
	/**
	 * @return Returns the end.
	 */
	public Integer getEnd() {
		return end;
	}
	/**
	 * @param end The end to set.
	 */
	public void setEnd(Integer end) {
		this.end = end;
	}
	/**
	 * @return Returns the start.
	 */
	public Integer getStart() {
		return start;
	}
	/**
	 * @param start The start to set.
	 */
	public void setStart(Integer start) {
		this.start = start;
	}
}
