/*
 * $Id$
 * 
 * Created on 24.08.2005
 *
 * Written by Tobias Sager (tsa) in 2005
 * 
 * See LICENSE for more information about licensing and warranties.
 */
package ch.toe.famix.model.container;

import java.util.Iterator;
import java.util.LinkedHashSet;
import java.util.Set;

import org.apache.log4j.Logger;

import ch.toe.famix.model.Method;

/**
 * This defines a container for Methods.
 * 
 * @author tsa
 */
public class MethodContainer {

	// logging device
	private static Logger logger = Logger.getLogger(MethodContainer.class);

	/**
	 * Container; child elements are of type Class
	 */
	private Set<Method> container = new LinkedHashSet<Method>();
	
	/**
	 * Contains claims for not yet existing classes
	 */
	private Set<Claim> claims = new LinkedHashSet<Claim>();
	
	/**
	 * Finds the Method with the specified name in the container.	
	 * 
	 * @param uniqueName
	 * @return the Method looking for or null if no such Method was found
	 */
	public Method findByName(String uniqueName) {
		Method ret = null;
		Iterator iter = this.getContainer().iterator();
		while (iter.hasNext()) {
			Method mymethod = (Method)iter.next();
			if (mymethod.getUniqueName().equals(uniqueName)) {
				ret = mymethod;
				break;
			}
		}						
		return ret;
	}

	/**
	 * Finds all claimers that claim the given Method resource. It compares uniqueName.
	 * 
	 * @param mymethod
	 * @return an iterator containing Claim objects that matched the Class name
	 */
	private Iterator findClaimers(Method mymethod) {
		Set<Claim> claims = new LinkedHashSet<Claim>();
		Iterator iter = this.getClaims().iterator();
		while (iter.hasNext()) {
			Claim claim = (Claim)iter.next();
			// check uniqueName and name
			if (claim.getClaimed().equals(mymethod.getUniqueName())) 
				claims.add(claim);
		}
		return claims.iterator();
	}
	
	/**
	 * Adds a method to the collection. Checks for possible claims and adds the Method as
	 * reference to those.
	 * 
	 * @param mymethod
	 */
	public boolean add(Method mymethod) {
		if (container.add(mymethod)) {
			// find possible claims of mymethod 
			// process all existing claimers and add mymethod as reference if requested
			Iterator iter = findClaimers(mymethod);
			while (iter.hasNext()) {
				logger.debug("Looking for claimers of "+mymethod.getUniqueName());
				Claim claim = (Claim)iter.next();
				// remove the claim from my claim container if I can add myclass to the claimer
				if (Claim.fulfilClaim(claim, mymethod))
					this.getClaims().remove(claim);
			}
			return true;
		}			
		return false;
	}
	
	/**
	 * @return Returns the container.
	 */
	public Set<Method> getContainer() {
		return container;
	}

	/**
	 * @return Returns the claims.
	 */
	public Set<Claim> getClaims() {
		return claims;
	}
}
