/*
 * $Id$
 * 
 * Created on 15.08.2005
 *
 * Written by Tobias Sager (tsa) in 2005
 * 
 * See LICENSE for more information about licensing and warranties.
 */
package ch.toe.famix.model.container;

import java.util.Iterator;
import java.util.LinkedHashSet;
import java.util.Set;

import ch.toe.famix.model.Package;

/**
 * This defines a container for Packages.
 * 
 * @author tsa
 */
public class PackageContainer {
	
	/**
	 * Container; child elements are of type Package
	 */
	private Set<Package> container = new LinkedHashSet<Package>();

	/**
	 * Finds the Package with the specified name in the container. Checks uniqueName and simple name.	
	 * 
	 * @param name
	 * @return the Package looking for or null if no such Package was found
	 */
	public Package findByName(String name) {
		Package ret = null;
		// if name contains dots, it's a uniqueName
		boolean simpleName = !(name.lastIndexOf('.') > -1);
		Iterator iter = this.getContainer().iterator();
		while (iter.hasNext()) {
			Package mypackage = (Package)iter.next();
			if (!simpleName) {
				if (mypackage.getUniqueName().equals(name)) {
					ret = mypackage;
					break;
				}				
			}
			else {
				if (mypackage.getName().equals(name)) {
					ret = mypackage;
					break;
				}				
			}
		}						
		return ret;
	}

	/**
	 * Adds a package to the collection.
	 * 
	 * @param mypackage
	 * @return true on success, false if Package was already in collection 
	 */
	public boolean add(Package mypackage) {
		if (container.add(mypackage)) {
			return true;
		}			
		return false;
	}
	
	/**
	 * @return Returns the container.
	 */
	public Set<Package> getContainer() {
		return container;
	}
}
