package ch.unibern.graphmatching2;

import java.util.*;

public class Graph {
	static int DEAD = 0;

	static int ALIVE = 1;

	static int ACTIVE = 2;

	static int DORMANT = 3;

	private Vector m_vertexList;

	private Vector m_depthFirstTraversal;

	private int m_state;

	private boolean m_isModel; // True if this graph is in the model set

	// A graph in the decomposition can be mapped to one or more subgraphs
	// of the target graph. We store these maps in another Vector.
	private Vector m_targetMapList;

	public Graph() {
		m_vertexList = new Vector(10, 10);
		m_depthFirstTraversal = new Vector(10, 10);
		m_targetMapList = new Vector(5, 5);
		m_state = DORMANT;
		m_isModel = false;
	}

	public void setIsModel(boolean isModel) {
		m_isModel = isModel;
	}

	public boolean isModel() {
		return m_isModel;
	}

	public int getNumberOfVertices() {
		return m_vertexList.size();
	}

	public Vector getVertexList() {
		return m_vertexList;
	}

	public void addVertex(Vertex newVertex) {
		m_vertexList.add(newVertex);
	}

	public HashMap getFirstTargetMap() {
		return (HashMap) m_targetMapList.firstElement();
	}

	public Vector getTargetMap() {
		return m_targetMapList;
	}

	public void addTargetMap(HashMap newMap) {
		m_targetMapList.add(newMap);
	}

	public String printTargetMap() {
		Enumeration targetList = m_targetMapList.elements();
		String result = "\n********* Target Map " + m_targetMapList.size()
				+ " elements\n";
		int mapNum = 1;
		while (targetList.hasMoreElements()) {
			result += "==========Map #" + mapNum++;
			HashMap map = (HashMap) targetList.nextElement();
			Set mapSet = map.entrySet();
			Iterator mapList = mapSet.iterator();
			while (mapList.hasNext()) {
				Map.Entry mapEntry = (Map.Entry) mapList.next();
				Vertex source = (Vertex) mapEntry.getKey();
				Vertex target = (Vertex) mapEntry.getValue();
				result += source.getLabel().toString() + " --> "
						+ target.getLabel().toString() + "\n";
			}
		}
		return result;
	}

	public String printVertexList() {
		Enumeration vertexList = m_vertexList.elements();
		String printList = "";
		while (vertexList.hasMoreElements()) {
			Vertex vertex = (Vertex) vertexList.nextElement();
			printList += vertex.getLabel().toString();
			if (vertexList.hasMoreElements()) {
				printList += ", ";
			}
		}
		return printList;
	}

	public String printDFT() {
		Enumeration DFTList = m_depthFirstTraversal.elements();
		String printList = "";
		while (DFTList.hasMoreElements()) {
			Vertex vertex = (Vertex) DFTList.nextElement();
			printList += vertex.getLabel().toString();
			if (DFTList.hasMoreElements()) {
				printList += ", ";
			}
		}
		return printList;
	}

	public Vertex getVertexByLabel(Object vertexLabel) {
		Enumeration vertexList = m_vertexList.elements();
		while (vertexList.hasMoreElements()) {
			Vertex vertex = (Vertex) vertexList.nextElement();
			if (vertex.getLabel().equals(vertexLabel)) {
				return vertex;
			}
		}
		return null;
	}

	public void addRandomEdges(double edgeProb) {
		Enumeration outerList = m_vertexList.elements();
		while (outerList.hasMoreElements()) {
			Vertex sourceVertex = (Vertex) outerList.nextElement();
			Enumeration innerList = m_vertexList.elements();
			while (innerList.hasMoreElements()) {
				Vertex destVertex = (Vertex) innerList.nextElement();
				if (sourceVertex != destVertex && Math.random() < edgeProb) {
					sourceVertex.addEdge(destVertex);
				}
			}
		}
	}

	public String printEdgeTable() {
		String edgeTable = "";
		Enumeration outerList = m_vertexList.elements();
		while (outerList.hasMoreElements()) {
			Vertex sourceVertex = (Vertex) outerList.nextElement();
			edgeTable += "(" + m_vertexList.indexOf(sourceVertex) + ") "
					+ sourceVertex.getLabel().toString() + ": ";
			edgeTable += sourceVertex.printEdgeList(m_vertexList);
			edgeTable += "\n";
		}
		return edgeTable;
	}

	private void recursiveDFT(Vertex start) {
		if (start.getVisited())
			return;
		m_depthFirstTraversal.add(start);
		start.setVisited(true);
		Enumeration edgeList = start.getEdgeList().elements();
		while (edgeList.hasMoreElements()) {
			Vertex nextVertex = ((Edge) edgeList.nextElement()).getDestVertex();
			recursiveDFT(nextVertex);
		}
	}

	public void depthFirstTraversal() {
		Enumeration vertexList = m_vertexList.elements();
		while (vertexList.hasMoreElements()) {
			((Vertex) vertexList.nextElement()).setVisited(false);
		}
		vertexList = m_vertexList.elements();
		while (vertexList.hasMoreElements()) {
			recursiveDFT((Vertex) vertexList.nextElement());
		}
	}

	public int getState() {
		return m_state;
	}

	public void setState(int state) {
		m_state = state;
	}
}
