package ch.unibern.graphmatching2;

import java.io.*;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;

public class TestMatch extends JFrame implements ActionListener, Runnable {
	public static int WIDTH = 450;

	public static int HEIGHT = 600;

	public static String TITLE = "Testing Graph Matching";

	public static int NUM_VERTEX = 10;

	public static int LABEL_RANGE = 10;

	JButton fileButton, m_testGraphButton;

	JTextArea m_sourceTextArea;

	JComboBox m_numModelsList, m_numTargetsList, m_probEdgesList,
			m_numVerticesList, m_numLabelsList;

	JRadioButton m_randomModel, m_constantModel;

	ButtonGroup m_modelSizeGroup;

	File lastDirectory;

	JFileChooser fileChoice;

	JProgressBar m_progressBar;

	JLabel m_progressLabel;

	JWindow m_progressWindow;

	Graph[] m_modelGraph;

	int m_numModels, m_numTargets, m_numVertices, m_numLabels;

	boolean m_constModelSize;

	double m_probEdges;

	Thread m_progressThread;

	public TestMatch() {
		super(TITLE);
		addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) {
				System.exit(0);
			}
		});
		setupGUI();
		setSize(WIDTH, HEIGHT);
		show();
	}

	Graph messmerTarget() {
		Graph target = new Graph();
		Vertex[] v = new Vertex[6];
		v[0] = new Vertex("d");
		v[1] = new Vertex("a");
		v[2] = new Vertex("d");
		v[3] = new Vertex("e");
		v[4] = new Vertex("c");
		v[5] = new Vertex("a");
		for (int vert = 0; vert < 6; vert++) {
			target.addVertex(v[vert]);
		}
		v[0].addEdge(v[1]);
		v[1].addEdge(v[2]);
		v[1].addEdge(v[4]);
		v[2].addEdge(v[3]);
		v[4].addEdge(v[3]);
		v[4].addEdge(v[5]);
		return target;
	}

	Graph messmerModel1() {
		Graph model1 = new Graph();
		Vertex[] v = new Vertex[5];
		v[0] = new Vertex("d");
		v[1] = new Vertex("a");
		v[2] = new Vertex("b");
		v[3] = new Vertex("c");
		v[4] = new Vertex("a");

		for (int vert = 0; vert < 5; vert++) {
			model1.addVertex(v[vert]);
		}
		v[0].addEdge(v[1]);
		v[1].addEdge(v[2]);
		v[2].addEdge(v[3]);
		v[1].addEdge(v[3]);
		return model1;
	}

	Graph messmerModel2() {
		Graph model2 = new Graph();
		Vertex[] v = new Vertex[4];
		v[0] = new Vertex("d");
		v[1] = new Vertex("a");
		v[2] = new Vertex("c");
		v[3] = new Vertex("e");

		for (int vert = 0; vert < 4; vert++) {
			model2.addVertex(v[vert]);
		}
		v[0].addEdge(v[1]);
		v[1].addEdge(v[2]);
		v[2].addEdge(v[3]);
		return model2;
	}

	Graph fixedGraph(int numVertex) {
		Graph randGraph = new Graph();
		for (int vert = 0; vert < numVertex; vert++) {
			String labelString = "#" + vert;
			Vertex vertex = new Vertex(labelString);
			randGraph.addVertex(vertex);
		}
		randGraph.addRandomEdges(0.3);
		return randGraph;
	}

	Graph randomGraph(int numVertex, int labelRange, double edgeProb) {
		Graph randGraph = new Graph();
		byte[] vertLabel = new byte[1];
		for (int vert = 0; vert < numVertex; vert++) {
			vertLabel[0] = (byte) ((Math.random() * labelRange) + 97);
			String labelString = new String(vertLabel);
			Vertex vertex = new Vertex(labelString);
			randGraph.addVertex(vertex);
		}
		randGraph.addRandomEdges(edgeProb);
		return randGraph;
	}

	void setupGraphs() {
		String numModelsString = (String) m_numModelsList.getSelectedItem();
		m_numModels = Integer.parseInt(numModelsString);
		m_modelGraph = new Graph[m_numModels];

		String numTargetsString = (String) m_numTargetsList.getSelectedItem();
		m_numTargets = Integer.parseInt(numTargetsString);

		String numVerticesString = (String) m_numVerticesList.getSelectedItem();
		m_numVertices = Integer.parseInt(numVerticesString);

		String probEdgesString = (String) m_probEdgesList.getSelectedItem();
		m_probEdges = Double.parseDouble(probEdgesString);

		String numLabelsString = (String) m_numLabelsList.getSelectedItem();
		m_numLabels = Integer.parseInt(numLabelsString);

		int numVertices = m_numVertices;
		for (int elem = 0; elem < m_numModels; elem++) {
			if (!m_constModelSize) {
				numVertices = (int) (Math.random() * m_numVertices + 1);
			}
			m_modelGraph[elem] = randomGraph(numVertices, m_numLabels,
					m_probEdges);
			m_modelGraph[elem].setIsModel(true);
		}
	}

	void setupProgressWindow() {
		m_progressWindow = new JWindow(this);
		m_progressWindow.setSize(200, 70);
		m_progressWindow.getContentPane().setLayout(new GridLayout(2, 1));
		m_progressWindow.setBackground(Color.yellow);
		m_progressLabel = new JLabel("Progress");
		m_progressLabel.setForeground(Color.black);
		m_progressWindow.getContentPane().add(m_progressLabel);
		m_progressBar = new JProgressBar();
		m_progressBar.setStringPainted(true);
		m_progressBar.setString("Progress");
		m_progressBar.setForeground(Color.red);
		m_progressBar.setBackground(Color.black);
		m_progressWindow.getContentPane().add(m_progressBar);
	}

	public void run() {
		setupGraphs();

		m_progressLabel.setText("Decomposing " + m_numModels + " models.");
		m_progressBar.setMinimum(0);
		m_progressBar.setMaximum(m_numModels);
		m_progressBar.setValue(0);
		m_progressBar.setString("Generating models...");

		Decomposition decomp = new Decomposition();
		String text = "Model graphs:\n";
		for (int model = 0; model < m_numModels; model++) {
			text += m_modelGraph[model].printEdgeTable() + "\n++++++++\n";
			decomp.decompose(m_modelGraph[model]);
			m_progressBar.setValue(model + 1);
			m_progressBar.setString("Model " + (model + 1));
		}
		m_progressLabel.setText("Matching " + m_numTargets + " targets.");
		m_progressBar.setMaximum(m_numTargets);
		m_progressBar.setValue(0);
		m_progressBar.setString("Starting matching...");
		for (int target = 0; target < m_numTargets; target++) {
			Graph targetGraph = randomGraph(m_numVertices, m_numLabels,
					m_probEdges);
			text += "\n======= Target graph # " + target + "\n"
					+ targetGraph.printEdgeTable();
			Graph largestSubgraph = decomp.matchTarget(targetGraph);

			if (largestSubgraph == null) {
				text += " --------- No subgraph found. ---------\n";
			} else {
				text += "--------- Largest subgraph: "
						+ largestSubgraph.getNumberOfVertices()
						+ " vertices -----------\n"
						+ largestSubgraph.printEdgeTable();
				if (largestSubgraph.isModel()) {
					text += "Subgraph is in the model set.";
				} else {
					text += "Subgraph is not in the model set.";
				}
			}
			m_progressBar.setValue(target + 1);
			m_progressBar.setString("Target " + (target + 1));
		}
		m_sourceTextArea.setText(text);
		m_progressWindow.hide();
		m_testGraphButton.setEnabled(true);
	}

	void setupGUI() {
		setupProgressWindow();

		fileButton = new JButton("Open file");

		// Initialize the directory for the file dialog
		File currentDir = new File(".");
		lastDirectory = new File(currentDir.getAbsolutePath());

		// Add extension filters for the file dialog
		ExtensionFileFilter extFilter = new ExtensionFileFilter();
		extFilter.addExtension("c");
		extFilter.addExtension("h");
		extFilter.addExtension("cc");
		fileChoice = new JFileChooser();
		fileChoice.setFileFilter(extFilter);

		String[] numModelsString = { "1", "2", "4", "6", "8", "10", "15", "20",
				"25", "30", "40", "50", "60", "70", "80", "90", "100" };
		m_numModelsList = new JComboBox(numModelsString);
		m_numModelsList.setEditable(true);
		m_numModelsList.setSelectedIndex(5);
		Panel numModelsPanel = new Panel(new GridLayout(2, 1));
		numModelsPanel.add(new JLabel("Num models:"));
		numModelsPanel.add(m_numModelsList);

		m_randomModel = new JRadioButton("Random");
		m_randomModel.addActionListener(this);
		m_constantModel = new JRadioButton("Same as target", true);
		m_constModelSize = true;
		m_constantModel.addActionListener(this);
		m_modelSizeGroup = new ButtonGroup();
		m_modelSizeGroup.add(m_randomModel);
		m_modelSizeGroup.add(m_constantModel);
		Panel modelSizePanel = new Panel(new GridLayout(3, 1));
		modelSizePanel.add(new JLabel("Model sizes:"));
		modelSizePanel.add(m_randomModel);
		modelSizePanel.add(m_constantModel);

		String[] numTargetsString = { "1", "2", "4", "6", "8", "10", "15",
				"20", "25", "30", "40", "50", "60", "70", "80", "90", "100" };
		m_numTargetsList = new JComboBox(numTargetsString);
		m_numTargetsList.setEditable(true);
		Panel numTargetsPanel = new Panel(new GridLayout(2, 1));
		numTargetsPanel.add(new JLabel("Num targets:"));
		numTargetsPanel.add(m_numTargetsList);

		String[] numVerticesString = { "2", "4", "6", "8", "10", "15", "20",
				"25", "30", "40", "50", "60", "70", "80", "90", "100" };
		m_numVerticesList = new JComboBox(numVerticesString);
		m_numVerticesList.setEditable(true);
		m_numVerticesList.setSelectedIndex(6);
		Panel numVerticesPanel = new Panel(new GridLayout(2, 1));
		numVerticesPanel.add(new JLabel("Target size:"));
		numVerticesPanel.add(m_numVerticesList);

		String[] probEdgesString = { "0.0", "0.1", "0.2", "0.3", "0.4", "0.5",
				"0.6", "0.7", "0.8", "0.9", "1.0" };
		m_probEdgesList = new JComboBox(probEdgesString);
		m_probEdgesList.setEditable(true);
		m_probEdgesList.setSelectedIndex(5);
		Panel probEdgesPanel = new Panel(new GridLayout(2, 1));
		probEdgesPanel.add(new JLabel("Edge prob:"));
		probEdgesPanel.add(m_probEdgesList);

		String[] numLabelsString = { "2", "4", "6", "8", "10", "12", "15",
				"20", "26" };
		m_numLabelsList = new JComboBox(numLabelsString);
		m_numLabelsList.setEditable(true);
		m_numLabelsList.setSelectedIndex(4);
		Panel numLabelsPanel = new Panel(new GridLayout(2, 1));
		numLabelsPanel.add(new JLabel("Num labels:"));
		numLabelsPanel.add(m_numLabelsList);

		Panel comboPanel = new Panel(new GridLayout(1, 5));
		comboPanel.add(numModelsPanel);
		comboPanel.add(numTargetsPanel);
		comboPanel.add(numVerticesPanel);
		comboPanel.add(probEdgesPanel);
		comboPanel.add(numLabelsPanel);

		m_testGraphButton = new JButton("Run test");
		Panel topPanel = new Panel(new GridLayout(1, 2));
		topPanel.add(modelSizePanel);
		topPanel.add(m_testGraphButton);

		Panel buttonPanel = new Panel(new GridLayout(2, 1));
		buttonPanel.add(topPanel);
		buttonPanel.add(comboPanel);

		m_sourceTextArea = new JTextArea(2, 40);
		m_sourceTextArea.setLineWrap(true);
		m_sourceTextArea.setWrapStyleWord(true);
		m_sourceTextArea.setFont(new Font("Monospaced", Font.PLAIN, 12));
		JScrollPane sourceScroll = new JScrollPane(m_sourceTextArea);
		getContentPane().setLayout(new BorderLayout());
		getContentPane().add("North", buttonPanel);
		getContentPane().add("Center", sourceScroll);

		/*
		 * fileButton.addActionListener( new ActionListener() { public void
		 * actionPerformed(ActionEvent event) { // Sets directory to last
		 * directory where a file was chosen
		 * fileChoice.setCurrentDirectory(lastDirectory); if
		 * (fileChoice.showOpenDialog(getContentPane()) ==
		 * JFileChooser.APPROVE_OPTION) { File chosenFile =
		 * fileChoice.getSelectedFile(); try { FileInputStream chosenInput = new
		 * FileInputStream(chosenFile); } catch (FileNotFoundException e) {
		 * m_sourceTextArea.setText(e.toString()); } catch (IOException e) {
		 * m_sourceTextArea.setText(e.toString()); } lastDirectory =
		 * fileChoice.getCurrentDirectory(); } else {
		 * System.out.println("Cancelled."); } } } );
		 */

		m_testGraphButton.addActionListener(this);
	}

	private void handleTestGraph() {
		Window owner = m_progressWindow.getOwner();
		Point parentLocation = owner.getLocationOnScreen();
		Dimension ownerDim = owner.getSize();
		Dimension windowDim = m_progressWindow.getSize();
		parentLocation.x += ownerDim.width / 2 - windowDim.width / 2;
		parentLocation.y += ownerDim.height / 2 - windowDim.height / 2;
		m_progressWindow.setLocation(parentLocation);
		m_progressWindow.show();
		m_progressWindow.toFront();
		m_testGraphButton.setEnabled(false);
		m_progressThread = new Thread(this);
		m_progressThread.start();
	}

	public void handleStopTest() {
		// Still have to handle the interrupt in decomposition class
		// not worth the bother for now
		try {
			m_progressThread.interrupt();
		} catch (SecurityException e) {
			System.out.println(e.toString());
		}
		m_progressThread = null;
		m_progressWindow.hide();
		m_testGraphButton.setEnabled(true);
	}

	public void actionPerformed(ActionEvent event) {
		if (event.getSource() == m_testGraphButton) {
			handleTestGraph();
		} else if (event.getSource() == m_randomModel) {
			m_constModelSize = false;
		} else if (event.getSource() == m_constantModel) {
			m_constModelSize = true;
		}
	}

	public static void main(String args[]) {
		TestMatch testFrame = new TestMatch();
	}
}
