package com.wcohen.ss;

import java.util.Iterator;

import com.wcohen.ss.api.StringWrapper;
import com.wcohen.ss.api.Token;
import com.wcohen.ss.api.Tokenizer;
import com.wcohen.ss.tokens.SimpleTokenizer;

/**
 */
public class SLIMToken extends AbstractStatisticalTokenDistance
{
	private Tokenizer tokenizer = SimpleTokenizer.DEFAULT_TOKENIZER;
	
	public SLIMToken() {;}
	
	public String toString() { return "[SLIMToken]"; }
	
	/** Preprocess a string by finding tokens and giving them appropriate weights */ 
	public StringWrapper prepare(String s) {
		BagOfTokens bag = new BagOfTokens(s, tokenizer.tokenize(s));
		// reweight by p(token)
		for (Iterator i=bag.tokenIterator(); i.hasNext(); ) {
			Token tok = (Token)i.next();
			if (collectionSize>0) {
				Integer dfInteger = (Integer)documentFrequency.get(tok);
				double df = dfInteger==null ? 1.0 : dfInteger.intValue();
				bag.setWeight( tok, df/collectionSize );
			} else {
				bag.setWeight( tok, 0.1 );
			}
		}
		return bag;
	}
	
	//
	// real code
	//
	
	public double score(StringWrapper s,StringWrapper t) 
	{
		return 0;
	}
	
	public String explainScore(StringWrapper s, StringWrapper t)	
	{
		StringBuffer buf = new StringBuffer();
		BagOfTokens a = (BagOfTokens)s;
		BagOfTokens b = (BagOfTokens)t;
		int numCommon = 0;
		for (Iterator i = a.tokenIterator(); i.hasNext(); ) {
			Token tok = (Token)i.next();
			if (b.contains(tok)) {
				numCommon++;
			}
		}
		// now estimate zijk for each matching token at i,j
		double numCommonTokens = numCommon-0.5;
		for (int k=0; k<3; k++) {
			double newNumCommonTokens = 0;
			for (Iterator i = a.tokenIterator(); i.hasNext(); ) {
				Token tok = (Token)i.next();
				if (b.contains(tok)) {
					double pk = b.getWeight(tok);
					double priorZijk = pk * (1.0/a.size()) * (1.0/b.size());
					double m1 = a.size() - numCommonTokens;
					double n1 = b.size() - numCommonTokens;
					double pMijkGivenZijk = 1.0;
					double pMijkGivenNotZijk = 1.0 - Math.exp( (m1+1) * (n1+1) * Math.log(1.0 - pk*pk) );
					double pMijk = (pMijkGivenZijk * priorZijk) + (pMijkGivenNotZijk * (1.0 - priorZijk ));
					double pZijkGivenMijk = (pMijkGivenZijk*priorZijk) / pMijk;
					buf.append(" tok='"+tok+"' pk="+pk+" m1:"+m1+" n1:"+n1);
					buf.append(" pMijk="+pMijk);
					buf.append("' pMijkGivenNotZijk="+pMijkGivenNotZijk+" pZijkGivenMijk="+pZijkGivenMijk+"\n");
					newNumCommonTokens += pZijkGivenMijk;
				}
			}
			numCommonTokens = newNumCommonTokens;
			buf.append("numCommonTokens -> "+numCommonTokens);
		}
		buf.append("score = "+numCommonTokens+" / "+ (a.size()+b.size()-numCommonTokens) );
		return buf.toString();
	}
	
	
	static public void main(String[] argv) {
		doMain(new SLIMToken(), argv);
	}
}
