package ch.unizh.ifi.isparql.indexing;

import ch.unizh.ifi.isparql.indexing.indexer.*;
import ch.unizh.ifi.isparql.indexing.api.impl.Indexer;

/**
 * This is a factory for indexing tools. There are two indexing
 * tools implemented yet, the selectivity estimation index (SEI)
 * and the similarity index (SI). In order to abstract from 
 * a specific indexer in the framework application, this factory
 * returns the corresponding instance object given a 
 * keyword which identifies the indexer (SEI or SI). If the indexer
 * is not found (ie, unknown keyword), an error message is printed
 * and the application is terminated.
 * 
 * @author		Markus Stocker
 * @version 	$Id: IndexerFactory.java,v 1.1 2006-11-20 18:48:54 ms Exp $
 */

public class IndexerFactory 
{	
	public static Indexer create(String inFileName, String type) 
	{	
		if (type.equals("SEI"))
			return new SelectivityEstimationIndexer(inFileName);
		if (type.equals("SI"))
			return new SimilarityIndexer(inFileName);
			
		System.out.println("[IndexerFactory ERROR] Indexer not found: " + type);
		System.exit(0);
		
		return null;
	}
}
/* 
 * (c) Copyright 2007 University of Zurich, Switzerland
 * All rights reserved.
 *
 * This file is licensed under a GNU Lesser General Public License.
 *
 * For more information about license terms see LICENSE.txt that comes with
 * the main iSPARQL-OptARQ distribution or visit the GNU website at
 * http://www.gnu.org/licenses/lgpl.html
 *
 */
