package ch.unizh.ifi.isparql.optimizer;

import java.util.*;

/**
 * Utility class for query variables statistics. 
 * Basically a map of query variables with a corresponding integer counter
 * This class is required to check if a variable is only used in the 
 * substitution triple pattern or elsewhere (projection, order by, as subject).
 * 
 * During query pattern prepare stage, the stats are incremented for all 
 * variables found in triple patterns, projection and order by.
 * During query pattern transform stage, the stats are used to check if 
 * a specific variable is used somewhere else 
 * (this is the case, if the counter is > 1).
 *
 * @author		Markus Stocker
 * @version 	$Id: QueryVarStats.java,v 1.1 2006-11-20 18:48:54 ms Exp $
 */

public class QueryVarStats
{	
	/** The datastructure where the variable statistics are stored */
	private Map<String, Integer> queryVarStats = new HashMap<String, Integer>();

	/**
	 * Increment the statistics for a variable
	 * 
	 * @param key
	 */
	public void inc(String key)
	{
		Integer value = get(key);
		
		value++;
		
		queryVarStats.put(key, value);
	}
	
	/**
	 * Decrement the statistics for a variable 
	 * 
	 * @param key
	 */
	public void dec(String key)
	{
		Integer value = get(key);
		
		value--;
		
		queryVarStats.put(key, value);
	}
	
	/**
	 * Get the statistics for a variable (the frequency)
	 * 
	 * @param key
	 * @return Integer
	 */
	public Integer get(String key)
	{	
		if (queryVarStats.containsKey(key))
			return queryVarStats.get(key);
		else 
			return new Integer(0);
	}
}
/* 
 * (c) Copyright 2007 University of Zurich, Switzerland
 * All rights reserved.
 *
 * This file is licensed under a GNU Lesser General Public License.
 *
 * For more information about license terms see LICENSE.txt that comes with
 * the main iSPARQL-OptARQ distribution or visit the GNU website at
 * http://www.gnu.org/licenses/lgpl.html
 *
 */
