package ch.unizh.ifi.isparql.optimizer;

import ch.unizh.ifi.isparql.optimizer.api.impl.AbstractRule;
import ch.unizh.ifi.isparql.optimizer.rule1.RewriteFilterVar;
import ch.unizh.ifi.isparql.optimizer.rule2.MoveUpFilterExpr;
import ch.unizh.ifi.isparql.optimizer.rule3.ReorderImprecise;
import ch.unizh.ifi.isparql.optimizer.rule4.ReorderBySelectivity;
import ch.unizh.ifi.isparql.optimizer.rule5.RemoveDispensablePattern;
import java.util.*;

/**
 * Registry for optimization rules. This is a module which allows dynamically
 * adding new rules, or removing existing ones. Rules are organized in packages
 * (ruleN.*) and extends the AbstractRule. The rule order specified in this 
 * registry is relevant for their execution. Rules are thus executed in 
 * the same order like they are added to the registry.
 *
 * @see 		ch.unizh.ifi.isparql.optimizer.api.impl.AbstractRule
 * @author		Markus Stocker
 * @version 	$Id: RuleRegistry.java,v 1.1 2006-11-20 18:48:54 ms Exp $
 */

public class RuleRegistry 
{
	/** The registry list of rules */
	private List<AbstractRule> registry = new ArrayList<AbstractRule>();
	
	/** Register new rules */
	public RuleRegistry()
	{		
		add(new RemoveDispensablePattern());
		add(new ReorderImprecise());
		add(new RewriteFilterVar());
		add(new MoveUpFilterExpr());
		add(new ReorderBySelectivity());
	}
	
	/**
	 * Add a rule to the registry
	 * 
	 * @param rule
	 */
	public void add(AbstractRule rule)
	{
		registry.add(rule);
	}
	
	/**
	 * Return the list of rules
	 * 
	 * @return List<AbstractRule>
	 */
	public List<AbstractRule> getAll()
	{	
		return registry;
	}
}
/* 
 * (c) Copyright 2007 University of Zurich, Switzerland
 * All rights reserved.
 *
 * This file is licensed under a GNU Lesser General Public License.
 *
 * For more information about license terms see LICENSE.txt that comes with
 * the main iSPARQL-OptARQ distribution or visit the GNU website at
 * http://www.gnu.org/licenses/lgpl.html
 *
 */
