package ch.unizh.ifi.isparql.optimizer.api;

import com.hp.hpl.jena.query.core.ElementTriplePattern;

/**
 * Cost Model interface. The interface need to be implemented
 * by specific cost models which allows the cost computation
 * for a specific triple pattern based on some selectivity 
 * estimation models.
 *
 * @author		Markus Stocker
 * @version 	$Id: ICostModel.java,v 1.1 2006-11-20 18:48:54 ms Exp $
 */

public interface ICostModel 
{
	/**
	 * To implement by a specific cost model. The framework allows
	 * the implementation of different cost models as specific 
	 * implementations which extends the AbstractCostModel.
	 * Each specific implementation needs to implement this 
	 * interface.
	 * 
	 * @see ch.unizh.ifi.isparql.optimizer.api.impl.AbstractCostModel
	 * @param el
	 * @return Double
	 */
	public Double costForTriplePattern(ElementTriplePattern el);
}
/* 
 * (c) Copyright 2007 University of Zurich, Switzerland
 * All rights reserved.
 *
 * This file is licensed under a GNU Lesser General Public License.
 *
 * For more information about license terms see LICENSE.txt that comes with
 * the main iSPARQL-OptARQ distribution or visit the GNU website at
 * http://www.gnu.org/licenses/lgpl.html
 *
 */
