package ch.unizh.ifi.isparql.optimizer.api;

import com.hp.hpl.jena.query.core.ElementGroup;

/**
 * Rule interface. This is the interface which specific rules
 * need to implement. Basically each rule, has a prepare and 
 * a transform stage. During prepare stage, we may collect information
 * about the query, which are required in transform stage to 
 * rewrite the query in oder that it is optimal against a 
 * statistical selectivity estimation model.
 *
 * @author		Markus Stocker
 * @version 	$Id: IRule.java,v 1.1 2006-11-20 18:48:54 ms Exp $
 */

public interface IRule {
	
	/** Optimization prepare stage. To implement by specific rules. */
	public abstract void prepare();
	
	/**
	 * Optimization transform stage.
	 * To implement by specific rules.
	 * 
	 * @return ElementGroup
	 */
	public abstract ElementGroup transform();
}
/* 
 * (c) Copyright 2007 University of Zurich, Switzerland
 * All rights reserved.
 *
 * This file is licensed under a GNU Lesser General Public License.
 *
 * For more information about license terms see LICENSE.txt that comes with
 * the main iSPARQL-OptARQ distribution or visit the GNU website at
 * http://www.gnu.org/licenses/lgpl.html
 *
 */
