package ch.unizh.ifi.isparql.optimizer.api.impl;

import ch.unizh.ifi.isparql.optimizer.CostModel;
import ch.unizh.ifi.isparql.optimizer.api.IRule;
import com.hp.hpl.jena.query.Query;
import com.hp.hpl.jena.query.core.ElementGroup;

/**
 * Abstract rule, which is extended by specific rules inside of the framework.
 * The abstract rule implements the exec() method which includes the general
 * logic of rule execution, ie, executing the prepare and transform stage.
 *
 * @author		Markus Stocker
 * @version 	$Id: AbstractRule.java,v 1.1 2006-11-20 18:48:54 ms Exp $
 */

public abstract class AbstractRule implements IRule
{
	/** The cost model object, required for query optimization */
	protected CostModel cm = null;
	/** The original ARQ query object, which is transformed to an optimized one */
	protected Query inQuery = null;
	/** The original ARQ query element group */
	protected ElementGroup inElementGroup = null;
	/** The optimized ARQ query element group */
	protected ElementGroup outElementGroup = null;
	
	/**
	 * This method is executed by ModOptimizer.
	 * Execute both stages prepare and transform of a rule.
	 * The specific stages are implemented in specific rules (ruleN.*).
	 * 
	 * @param inQuery
	 * @param outElementGroup
	 * @param cm
	 * @return ElementGroup
	 */
	public ElementGroup exec(Query inQuery, ElementGroup outElementGroup, CostModel cm)
	{
		// The cost model
		this.cm = cm;
		// The input query (not optimized)
		this.inQuery = inQuery;
		// The input query element group (to be transformed)
		this.inElementGroup = outElementGroup;
		// The output query element group (optimized)
		this.outElementGroup = new ElementGroup();
		
		// Execute prepare stage for specific rule (to implement)
		prepare();
		
		// Execute transform stage for specific rule (to implement)
		return transform();
	}
}
/* 
 * (c) Copyright 2007 University of Zurich, Switzerland
 * All rights reserved.
 *
 * This file is licensed under a GNU Lesser General Public License.
 *
 * For more information about license terms see LICENSE.txt that comes with
 * the main iSPARQL-OptARQ distribution or visit the GNU website at
 * http://www.gnu.org/licenses/lgpl.html
 *
 */
