package ch.unizh.ifi.isparql.optimizer.rule1;

import ch.unizh.ifi.isparql.optimizer.QueryVarStats;
import ch.unizh.ifi.isparql.optimizer.api.impl.AbstractRule;
import ch.unizh.ifi.isparql.optimizer.rule1.visitor.ElementGroupPrepare;
import ch.unizh.ifi.isparql.optimizer.rule1.visitor.ElementGroupTransform;
import com.hp.hpl.jena.graph.Node;
import com.hp.hpl.jena.query.core.ElementGroup;
import com.hp.hpl.jena.query.expr.ExprFunction;
import java.util.*;

/**
 * Rewrite Filter Variables.
 * This rule provides the logic to extract FILTER variables which can be
 * substituted in triple pattern elements as objects
 *
 * @author		Markus Stocker
 * @version 	$Id: RewriteFilterVar.java,v 1.1 2006-11-20 18:48:54 ms Exp $
 */

public class RewriteFilterVar extends AbstractRule
{
	private QueryVarStats queryVarStats = null;
	private Map<String, Node> exprEqualsMap = null;
	private List<ExprFunction> exprNotLogicalAnd = new ArrayList<ExprFunction>();
	
	/**
	 * Optimization prepare stage.
	 * The prepare stage extracts information about the input query which
	 * are required during the following transform stage to optimize the query.
	 */
	public void prepare()
	{	
		ElementGroupPrepare visitor = new ElementGroupPrepare(inQuery, 
															  inElementGroup);
		
		visitor.exec();
		
		exprEqualsMap = visitor.getExprEqualsMap();
		queryVarStats = visitor.getQueryVarStats();
		exprNotLogicalAnd = visitor.getExprNotLogicalAnd();
	}
	
	/**
	 * Optimization transform stage.
	 * Use the previous extracted information to optimize the query.
	 * Basically a new element group is constructed, using a visitor which
	 * gets throught each element of the query input element group and 
	 * constructs an optimized output element group based on the input
	 * element group.
	 * 
	 * @return ElementGroup
	 */
	public ElementGroup transform()
	{		
		ElementGroupTransform visitor = new ElementGroupTransform(queryVarStats,
																  inElementGroup, 
																  outElementGroup,
																  exprEqualsMap,
																  exprNotLogicalAnd);

		visitor.exec();
		
		return visitor.getOutElementGroup();
	}
}/* 
 * (c) Copyright 2007 University of Zurich, Switzerland
 * All rights reserved.
 *
 * This file is licensed under a GNU Lesser General Public License.
 *
 * For more information about license terms see LICENSE.txt that comes with
 * the main iSPARQL-OptARQ distribution or visit the GNU website at
 * http://www.gnu.org/licenses/lgpl.html
 *
 */
