package ch.unizh.ifi.isparql.optimizer.rule1.visitor;

import com.hp.hpl.jena.query.core.ElementGroup;
import com.hp.hpl.jena.query.core.ElementFilter;
import com.hp.hpl.jena.query.expr.Expr;
import com.hp.hpl.jena.query.expr.ExprFunction;
import com.hp.hpl.jena.query.expr.ExprVisitorBase;
import java.util.*;

/**
 * This class provides the logic to visit a FILTER expression during
 * transform stage.
 * 
 * We need to process all FILTER expressions that couldn't be substituted
 * Create a new FILTER expression to add to the ouput element group.
 *
 * @author		Markus Stocker
 * @version 	$Id: FilterExprTransform.java,v 1.1 2006-11-20 18:48:54 ms Exp $
 */

public class FilterExprTransform extends ExprVisitorBase
{	
	/** The optimized ARQ element group */
	private ElementGroup outElementGroup = null;
	/** List of FILTER expressions that are not concatenated by a logical AND */
	private List<ExprFunction> exprNotLogicalAnd = new ArrayList<ExprFunction>();
	
	/**
	 * Constructor, takes just a list of arguments
	 * 
	 * @param exprNotLogicalAnd
	 * @param outElementGroup
	 */
	public FilterExprTransform(List<ExprFunction> exprNotLogicalAnd, ElementGroup outElementGroup)
	{
		this.exprNotLogicalAnd = exprNotLogicalAnd;
		this.outElementGroup = outElementGroup;
	}
	
	/** Create the FILTER expression to add to the output element group */
	public void exec()
	{
		/* 
		 * Take all not logical AND expressions found in previous stage 
		 * (prepare) and create filter elements for them
		 */
		for (Iterator iter = exprNotLogicalAnd.iterator(); iter.hasNext(); )
			outElementGroup.addElementFilter(new ElementFilter((Expr)iter.next()));
	}
	
	/**
	 * Retur the output element group with the new FILTER element
	 * 
	 * @return ElementGroup
	 */
	public ElementGroup getOutElementGroup()
	{
		return outElementGroup;
	}
}
/* 
 * (c) Copyright 2007 University of Zurich, Switzerland
 * All rights reserved.
 *
 * This file is licensed under a GNU Lesser General Public License.
 *
 * For more information about license terms see LICENSE.txt that comes with
 * the main iSPARQL-OptARQ distribution or visit the GNU website at
 * http://www.gnu.org/licenses/lgpl.html
 *
 */
