package ch.unizh.ifi.isparql.optimizer.rule2;

import java.util.ArrayList;
import java.util.List;
import ch.unizh.ifi.isparql.optimizer.api.impl.AbstractRule;
import ch.unizh.ifi.isparql.optimizer.rule2.visitor.ElementGroupPrepare;
import ch.unizh.ifi.isparql.optimizer.rule2.visitor.ElementGroupTransform;
import com.hp.hpl.jena.query.core.ElementGroup;
import com.hp.hpl.jena.query.expr.ExprFunction;

/**
 * This rule identifies FILTER expressions that could not be rewritten,
 * because they don't have a (=) equals operator.
 *
 * @author		Markus Stocker
 * @version 	$Id: MoveUpFilterExpr.java,v 1.1 2006-11-20 18:48:54 ms Exp $
 */

public class MoveUpFilterExpr extends AbstractRule
{	
	/** The list of FILTER expressions concatenated by a logical AND */
	private List<ExprFunction> logicalAndExpr = new ArrayList<ExprFunction>();
	/** The list of FILTER expressions not concatenated by a logical AND */
	private List<ExprFunction> notLogicalAndExpr = new ArrayList<ExprFunction>();
	
	/** 
	 * The prepare stage for this rule. Identify the list of FILTER expressions
	 * which are concatenated by a logical AND, and the list of FILTER
	 * expression elements not concatenated by a logical AND.
	 */
	public void prepare()
	{
		ElementGroupPrepare visitor = new ElementGroupPrepare(inElementGroup);

		visitor.exec();
		
		logicalAndExpr = visitor.getLogicalAndExpr();
		notLogicalAndExpr = visitor.getNotLogicalAndExpr();
	}
	
	/** 
	 * The transform stage for this rule. Transform the ARQ ElementGroup, against
	 * the previously identified list of FILTER expression elements concatenated 
	 * by a logical AND.
	 */
	public ElementGroup transform()
	{			
		ElementGroupTransform visitor = new ElementGroupTransform(inElementGroup, 
				  												  outElementGroup,
				  												  logicalAndExpr,
				  												  notLogicalAndExpr);

		visitor.exec();

		return visitor.getOutElementGroup();
	}
}/* 
 * (c) Copyright 2007 University of Zurich, Switzerland
 * All rights reserved.
 *
 * This file is licensed under a GNU Lesser General Public License.
 *
 * For more information about license terms see LICENSE.txt that comes with
 * the main iSPARQL-OptARQ distribution or visit the GNU website at
 * http://www.gnu.org/licenses/lgpl.html
 *
 */
