package ch.unizh.ifi.isparql.optimizer.rule4;

import ch.unizh.ifi.isparql.optimizer.api.impl.AbstractRule;
import ch.unizh.ifi.isparql.optimizer.rule4.visitor.ElementGroupPrepare;
import ch.unizh.ifi.isparql.optimizer.rule4.visitor.ElementGroupTransform;
import com.hp.hpl.jena.query.core.Element;
import com.hp.hpl.jena.query.core.ElementGroup;
import java.util.*;

/**
 * Reorder triple pattern by selectivity
 *
 * @author		Markus Stocker
 * @version 	$Id: ReorderBySelectivity.java,v 1.1 2006-11-20 18:48:54 ms Exp $
 */

public class ReorderBySelectivity extends AbstractRule
{	
	private Map<Double, List<Element>> stages = new TreeMap<Double, List<Element>>();
	
	/**
	 * During prepare stage, we calculate the selectivity (the estimated size of 
	 * triples matching a pattern) to get the execution costs for each triple 
	 * pattern defined. This leads to an ordering, which will be used in transform
	 * stage to reorder the triple patterns in an optimal execution.
	 * Visit each pattern in element group and calculate the costs, except for 
	 * imprecise (iSPARQL) pattern. Reorder the patterns by costs and return
	 * a list.
	 */
	public void prepare() 
	{
		ElementGroupPrepare visitor = new ElementGroupPrepare(inElementGroup, cm);
		
		visitor.exec();
		
		stages = visitor.getCostStages();
	}
	
	/**
	 * The transform stage of this rule
	 * 
	 * @return ElementGroup
	 */
	public ElementGroup transform()
	{			
		/* Get the list of pattern reordered by costs and write the new element group */
		ElementGroupTransform visitor = new ElementGroupTransform(inElementGroup, 
				  												  outElementGroup,
				  												  stages);

		visitor.exec();

		return visitor.getOutElementGroup();
	}
}/* 
 * (c) Copyright 2007 University of Zurich, Switzerland
 * All rights reserved.
 *
 * This file is licensed under a GNU Lesser General Public License.
 *
 * For more information about license terms see LICENSE.txt that comes with
 * the main iSPARQL-OptARQ distribution or visit the GNU website at
 * http://www.gnu.org/licenses/lgpl.html
 *
 */
