package ch.unizh.ifi.isparql.optimizer.rule4.visitor;

import com.hp.hpl.jena.query.core.Element;
import com.hp.hpl.jena.query.core.ElementFilter;
import com.hp.hpl.jena.query.core.ElementGroup;
import com.hp.hpl.jena.query.core.ElementTriplePattern;
import com.hp.hpl.jena.query.core.ElementVisitorBase;
import java.util.*;

/**
 * This class provides the logic to visit query triple pattern
 * during the transform stage. We use the information extracted from
 * the previous prepare stage to transform and optimize the query.
 *
 * @author		Markus Stocker
 * @version 	$Id: ElementGroupTransform.java,v 1.1 2006-11-20 18:48:54 ms Exp $
 */

public class ElementGroupTransform extends ElementVisitorBase
{
	/** The output query element group (optimized) */
	private ElementGroup outElementGroup = null;
	/** Map of cost stages with the corresponding triple patterns for the stage */
	private Map<Double, List<Element>> stages = null;
	
	/**
	 * Constructor, takes some params
	 * 
	 * @param inElementGroup
	 * @param outElementGroup
	 * @param stages
	 */
	public ElementGroupTransform(ElementGroup inElementGroup, ElementGroup outElementGroup, Map<Double, List<Element>> stages)
	{	
		this.outElementGroup = outElementGroup;
		this.stages = stages;
	}
	
	/** Process logic when transforming the input query */
	public void exec()
	{
		processElementGroup();
	}
	
	/**
	 * Return the output element group
	 * 
	 * @return ElementGroup
	 */
	public ElementGroup getOutElementGroup()
	{
		return outElementGroup;
	}

	// Walk throught the input element group
	private void processElementGroup()
	{		
		for (Iterator iter = stages.keySet().iterator(); iter.hasNext(); )
		{	
			List<Element> elements = stages.get((Double)iter.next());
			
			for (Iterator it = elements.iterator(); it.hasNext(); )
			{
				Element element = (Element)it.next();
				
				if (element.getClass().equals(ElementTriplePattern.class))
					outElementGroup.addElementTriplePattern((ElementTriplePattern)element);
				else
					outElementGroup.addElementFilter((ElementFilter)element);
			}
		}
	}
}
/* 
 * (c) Copyright 2007 University of Zurich, Switzerland
 * All rights reserved.
 *
 * This file is licensed under a GNU Lesser General Public License.
 *
 * For more information about license terms see LICENSE.txt that comes with
 * the main iSPARQL-OptARQ distribution or visit the GNU website at
 * http://www.gnu.org/licenses/lgpl.html
 *
 */
