package ch.unizh.ifi.isparql.optimizer.rule5;


import ch.unizh.ifi.isparql.optimizer.QueryVarStats;
import ch.unizh.ifi.isparql.optimizer.api.impl.AbstractRule;
import ch.unizh.ifi.isparql.optimizer.rule5.visitor.ElementGroupPrepare;
import ch.unizh.ifi.isparql.optimizer.rule5.visitor.ElementGroupTransform;
import com.hp.hpl.jena.query.core.ElementGroup;

/**
 * This rule identifies potentially dispensable patterns. Patterns with variables,
 * that are not read in query (projection, other triple patterns, order by) are
 * considered to be dispensable. Please note, that removing such a pattern 
 * usually affects the result set!
 *
 * @author		Markus Stocker
 * @version 	$Id: RemoveDispensablePattern.java,v 1.1 2006-11-20 18:48:54 ms Exp $
 */

public class RemoveDispensablePattern extends AbstractRule
{	
	/** Store some statistics about the number of occurrences of variables */
	private QueryVarStats queryVarStats = null;
	
	/** The prepare stage of the rule, just create statistics about the variables occurrences */
	public void prepare() 
	{
		ElementGroupPrepare visitor = new ElementGroupPrepare(inQuery, inElementGroup);
		
		visitor.exec();
		
		queryVarStats = visitor.getQueryVarStats();
	}
	
	/**
	 * The transform stage of the rule. Use the variable statistics gathered during 
	 * prepare stage to identify dispensable triple patterns
	 * 
	 * @return ElementGroup
	 */
	public ElementGroup transform()
	{	
		ElementGroupTransform visitor = new ElementGroupTransform(inElementGroup, 
				  												  outElementGroup,
				  												  queryVarStats);

		visitor.exec();

		return visitor.getOutElementGroup();
	}
}/* 
 * (c) Copyright 2007 University of Zurich, Switzerland
 * All rights reserved.
 *
 * This file is licensed under a GNU Lesser General Public License.
 *
 * For more information about license terms see LICENSE.txt that comes with
 * the main iSPARQL-OptARQ distribution or visit the GNU website at
 * http://www.gnu.org/licenses/lgpl.html
 *
 */
