package ch.unizh.ifi.isparql.query;

import ch.unizh.ifi.isparql.query.aggregator.*;
import ch.unizh.ifi.isparql.query.api.impl.AbstractAggregator;
import ch.unizh.ifi.isparql.query.exception.AggregatorNotDefined;

/**
 * Factory for aggregation methods of similarity scores
 * 
 * @see ch.unizh.ifi.isparql.Aggregator
 * @see ch.unizh.ifi.isparql.aggregator
 *
 * @author		Markus Stocker
 * @version 	$Id: AggregatorFactory.java,v 1.3 2006-11-21 18:31:46 ms Exp $
 */

public class AggregatorFactory 
{	
	/**
	 * Factory method for specific aggregators, given a keyword
	 * which represents the aggregator. Implemented aggregation methods
	 * are: simple, avg, max, median, min, noisyand, noisyor, sum,
	 * geometric, harmonic. Please use AbstractAggregator.* to
	 * address the name of a specific aggregator. To add new aggregation
	 * methods, the specific class has to be included to this factory 
	 * and the AbstractFactory must include the name. If an aggregation
	 * method is adresses (in query or in some similarity strategy 
	 * implementation) which is not found, this method throws an
	 * error.
	 * 
	 * @see ch.unizh.ifi.isparql.query.api.impl.AbstractAggregator
	 * @param aggregator
	 * @return AbstractAggregator
	 */
	static public AbstractAggregator create(String aggregator)
	{		
		if (aggregator.equals(AbstractAggregator.simple))
			return new SimpleAggregator();
		if (aggregator.equals(AbstractAggregator.avg))
			return new AvgAggregator();
		if (aggregator.equals(AbstractAggregator.max))
			return new MaxAggregator();
		if (aggregator.equals(AbstractAggregator.median))
			return new MedianAggregator();
		if (aggregator.equals(AbstractAggregator.min))
			return new MinAggregator();
		if (aggregator.equals(AbstractAggregator.noisyand))
			return new NoisyAndAggregator();
		if (aggregator.equals(AbstractAggregator.noisyor))
			return new NoisyOrAggregator();
		if (aggregator.equals(AbstractAggregator.sum))
			return new SumAggregator();
		if (aggregator.equals(AbstractAggregator.geometric))
			return new GeometricAggregator();
		if (aggregator.equals(AbstractAggregator.harmonic))
			return new HarmonicAggregator();
		
		// The requested aggregator is not implemented, get an error and exit
		new AggregatorNotDefined("AggregatorFactory: aggregator not found (" + aggregator + ")");
		
		return null;
	}
}

/* 
 * (c) Copyright 2007 University of Zurich, Switzerland
 * All rights reserved.
 *
 * This file is licensed under a GNU Lesser General Public License.
 *
 * For more information about license terms see LICENSE.txt that comes with
 * the main iSPARQL-OptARQ distribution or visit the GNU website at
 * http://www.gnu.org/licenses/lgpl.html
 *
 */
