package ch.unizh.ifi.isparql.query;

/**
 * Global constant declarations for iSPARQL framework.
 * 
 * @author		Markus Stocker
 * @version 	$Id: Constants.java,v 1.3 2006-11-21 18:31:46 ms Exp $
 */

public class Constants 
{
	/** 
	 * Constant symbol which identifies the strategy registry saved to the ARQ execution context. 
	 * The strategy registry is required, to store the strategy configuration and similarity
	 * strategy execution, which is processed at different stages during ARQ query processing.
	 * The strategy registry, which is save to the ARQ execution context, allows a "memory" 
	 * for the similarity strategy processing during query execution. Since, similarity 
	 * strategy are configured and executed not during a single stage (in a single virtual 
	 * triple, imprecise statement), we need a way to store the configuration (the 
	 * params specified over imprecise statements, threshold, strategy name, ignore case, ...).
	 * The best way I found, was to store the strategy object to the ARQ execution context,
	 * and ready is from there while a strategy is processed.
	 * 
	 * @see ch.unizh.ifi.isparql.StrategyRegistry
	 */
	public static final String STRATEGY_REGISTRY = new String("ISPARQL_STRATEGY_REGISTRY");

	/**
	 * Registry for measure objects: similarity measures may be instantiated only once, 
	 * saved to the registry and then extracted again. Created for TFIDF because of training.
	 * TFIDF training is executed when the constructor is called, thus we should
	 * instantiate the measure only once to avoid the overhead of TFIDF documents training
	 * more than once. Saving similarity measure objects to this registry
	 * may be useful also in other cases. The registry is saved to the ARQ execution context
	 * in other that it can be reused at another stage of query processing.
	 * 
	 * @see ch.unizh.ifi.isparql.query.measure.weightingscheme.MeasureStringTFIDF
	 */
	public static final String MEASURE_REGISTRY = new String("ISPARQL_MEASURE_REGISTRY");

	/**
	 * This keyword holds the similarity index. The similarity index may be an in-memory
	 * hash (SimilarityIndexMem) or a wrapper class to a persistent MySQL database store
	 * (SimilarityIndexDB). The similarity index has to be created previously. For 
	 * a persistent store, a specific process (SimilarityIndexer) may be executed.
	 * For an in-memory similarity index, a query execution environment is required.
	 * The environment has to create the similarity index before processing the query. 
	 * Futher the environment has to save the created SimilarityIndex object to the
	 * ARQ query execution context after the ARQ QueryExecution is created 
	 * (for an example, refer to the similarity index evaluation framework, line 161)
	 * 
	 * @see ch.unizh.ifi.isparql.evaluation.si.Framework
	 * @see ch.unizh.ifi.isparql.indexing.indexer
	 * @see ch.unizh.ifi.isparql.query.optimizer
	 */
	public static final String SIMILARITY_INDEX = new String("ISPARQL_SIMILARITY_INDEX");

	/**
	 * This is the mysql database username used to store the similarity index. It is
	 * used in both similarity index creation class and in iSPARQL wrapper class,
	 * to similarity index optimization. Please note that it should be possible to 
	 * configure all database specific information in this class, not only the 
	 * username and password (for example also, the table name). Because of limited
	 * time, this is not implemented yet.
	 * 
	 * @see ch.unizh.ifi.isparql.indexing.indexer.SimilarityIndexer
	 * @see ch.unizh.ifi.isparql.query.optimizer.SimilarityIndexDB
	 */
	public static final String MYSQL_DB_UID = "stocker";
	
	/**
	 * This is the mysql database password used to access the persistent MYSQL similarity
	 * index. Please refer to the documentation for MYSQL_DB_UID for further information.
	 */
	public static final String MYSQL_DB_PWD = "stocker";
}
/* 
 * (c) Copyright 2007 University of Zurich, Switzerland
 * All rights reserved.
 *
 * This file is licensed under a GNU Lesser General Public License.
 *
 * For more information about license terms see LICENSE.txt that comes with
 * the main iSPARQL-OptARQ distribution or visit the GNU website at
 * http://www.gnu.org/licenses/lgpl.html
 *
 */
