package ch.unizh.ifi.isparql.query;

import simpack.api.impl.SimilarityMeasure;
import java.util.HashMap;
import java.util.Map;

/**
 * Registry for similarity measures (SimPack).
 * The registry is saved to the engine execution context. The measure registry
 * is used to store instantiated measure objects and is currently used
 * only by the TFIDF measure. The purpose is to avoid that the measure is 
 * instantiated more than once because of the inherent documents training
 * which is executed for TFIDF when the measure is instantiated (constructor).
 * Since this training requires some time, we should avoid that it
 * is executed more than once during iSPARQL query processing. Thus, we store
 * the instantiated measure object to this registry, which is stored
 * to the ARQ execution context. During a later stage in iSPARQL query
 * processing, we may extract the previously instantiated measure 
 * from the registry, instead to instantiate it again. Thus, we can avoid
 * training over and over.
 *
 * @author		Markus Stocker
 * @version 	$Id: MeasureRegistry.java,v 1.2 2006-11-21 12:41:56 ms Exp $
 */

public class MeasureRegistry 
{
	/**
	 * Map URI-references with simpack measure instances.
	 * (http://www.ifi.unizh.ch/isparql/measure/sequence/levenshtein).
	 * The URI-references are stored and organized in URIRegistry class.
	 * Mapping: URI (string) => SimPack SimilarityMeasure.
	 * 
	 * @see simpack.api.impl.SimilarityMeasure
	 * @see ch.unizh.ifi.isparql.query.URIRegistry
	 */
	Map<String, SimilarityMeasure> registry = new HashMap<String, SimilarityMeasure>();
		
	/**
	 * Put a simpack measure instance,
	 * identified by a URI-reference to the registry
	 * 
	 * @param uri
	 * @param measure
	 */
	public void put(String uri, SimilarityMeasure measure)
	{
		registry.put(uri, measure);
	}
	
	/**
	 * Get the measure from the registry by URI-reference
	 * 
	 * @param uri
	 * @return SimilarityMeasure
	 */
	public SimilarityMeasure get(String uri)
	{		
		return registry.get(uri);
	}
}
/* 
 * (c) Copyright 2007 University of Zurich, Switzerland
 * All rights reserved.
 *
 * This file is licensed under a GNU Lesser General Public License.
 *
 * For more information about license terms see LICENSE.txt that comes with
 * the main iSPARQL-OptARQ distribution or visit the GNU website at
 * http://www.gnu.org/licenses/lgpl.html
 *
 */
