package ch.unizh.ifi.isparql.query;

import com.hp.hpl.jena.graph.Node;
import com.hp.hpl.jena.query.core.Binding;
import com.hp.hpl.jena.query.core.Binding1;
import com.hp.hpl.jena.query.expr.*;
import java.util.*;

/**
 * iSPARQL strategy properties. This class defines all available isparql strategy 
 * properties, ie all properties that can be used as predicates in isparql imprecise
 * statements. Add new properties to this class. Moreover, this class is used
 * to bind values (similarity values, returned by isparql:similarity, or 
 * strategy URI-references (?strategy1 isparql:name "") to the ARQ bindings.
 * This may be not the best place where bindings should be implemented.
 *
 * @author		Markus Stocker
 * @version 	$Id: StrategyProperty.java,v 1.2 2006-11-21 12:41:56 ms Exp $
 */

public class StrategyProperty 
{	
	/** java:ch.unizh.ifi.isparql.query.property.aggregator */
	public static final String aggregator = new String("aggregator");
	/** java:ch.unizh.ifi.isparql.query.property.argument1 */
	public static final String argument1 = new String("argument1");
	/** java:ch.unizh.ifi.isparql.query.property.argument2 */
	public static final String argument2 = new String("argument2");
	/** java:ch.unizh.ifi.isparql.query.property.arguments */
	public static final String arguments = new String("arguments");
	/** java:ch.unizh.ifi.isparql.query.property.between */
	public static final String between = new String("between");
	/** java:ch.unizh.ifi.isparql.query.property.ignorecase */
	public static final String ignorecase = new String("ignorecase");
	/** java:ch.unizh.ifi.isparql.query.property.name */
	public static final String name = new String("name");
	/** java:ch.unizh.ifi.isparql.query.property.ontologyURI */
	public static final String ontologyURI = new String("ontologyURI");
	/** java:ch.unizh.ifi.isparql.query.property.ontologyBaseURI */
	public static final String ontologyBaseURI = new String("ontologyBaseURI");
	/** java:ch.unizh.ifi.isparql.query.property.ontologyRootURI */
	public static final String ontologyRootURI = new String("ontologyRootURI");
	/** java:ch.unizh.ifi.isparql.query.property.scores */
	public static final String scores = new String("scores");
	/** java:ch.unizh.ifi.isparql.query.property.weights */
	public static final String weights = new String("weights");
	/** java:ch.unizh.ifi.isparql.query.property.similarity */
	public static final String similarity = new String("similarity");
	/** java:ch.unizh.ifi.isparql.query.property.threshold */
	public static final String threshold = new String("threshold");
	
	/**
	 * Required to bind the URI of a strategy (?strategy1 isparql:name "").
	 * The subject holds the binding variable name. The node the URI to bind.
	 * 
	 * @see ch.unizh.ifi.isparql.query.property.name
	 * 
	 * @param binding
	 * @param node 
	 * @param node
	 * @return List<Binding>
	 */
	public static List<Binding> bind(Binding binding, Node subject, Node node)
	{
		List<Binding> bindings = new ArrayList<Binding>();
		
		bindings.add(new Binding1(binding, subject.getName(), node));
		
		return bindings;
	}
	
	/**
	 * This method is used to bind a similarity calculated from a strategy
	 * to the variable (?strategy1 isparql:similarity ?similarity)
	 * 
	 * @see ch.unizh.ifi.isparql.query.property.similarity
	 * 
	 * @param binding
	 * @param object
	 * @param similarity
	 * @return List<Binding>
	 */
	public static List<Binding> bind(Binding binding, Node object, Double similarity)
	{
		List<Binding> bindings = new ArrayList<Binding>();
		
		NodeValue nv = NodeValue.makeDouble(similarity);
		bindings.add(new Binding1(binding, object.getName(), nv.asNode()));
		
		return bindings;
	}
}
/* 
 * (c) Copyright 2007 University of Zurich, Switzerland
 * All rights reserved.
 *
 * This file is licensed under a GNU Lesser General Public License.
 *
 * For more information about license terms see LICENSE.txt that comes with
 * the main iSPARQL-OptARQ distribution or visit the GNU website at
 * http://www.gnu.org/licenses/lgpl.html
 *
 */
