package ch.unizh.ifi.isparql.query;

import java.util.*;
import com.hp.hpl.jena.query.expr.*;
import ch.unizh.ifi.isparql.query.api.impl.AbstractStrategy;
import ch.unizh.ifi.isparql.query.api.impl.AbstractMeasure;
import ch.unizh.ifi.isparql.query.exception.URINotFoundInRegistry;

/**
 * Registry for URI-references used in iSPARQL framework.
 * This class defines all URI-references for strategies, measures, and properties.
 *
 * @author		Markus Stocker
 * @version 	$Id: URIRegistry.java,v 1.2 2006-11-21 12:41:56 ms Exp $
 */

public class URIRegistry 
{	
	/** Mapping: String => String */
	private Map<String,String> registry = new HashMap<String,String>();
	
	/** http://www.ifi.unizh.ch/isparql/ */
	public static final String base = new String("http://www.ifi.unizh.ch/isparql/");
	/** http://www.ifi.unizh.ch/isparql/measure/ */
	public static final String measure = new String(base + "measure/");
	/** http://www.ifi.unizh.ch/isparql/measure/it/ */
	public static final String it = new String(measure + "it/");
	/** http://www.ifi.unizh.ch/isparql/measure/graph/ */
	public static final String graph = new String(measure + "graph/");
	/** http://www.ifi.unizh.ch/isparql/measure/tree/ */
	public static final String tree = new String(measure + "tree/");
	/** http://www.ifi.unizh.ch/isparql/measure/sequence/ */
	public static final String sequence = new String(measure + "sequence/");
	/** http://www.ifi.unizh.ch/isparql/measure/simmetrics/ */
	public static final String simmetrics = new String(measure + "simmetrics/");
	/** http://www.ifi.unizh.ch/isparql/measure/secondstring/ */
	public static final String secondstring = new String(measure + "secondstring/");
	/** http://www.ifi.unizh.ch/isparql/measure/owlsmx/ */
	public static final String owlsmx = new String(measure + "owlsmx/");
	/** http://www.ifi.unizh.ch/isparql/measure/weightingscheme/ */
	public static final String weightingscheme = new String(measure + "weightingscheme/");
	/** http://www.ifi.unizh.ch/isparql/measure/vector/ */
	public static final String vector = new String(measure + "vector/");
	/** http://www.ifi.unizh.ch/isparql/strategy/ */ 
	public static final String strategy = new String(base + "strategy/");
	/** java:ch.unizh.ch.ifi.isparql.query.property. */
	public static final String property = new String("java:ch.unizh.ifi.isparql.query.property.");
	
	/** Constructor. It creates the registry with URI-references. */
	public URIRegistry()
	{
		// URI-references for iSPARQL similarity strategies
		put(AbstractStrategy.levenshteinStrCmp, new String(strategy + AbstractStrategy.levenshteinStrCmp));
		put(AbstractStrategy.levenshteinResCmp, new String(strategy + AbstractStrategy.levenshteinResCmp));
		put(AbstractStrategy.linResCmp, new String(strategy + AbstractStrategy.linResCmp));
		put(AbstractStrategy.resnikResCmp, new String(strategy + AbstractStrategy.resnikResCmp));
		put(AbstractStrategy.shortestPathResCmp, new String(strategy + AbstractStrategy.shortestPathResCmp));
		put(AbstractStrategy.MITPHLevenshteinResCmp, new String(strategy + AbstractStrategy.MITPHLevenshteinResCmp));
		put(AbstractStrategy.MITPHTreeEditDistanceResCmp, new String(strategy + AbstractStrategy.MITPHTreeEditDistanceResCmp));
		put(AbstractStrategy.subGraphIsomorphismResCmp, new String(strategy + AbstractStrategy.subGraphIsomorphismResCmp));
		put(AbstractStrategy.scoreAggregator, new String(strategy + AbstractStrategy.scoreAggregator));
		put(AbstractStrategy.cosineStrCmp, new String(strategy + AbstractStrategy.cosineStrCmp));
		put(AbstractStrategy.jensenShannonStrCmp, new String(strategy + AbstractStrategy.jensenShannonStrCmp));
		put(AbstractStrategy.owlstcServicesCmp, new String(strategy + AbstractStrategy.owlstcServicesCmp));
		put(AbstractStrategy.MITPHLoLResCmp, new String(strategy + AbstractStrategy.MITPHLoLResCmp));
		put(AbstractStrategy.MITPHTFIDFResCmp, new String(strategy + AbstractStrategy.MITPHTFIDFResCmp));
		put(AbstractStrategy.LOLStrCmp, new String(strategy + AbstractStrategy.LOLStrCmp));
		put(AbstractStrategy.diceStrCmp, new String(strategy + AbstractStrategy.diceStrCmp));
		
		// URI-references for iSPARQL similarity measures
		put(AbstractMeasure.levenshtein, new String(sequence + AbstractMeasure.levenshtein));
		put(AbstractMeasure.resnik, new String(it + AbstractMeasure.resnik));
		put(AbstractMeasure.lin, new String(it + AbstractMeasure.lin));
		put(AbstractMeasure.shortestPath, new String(graph + AbstractMeasure.shortestPath));
		put(AbstractMeasure.subGraphIsomorphism, new String(graph + AbstractMeasure.subGraphIsomorphism));
		put(AbstractMeasure.treeEditDistance, new String(tree + AbstractMeasure.treeEditDistance));
		put(AbstractMeasure.cosine, new String(simmetrics + AbstractMeasure.cosine));
		put(AbstractMeasure.jensenShannon, new String(owlsmx + AbstractMeasure.jensenShannon));
		put(AbstractMeasure.stringTFIDF, new String(weightingscheme + AbstractMeasure.stringTFIDF));
		put(AbstractMeasure.jaccard, new String(simmetrics + AbstractMeasure.jaccard));
		put(AbstractMeasure.alignmentComparison, new String(vector + AbstractMeasure.alignmentComparison));
		put(AbstractMeasure.LOL, new String(sequence + AbstractMeasure.LOL));
		put(AbstractMeasure.level2, new String(sequence + AbstractMeasure.level2));
		put(AbstractMeasure.dice, new String(simmetrics + AbstractMeasure.dice));
		
		// URI-references for iSPARQL properties (isparql:*) used in iSPARQL queries
		put(StrategyProperty.aggregator, new String(property + StrategyProperty.aggregator));
		put(StrategyProperty.argument1, new String(property + StrategyProperty.argument1));
		put(StrategyProperty.argument2, new String(property + StrategyProperty.argument2));
		put(StrategyProperty.arguments, new String(property + StrategyProperty.arguments));
		put(StrategyProperty.between, new String(property + StrategyProperty.between));
		put(StrategyProperty.ignorecase, new String(property + StrategyProperty.ignorecase));
		put(StrategyProperty.name, new String(property + StrategyProperty.name));
		put(StrategyProperty.ontologyBaseURI, new String(property + StrategyProperty.ontologyBaseURI));
		put(StrategyProperty.ontologyRootURI, new String(property + StrategyProperty.ontologyRootURI));
		put(StrategyProperty.ontologyURI, new String(property + StrategyProperty.ontologyURI));
		put(StrategyProperty.scores, new String(property + StrategyProperty.scores));
		put(StrategyProperty.similarity, new String(property + StrategyProperty.similarity));
		put(StrategyProperty.threshold, new String(property + StrategyProperty.threshold));
		put(StrategyProperty.weights, new String(property + StrategyProperty.weights));
	}
	
	/**
	 * Put a URI-reference to the registry, together with his key,
	 * which is the name of a measure, property or strategy
	 * 
	 * @see ch.unizh.ifi.isparql.query.api.impl.AbstractStrategy
	 * @see ch.unizh.ifi.isparql.query.api.impl.AbstractMeasure
	 * @see ch.unizh.ifi.isparql.query.StrategyProperty
	 * @param key
	 * @param uri
	 */
	public void put(String key, String uri)
	{
		registry.put(key, uri);
	}
	
	/**
	 * Get a URI by an ARQ expression.
	 * An expression can be a variable or a constant
	 * 
	 * @see com.hp.hpl.jena.query.expr.Expr
	 * @param expr
	 * @return String
	 */
	public String get(Expr expr)
	{
		String key = null;
		
		if (expr.isVariable())
			key = expr.getVarName();
		
		if (expr.isConstant());
			key = expr.getConstant().asUnquotedString();
			
		return get(key);
	}
	
	/**
	 * Get the URI by keyword.
	 * The method returns an URINotFoundInRegistry error if 
	 * the key is not found in registry.
	 * 
	 * @param key
	 * @return String
	 */
	public String get(String key)
	{
		if (registry.containsKey(key))
			return registry.get(key);
		else
			new URINotFoundInRegistry("URIRegistry", key);
		
		return null;
	}
	
	/**
	 * The method returns a list of URI-references for isparq:* 
	 * properties of imprecise statements. The list is used during 
	 * optimization to identify (virtual) triple pattern which 
	 * corresponds to an iSPARQL similarity strategy definition.
	 * 
	 * @return List<String>
	 */
	public List<String> getStrategyPropertyURI()
	{
		List<String> properties = new ArrayList<String>();
		
		properties.add(get(StrategyProperty.aggregator));
		properties.add(get(StrategyProperty.argument1));
		properties.add(get(StrategyProperty.argument2));
		properties.add(get(StrategyProperty.arguments));
		properties.add(get(StrategyProperty.between));
		properties.add(get(StrategyProperty.ignorecase));
		properties.add(get(StrategyProperty.name));
		properties.add(get(StrategyProperty.ontologyBaseURI));
		properties.add(get(StrategyProperty.ontologyRootURI));
		properties.add(get(StrategyProperty.ontologyURI));
		properties.add(get(StrategyProperty.similarity));
		properties.add(get(StrategyProperty.threshold));
		properties.add(get(StrategyProperty.scores));
		properties.add(get(StrategyProperty.weights));
		
		return properties;
	}
}
/* 
 * (c) Copyright 2007 University of Zurich, Switzerland
 * All rights reserved.
 *
 * This file is licensed under a GNU Lesser General Public License.
 *
 * For more information about license terms see LICENSE.txt that comes with
 * the main iSPARQL-OptARQ distribution or visit the GNU website at
 * http://www.gnu.org/licenses/lgpl.html
 *
 */
