package ch.unizh.ifi.isparql.query.aggregator;

import ch.unizh.ifi.isparql.query.api.impl.AbstractAggregator;
import java.util.*;

/**
 * Geometric aggregator. Extension of AbstractAggregator.
 * 
 * @see			ch.unizh.ifi.isparql.query.api.impl.AbstractAggregator
 * @author		Markus Stocker
 * @version 	$Id: GeometricAggregator.java,v 1.2 2006-11-21 12:41:57 ms Exp $
 */

public class GeometricAggregator extends AbstractAggregator 
{	
	/** @return String */
	public String getName()
	{
		return geometric;
	}
	
	/** @return double */
	public double defaultValue()
	{
		return 1d;
	}
	
	/**
	 * Aggregate a list of double values 
	 * and return the geometric aggregated value
	 * 
	 * @param scores
	 * @return Double
	 */
	public Double aggregate(List<Double> scores)
	{
		// Add the scores to the optimizer
		optimizer.setAggregatorScores(scores);
		
		double sim = optimizer.lookup();
		
		if (sim == 0d)
			return sim;
		
		for (Double d : scores)
			sim *= d;
		
		return Math.pow(sim, 1 / new Double(scores.size()));
	}
}

/* 
 * (c) Copyright 2007 University of Zurich, Switzerland
 * All rights reserved.
 *
 * This file is licensed under a GNU Lesser General Public License.
 *
 * For more information about license terms see LICENSE.txt that comes with
 * the main iSPARQL-OptARQ distribution or visit the GNU website at
 * http://www.gnu.org/licenses/lgpl.html
 *
 */
