package ch.unizh.ifi.isparql.query.aggregator;

import ch.unizh.ifi.isparql.query.api.impl.AbstractAggregator;
import java.util.*;

/**
 * Median aggregator. Extension of AbstractAggregator.
 * 
 * @see			ch.unizh.ifi.isparql.query.api.impl.AbstractAggregator
 * @author		Markus Stocker
 * @version 	$Id: MedianAggregator.java,v 1.2 2006-11-21 12:41:57 ms Exp $
 */

public class MedianAggregator extends AbstractAggregator 
{
	/** @return String */
	public String getName()
	{
		return median;
	}
	
	/** @return double */
	public double defaultValue()
	{
		return 0d;
	}
	
	/**
	 * Aggregate list of double values.
	 * For even number of elements, median is the mean of the two middle numbers
	 * We can take the Untermedian (size / 2) - 1 or the Obermedian (size / 2)
	 * For odd number of elements, median is simply the middle number.
	 * 
	 * @param scores
	 * @return Double
	 */
	public Double aggregate(List<Double> scores)
	{	
		Collections.sort(scores);

		int index;
		int size = scores.size();
		
		if ((size % 2) == 0)
			index = (size / 2) - 1;
		else
			index = size / 2;
		
		return scores.get(index);
	}
}

/* 
 * (c) Copyright 2007 University of Zurich, Switzerland
 * All rights reserved.
 *
 * This file is licensed under a GNU Lesser General Public License.
 *
 * For more information about license terms see LICENSE.txt that comes with
 * the main iSPARQL-OptARQ distribution or visit the GNU website at
 * http://www.gnu.org/licenses/lgpl.html
 *
 */
