package ch.unizh.ifi.isparql.query.aggregator;

import ch.unizh.ifi.isparql.query.api.impl.AbstractAggregator;
import java.util.*;

/**
 * Min aggregator. Extension of AbstractAggregator.
 * 
 * @see			ch.unizh.ifi.isparql.query.api.impl.AbstractAggregator
 * @author		Markus Stocker
 * @version 	$Id: MinAggregator.java,v 1.2 2006-11-21 12:41:57 ms Exp $
 */


public class MinAggregator extends AbstractAggregator 
{
	/** @return String */
	public String getName()
	{
		return min;
	}
	
	/** @return double */
	public double defaultValue()
	{
		return Double.MAX_VALUE;
	}
	
	/**
	 * Aggregate list of double values
	 * and return the min value found in the list.
	 * 
	 * @param scores
	 * @return Double
	 */
	public Double aggregate(List<Double> scores)
	{
		// Add the scores to the optimizer
		optimizer.setAggregatorScores(scores);
		
		double sim = optimizer.lookup();

		if (sim == 0d)
			return sim;
		
		for (Double d : scores)
		{
			if (d < sim)
				sim = d;
		}
		
		return sim;
	}
}

/* 
 * (c) Copyright 2007 University of Zurich, Switzerland
 * All rights reserved.
 *
 * This file is licensed under a GNU Lesser General Public License.
 *
 * For more information about license terms see LICENSE.txt that comes with
 * the main iSPARQL-OptARQ distribution or visit the GNU website at
 * http://www.gnu.org/licenses/lgpl.html
 *
 */
