package ch.unizh.ifi.isparql.query.api;

import java.util.List;

/**
 * Interface for specific aggregation methods
 * 
 * @see 		ch.unizh.ifi.isparql.query.aggregator
 * @see			ch.unizh.ifi.isparql.query.api.impl.AbstractAggregator
 * @author		Markus Stocker
 * @version 	$Id: IAggregator.java,v 1.3 2006-11-21 18:31:46 ms Exp $
 */

public interface IAggregator 
{
	/**
	 * Aggregator names are defined as public static final strings
	 * in AbstractAggregator. To implement by specific aggregation
	 * methods (aggregator.*).
	 * 
	 * @see ch.unizh.ifi.isparql.query.api.impl.AbstractAggregator
	 * @see ch.unizh.ifi.isparql.query.aggregator
	 * @return String
	 */
	public abstract String getName();
	
	/**
     * To implement by specific aggregation methods (aggregator.*)
     * The purpose is to provide a logic to aggregate similarity
     * scores.
     * 
     * @see ch.unizh.ifi.isparql.query.aggregator
     * @param scores
     * @return Double
     */
	public abstract Double aggregate(List<Double> scores);
	
	/**
	 * To implement by specific aggregation methods (aggregator.*)
	 * The default value is used as init value on aggregation,
	 * usually either 1d (multiplication) or 0d (summation)
	 * 
	 * @see ch.unizh.ifi.isparql.query.aggregator
	 * @return double
	 */
	public abstract double defaultValue();
}
/* 
 * (c) Copyright 2007 University of Zurich, Switzerland
 * All rights reserved.
 *
 * This file is licensed under a GNU Lesser General Public License.
 *
 * For more information about license terms see LICENSE.txt that comes with
 * the main iSPARQL-OptARQ distribution or visit the GNU website at
 * http://www.gnu.org/licenses/lgpl.html
 *
 */
