package ch.unizh.ifi.isparql.query.api.impl;

import ch.unizh.ifi.isparql.query.URIRegistry;
import ch.unizh.ifi.isparql.query.api.IMeasure;
import com.hp.hpl.jena.ontology.OntModelSpec;

/**
 * Abstract measure.
 * Each implemented measure extends this abstract class. This abstract class
 * implements the IMeasure interface.
 *
 * @see			ch.unizh.ifi.isparql.query.api.IMeasure
 * @author		Markus Stocker
 * @version 	$Id: AbstractMeasure.java,v 1.2 2006-11-21 18:31:46 ms Exp $
 */

public abstract class AbstractMeasure implements IMeasure
{
	/** The URI-reference to identify the measure */
	private String uri = null;
	/** The measure name */
	private String name = null;
	/** 
	 * The strategy which is assigned to the measure.
	 * A measure requires the strategy, because it uses the
	 * strategy configuration (for examples, isparq:arguments
	 * are extracted from the strategy and used in a measure
	 * object to compute the similarity.
	 */
	protected AbstractStrategy strategy = null;
	/** The registry for iSPARQL specific URI-references */
	protected URIRegistry urireg = new URIRegistry();
	/** Jena stuff */
	protected OntModelSpec ontModelSpec = OntModelSpec.OWL_MEM_RDFS_INF;
	
	/** The name for the Levenshtein measure, levenshtein */
	public static final String levenshtein = new String("levenshtein");
	/** The name for the Resnik measure, resnik */
	public static final String resnik = new String("resnik");
	/** The name for the Lin measure, lin */
	public static final String lin = new String("lin");
	/** The name for the shortest path measure, shortestPath */
	public static final String shortestPath = new String("shortestPath");
	/** The name for the sub graph isomorphism, subGraphIsomorphism */
	public static final String subGraphIsomorphism = new String("subGraphIsomorphism");
	/** The name for the tree edit distance measure, treeEditDistance */
	public static final String treeEditDistance = new String("treeEditDistance");
	/** The name for the cosine measure, cosine */
	public static final String cosine = new String("cosine");
	/** The name for the Jensen Shannon measure, jensenShannon */
	public static final String jensenShannon = new String("jensenShannon");
	/** The name for the TFIDF measure, stringTFIDF */
	public static final String stringTFIDF = new String("stringTFIDF");
	/** The name for the Jaccard measure, jaccard */
	public static final String jaccard = new String("jaccard");
	/** The name for the alignment comparison, alignmentComparison */
	public static final String alignmentComparison = new String("alignmentComparison");
	/** The name for the Levenshtein of Levenshtein measure, LOL */
	public static final String LOL = new String("LOL");
	/** The name for the Level2 measure, level2 */
	public static final String level2 = new String("level2");
	/** The name for the Dice measure, dice */
	public static final String dice = new String("dice");
	
	/** 
	 * Used to specify an absolute complexity for the measure.
	 * This feature is not used yet, but may be used for future
	 * optimization techniques.
	 */
	public static int complexity;
	
	/** @param value */
	public void setURI(String value)
	{
		uri = value;
	}
	
	/** @return String */
	public String getURI()
	{
		return uri;
	}
	
	/** @param value */
	public void setName(String value)
	{
		name = value;
	}
	
	/** @return String */
	public String getName()
	{
		return name;
	}
}
/* 
 * (c) Copyright 2007 University of Zurich, Switzerland
 * All rights reserved.
 *
 * This file is licensed under a GNU Lesser General Public License.
 *
 * For more information about license terms see LICENSE.txt that comes with
 * the main iSPARQL-OptARQ distribution or visit the GNU website at
 * http://www.gnu.org/licenses/lgpl.html
 *
 */
