package ch.unizh.ifi.isparql.query.measure.graph;

import ch.unizh.ifi.isparql.query.api.impl.AbstractStrategy;
import ch.unizh.ifi.isparql.query.api.impl.AbstractMeasure;
import com.hp.hpl.jena.graph.Node;
import simpack.accessor.graph.JenaOntologyAccessor;
import simpack.exception.InvalidElementException;
import simpack.util.conversion.CommonDistanceConversion;
import simpack.util.graph.GraphNode;
import simpack.measure.graph.ShortestPath;

/**
 * SimPack wrapper for ShortestPath similarity measure.
 * The class extends the AbstractMeasure.
 *
 * @see			ch.unizh.ifi.isparql.query.api.impl.AbstractMeasure
 * @see 		simpack.measure.graph.ShortestPath
 * @author		Markus Stocker
 * @version 	$Id: MeasureShortestPath.java,v 1.2 2006-11-21 18:31:46 ms Exp $
 */

public class MeasureShortestPath extends AbstractMeasure 
{
	public static final int complexity = 1;
	public static final String altOntologyURL = "";
	
	/**
	 * Constructor, set the URI and name for the measure.
	 * The strategy which invoques the wrapper is given
	 * as param, to get the strategy configuration which
	 * is relevant for the measure computation.
	 * 
	 * @param strategy
	 */
	public MeasureShortestPath(AbstractStrategy strategy) 
	{
		this.strategy = strategy;
		
		setURI(urireg.get(shortestPath));
		setName(shortestPath);
	}
	
	/**
	 * Compute the similarity, using the SimPack ShortestPath class
	 * 
	 * @return Double
	 */
	public Double similarity()
	{
		Node node1 = strategy.getArgument1();
		Node node2 = strategy.getArgument2();
		String ontologyURI = strategy.getOntologyURI1();
		String ontologyBaseURI = strategy.getOntologyBaseURI1();
		String ontologyRootURI = strategy.getOntologyRootURI1();
		
		try {
			ShortestPath measure = new ShortestPath(
										new JenaOntologyAccessor(
												ontologyURI,
												altOntologyURL,
												ontologyBaseURI, 
												ontologyRootURI, 
												ontModelSpec
										),
										new GraphNode(node1.toString(false)), 
										new GraphNode(node2.toString(false)),
										new CommonDistanceConversion());
			measure.calculate();
			return measure.getSimilarity();
		} catch (InvalidElementException e) {
			e.printStackTrace();
		}
		
		return null;
	}
}
/* 
 * (c) Copyright 2007 University of Zurich, Switzerland
 * All rights reserved.
 *
 * This file is licensed under a GNU Lesser General Public License.
 *
 * For more information about license terms see LICENSE.txt that comes with
 * the main iSPARQL-OptARQ distribution or visit the GNU website at
 * http://www.gnu.org/licenses/lgpl.html
 *
 */
