package ch.unizh.ifi.isparql.query.measure.graph;

import ch.unizh.ifi.isparql.query.api.impl.AbstractStrategy;
import ch.unizh.ifi.isparql.query.api.impl.AbstractMeasure;
import simpack.accessor.graph.JenaOntologyAccessor;
import simpack.measure.graph.SubgraphIsomorphism;

/**
 * SimPack wrapper for SubgraphIsomorphism similarity measure.
 * The class extends the AbstractMeasure.
 *
 * @see			ch.unizh.ifi.isparql.query.api.impl.AbstractMeasure
 * @see 		simpack.measure.graph.SubgraphIsomorphism
 * @author		Markus Stocker
 * @version 	$Id: MeasureSubgraphIsomorphism.java,v 1.2 2006-11-21 18:31:46 ms Exp $
 */

public class MeasureSubgraphIsomorphism extends AbstractMeasure 
{
	public static final int complexity = 1;
	public static final String altOntologyURL = "";
	
	/**
	 * Constructor, set the URI and name for the measure.
	 * The strategy which invoques the wrapper is given
	 * as param, to get the strategy configuration which
	 * is relevant for the measure computation.
	 * 
	 * @param strategy
	 */
	public MeasureSubgraphIsomorphism(AbstractStrategy strategy) 
	{
		this.strategy = strategy;
		
		setURI(urireg.get(subGraphIsomorphism));
		setName(subGraphIsomorphism);
	}
	
	/**
	 * Calculate the similarity
	 * 
	 * @return Double
	 */
	public Double similarity()
	{
		String ontologyURI1 = strategy.getOntologyURI1();
		String ontologyBaseURI1 = strategy.getOntologyBaseURI1();
		String ontologyRootURI1 = strategy.getOntologyRootURI1();
		String ontologyURI2 = strategy.getOntologyURI2();
		String ontologyBaseURI2 = strategy.getOntologyBaseURI2();
		String ontologyRootURI2 = strategy.getOntologyRootURI2();
		
		JenaOntologyAccessor subGraph1 = new JenaOntologyAccessor(ontologyURI1,
																  altOntologyURL,
											 					  ontologyBaseURI1,
											 					  ontologyRootURI1, 
											 					  ontModelSpec);
		
		JenaOntologyAccessor subGraph2 = new JenaOntologyAccessor(ontologyURI2,
																  altOntologyURL,
											   					  ontologyBaseURI2,
											   					  ontologyRootURI2, 
											   					  ontModelSpec);
		
		SubgraphIsomorphism measure = new SubgraphIsomorphism(subGraph1,
															  subGraph2, 
															  7, 
															  SubgraphIsomorphism.DEFAULT_LABEL_WEIGHT,
															  SubgraphIsomorphism.DEFAULT_STRUCTURE_WEIGHT,
															  SubgraphIsomorphism.DEFAULT_DENOMINATOR, 
															  true);
		
		return measure.getSimilarity();
	}
}
/* 
 * (c) Copyright 2007 University of Zurich, Switzerland
 * All rights reserved.
 *
 * This file is licensed under a GNU Lesser General Public License.
 *
 * For more information about license terms see LICENSE.txt that comes with
 * the main iSPARQL-OptARQ distribution or visit the GNU website at
 * http://www.gnu.org/licenses/lgpl.html
 *
 */
