package ch.unizh.ifi.isparql.query.measure.it;

import ch.unizh.ifi.isparql.query.api.impl.AbstractStrategy;
import ch.unizh.ifi.isparql.query.api.impl.AbstractMeasure;
import simpack.measure.it.Lin;
import simpack.util.graph.GraphNode;
import simpack.accessor.graph.JenaOntologyAccessor;
import com.hp.hpl.jena.graph.Node;
import com.hp.hpl.jena.query.expr.ExprEvalException;

/**
 * SimPack wrapper for Lin similarity measure.
 * The class extends the AbstractMeasure.
 *
 * @see			ch.unizh.ifi.isparql.query.api.impl.AbstractMeasure
 * @see 		simpack.measure.it.Lin
 * @author		Markus Stocker
 * @version 	$Id: MeasureLin.java,v 1.2 2006-11-21 18:31:46 ms Exp $
 */

public class MeasureLin extends AbstractMeasure 
{	
	public static final int complexity = 1;
	
	/**
	 * Constructor, set the URI and name for the measure.
	 * The strategy which invoques the wrapper is given
	 * as param, to get the strategy configuration which
	 * is relevant for the measure computation.
	 * 
	 * @param strategy
	 */
	public MeasureLin(AbstractStrategy strategy) 
	{
		this.strategy = strategy;
		
		setURI(urireg.get(lin));
		setName(lin);
	}
	
	/**
	 * Calculate the similarity, using the SimPack Lin class
	 * 
	 * @return Double
	 */
	public Double similarity()
	{
		Node node1 = strategy.getArgument1();
		Node node2 = strategy.getArgument2();
		String ontologyURI = strategy.getOntologyURI1();
		String ontologyBaseURI = strategy.getOntologyBaseURI1();
		String ontologyRootURI = strategy.getOntologyRootURI1();
		String ontologyAltURI = strategy.getOntologyAltURI();
		
		try {
			Lin measure = new Lin(
								new JenaOntologyAccessor(
										ontologyURI,
										ontologyAltURI,
										ontologyBaseURI, 
										ontologyRootURI, 
										ontModelSpec
								),
								new GraphNode(node1.toString(false)), 
								new GraphNode(node2.toString(false)));
			measure.calculate();
			return measure.getSimilarity();
		} catch (Exception e) {
			throw new ExprEvalException("Error fetching Last-Modified", e);
		}
	}
}
/* 
 * (c) Copyright 2007 University of Zurich, Switzerland
 * All rights reserved.
 *
 * This file is licensed under a GNU Lesser General Public License.
 *
 * For more information about license terms see LICENSE.txt that comes with
 * the main iSPARQL-OptARQ distribution or visit the GNU website at
 * http://www.gnu.org/licenses/lgpl.html
 *
 */
