package ch.unizh.ifi.isparql.query.measure.owlsmx;

import ch.unizh.ifi.isparql.query.api.impl.AbstractStrategy;
import ch.unizh.ifi.isparql.query.api.impl.AbstractMeasure;
import simpack.measure.external.owlsmx.JensenShannonMeasure;

/**
 * SimPack wrapper for JensenShannon similarity measure.
 * The class extends the AbstractMeasure.
 *
 * @see			ch.unizh.ifi.isparql.query.api.impl.AbstractMeasure
 * @see 		simpack.measure.external.owlsmx.JensenShannonMeasure
 * @author		Markus Stocker
 * @version 	$Id: MeasureJensenShannon.java,v 1.2 2006-11-21 18:31:46 ms Exp $
 */

public class MeasureJensenShannon extends AbstractMeasure 
{
	public static final int complexity = 1;
	
	/**
	 * Constructor, set the URI and name for the measure.
	 * The strategy which invoques the wrapper is given
	 * as param, to get the strategy configuration which
	 * is relevant for the measure computation.
	 * 
	 * @param strategy
	 */
	public MeasureJensenShannon(AbstractStrategy strategy) 
	{
		this.strategy = strategy;
		
		setURI(urireg.get(jensenShannon));
		setName(jensenShannon);
	}
	
	/**
	 * Calculate the similarity, using the SimPack JensenShannonMeasure class
	 * 
	 * @return Double
	 */
	public Double similarity()
	{		
		return similarity(strategy.getArgument1().toString(false),
						  strategy.getArgument2().toString(false));
	}
	
	/**
	 * Calculate the similarity, using the SimPack JensenShannonMeasure class
	 * 
	 * @param argument1
	 * @param argument2
	 * @return Double
	 */
	public Double similarity(String argument1, String argument2)
	{	
		JensenShannonMeasure measure = new JensenShannonMeasure(argument1, argument2);
		measure.calculate();
		
		return measure.getSimilarity();
	}
}
/* 
 * (c) Copyright 2007 University of Zurich, Switzerland
 * All rights reserved.
 *
 * This file is licensed under a GNU Lesser General Public License.
 *
 * For more information about license terms see LICENSE.txt that comes with
 * the main iSPARQL-OptARQ distribution or visit the GNU website at
 * http://www.gnu.org/licenses/lgpl.html
 *
 */
