package ch.unizh.ifi.isparql.query.measure.secondstring;

import ch.unizh.ifi.isparql.query.api.impl.AbstractStrategy;
import ch.unizh.ifi.isparql.query.api.impl.AbstractMeasure;
import com.hp.hpl.jena.graph.Node;
import simpack.measure.external.secondstring.Level2;
import simpack.accessor.string.StringAccessor;
import com.wcohen.ss.Levenstein;
import com.wcohen.ss.tokens.SimpleTokenizer;

/**
 * SimPack wrapper for Level2 similarity measure.
 * The class extends the AbstractMeasure.
 *
 * @see			ch.unizh.ifi.isparql.query.api.impl.AbstractMeasure
 * @see 		simpack.measure.external.secondstring.Level2
 * @author		Markus Stocker
 * @version 	$Id: MeasureLevel2.java,v 1.2 2006-11-21 18:31:46 ms Exp $
 */

public class MeasureLevel2 extends AbstractMeasure 
{
	public static final int complexity = 1;
	
	/**
	 * Constructor, set the URI and name for the measure.
	 * The strategy which invoques the wrapper is given
	 * as param, to get the strategy configuration which
	 * is relevant for the measure computation.
	 * 
	 * @param strategy
	 */
	public MeasureLevel2(AbstractStrategy strategy) 
	{
		this.strategy = strategy;
		
		setURI(urireg.get(level2));
		setName(level2);
	}
	
	/**
	 * Compute the similarity, using the SimPack Level2 class
	 * 
	 * @return Double
	 */
	public Double similarity()
	{		
		Node argument1 = strategy.getArgument1();
		Node argument2 = strategy.getArgument2();
		
		// Make sure to remove XSD datatype information
		if (argument1.isLiteral())
			return similarity(argument1.getLiteralValue().toString(),
							  argument2.getLiteralValue().toString());
		else
			return similarity(argument1.getURI(),
							  argument2.getURI());
	}
	
	/**
	 * Compute the similarity, using the SimPack Level2 class
	 * 
	 * @param argument1
	 * @param argument2
	 * @return Double
	 */
	public Double similarity(String argument1, String argument2)
	{
		
		StringAccessor sa1 = new StringAccessor(argument1);
		StringAccessor sa2 = new StringAccessor(argument2);

		Level2 measure = new Level2(sa1, sa2, SimpleTokenizer.DEFAULT_TOKENIZER, new Levenstein());

		return measure.getSimilarity();
	}
}
/* 
 * (c) Copyright 2007 University of Zurich, Switzerland
 * All rights reserved.
 *
 * This file is licensed under a GNU Lesser General Public License.
 *
 * For more information about license terms see LICENSE.txt that comes with
 * the main iSPARQL-OptARQ distribution or visit the GNU website at
 * http://www.gnu.org/licenses/lgpl.html
 *
 */
