package ch.unizh.ifi.isparql.query.measure.sequence;

import ch.unizh.ifi.isparql.query.api.impl.AbstractStrategy;
import ch.unizh.ifi.isparql.query.api.impl.AbstractMeasure;
import simpack.accessor.string.StringAccessor;
import simpack.measure.sequence.LOL;
import simpack.tokenizer.SplittedStringTokenizer;
import simpack.util.corpus.StringUtils;

/**
 * SimPack wrapper for LOL similarity measure.
 * The class extends the AbstractMeasure.
 *
 * @see			ch.unizh.ifi.isparql.query.api.impl.AbstractMeasure
 * @see 		simpack.measure.sequence.LOL
 * @author		Markus Stocker
 * @version 	$Id: MeasureLOL.java,v 1.2 2006-11-21 18:31:46 ms Exp $
 */

public class MeasureLOL extends AbstractMeasure 
{
	public static final int complexity = 1;
	
	/**
	 * Constructor, set the URI and name for the measure.
	 * The strategy which invoques the wrapper is given
	 * as param, to get the strategy configuration which
	 * is relevant for the measure computation.
	 * 
	 * @param strategy
	 */
	public MeasureLOL(AbstractStrategy strategy) 
	{
		this.strategy = strategy;
		
		setURI(urireg.get(LOL));
		setName(LOL);
	}
	
	/**
	 * Compute the similarity, using the SimPack LOL class
	 * 
	 * @return Double
	 */
	public Double similarity()
	{		
		return similarity(strategy.getArgument1().toString(false),
						  strategy.getArgument2().toString(false));
	}
	
	/**
	 * Compute the similarity, using the SimPack LOL class
	 * 
	 * @param argument1
	 * @param argument2
	 * @return Double
	 */
	public Double similarity(String argument1, String argument2)
	{
		//Boolean ignorecase = strategy.getIgnorecase();
		StringAccessor sa1 = new StringAccessor(StringUtils.clean(argument1), new SplittedStringTokenizer("\\s+"));
		StringAccessor sa2 = new StringAccessor(StringUtils.clean(argument2), new SplittedStringTokenizer("\\s+"));

		LOL<String> measure = new LOL<String>(sa1, sa2);
		
		return measure.getSimilarity();
	}
}
/* 
 * (c) Copyright 2007 University of Zurich, Switzerland
 * All rights reserved.
 *
 * This file is licensed under a GNU Lesser General Public License.
 *
 * For more information about license terms see LICENSE.txt that comes with
 * the main iSPARQL-OptARQ distribution or visit the GNU website at
 * http://www.gnu.org/licenses/lgpl.html
 *
 */
