package ch.unizh.ifi.isparql.query.measure.simmetrics;

import ch.unizh.ifi.isparql.query.api.impl.AbstractStrategy;
import ch.unizh.ifi.isparql.query.api.impl.AbstractMeasure;
import simmetrics.tokenisers.TokeniserWhitespace;
import simpack.measure.external.simmetrics.DiceSimilarity;
import simpack.util.corpus.StringUtils;

/**
 * SimPack wrapper for DicdeSimilarity similarity measure.
 * The class extends the AbstractMeasure.
 *
 * @see			ch.unizh.ifi.isparql.query.api.impl.AbstractMeasure
 * @see 		simpack.measure.external.simmetrics.DiceSimilarity
 * @author		Markus Stocker
 * @version 	$Id: MeasureDice.java,v 1.2 2006-11-21 18:31:46 ms Exp $
 */

public class MeasureDice extends AbstractMeasure 
{
	public static final int complexity = 1;

	/**
	 * Constructor, set the URI and name for the measure.
	 * The strategy which invoques the wrapper is given
	 * as param, to get the strategy configuration which
	 * is relevant for the measure computation.
	 * 
	 * @param strategy
	 */
	public MeasureDice(AbstractStrategy strategy) 
	{
		this.strategy = strategy;

		setURI(urireg.get(dice));
		setName(dice);
	}

	/**
	 * Compute the similarity, using the SimPack DiceSimilarity class
	 * 
	 * @return Double
	 */
	public Double similarity() 
	{
		return similarity(strategy.getArgument1().toString(false), 
						  strategy.getArgument2().toString(false));
	}

	/**
	 * Compute the similarity, using the SimPack DiceSimilarity class
	 * 
	 * @param argument1
	 * @param argument2
	 * @return Double
	 */
	public Double similarity(String argument1, String argument2) 
	{
		TokeniserWhitespace tokeniser = new TokeniserWhitespace();
		DiceSimilarity measure = new DiceSimilarity(StringUtils.clean(argument1), 
													StringUtils.clean(argument2), 
													tokeniser);

		return measure.getSimilarity();
	}
}
/* 
 * (c) Copyright 2007 University of Zurich, Switzerland
 * All rights reserved.
 *
 * This file is licensed under a GNU Lesser General Public License.
 *
 * For more information about license terms see LICENSE.txt that comes with
 * the main iSPARQL-OptARQ distribution or visit the GNU website at
 * http://www.gnu.org/licenses/lgpl.html
 *
 */
