package ch.unizh.ifi.isparql.query.measure.tree;

import ch.unizh.ifi.isparql.query.api.impl.AbstractStrategy;
import ch.unizh.ifi.isparql.query.api.impl.AbstractMeasure;
import simpack.api.ITreeNode;
import simpack.measure.tree.TreeEditDistance;
import simpack.accessor.tree.SimpleTreeAccessor;
import simpack.exception.InvalidElementException;
import simpack.util.conversion.WorstCaseDistanceConversion;
import simpack.util.tree.comparator.AlwaysTrueTreeNodeComparator;

/**
 * SimPack wrapper for TreeEditDistance similarity measure.
 * The class extends the AbstractMeasure.
 * We compare the similarity of tree nodes. The similarity strategy has to construct
 * the tree with a specific logic (eg, create trees of specific predicates)
 * Thus, the strategy invoking this measure has to provide both ITreeNodes
 *
 * @see			ch.unizh.ifi.isparql.query.api.impl.AbstractMeasure
 * @see 		simpack.measure.tree.TreeEditDistance
 * @author		Markus Stocker
 * @version 	$Id: MeasureTreeEditDistance.java,v 1.2 2006-11-21 18:31:46 ms Exp $
 */

public class MeasureTreeEditDistance extends AbstractMeasure 
{	
	private ITreeNode tree1 = null;
	private ITreeNode tree2 = null;
	
	public static final int complexity = 1;
	
	/**
	 * Constructor, set the URI and name for the measure.
	 * The strategy which invoques the wrapper is given
	 * as param, to get the strategy configuration which
	 * is relevant for the measure computation.
	 * 
	 * @param strategy
	 */
	public MeasureTreeEditDistance(AbstractStrategy strategy) 
	{
		this.strategy = strategy;
		
		setURI(urireg.get(treeEditDistance));
		setName(treeEditDistance);
	}
	
	/**
	 * Compute the similarity, using the SimPack TreeEditDistance class
	 * 
	 * @return Double
	 */
	public Double similarity()
	{	
		// Check whether the trees are provided by the strategy
		if (tree1 == null || tree2 == null)
			return 0d;
		
		try {
			TreeEditDistance measure = new TreeEditDistance(new SimpleTreeAccessor(tree1),
															new SimpleTreeAccessor(tree2),
															new AlwaysTrueTreeNodeComparator(),
															new WorstCaseDistanceConversion());
			measure.calculate();
			return measure.getSimilarity();
		} catch (NullPointerException e) {
			e.printStackTrace();
		} catch (InvalidElementException e) {
			e.printStackTrace();
		}
		
		return 0d;
	}
	
	/**
	 * Allows setting the first tree
	 *  
	 * @param tree 
	 */
	public void setTree1(ITreeNode tree)
	{
		tree1 = tree;
	}
	
	/** 
	 * Allows setting the second tree
	 * 
	 * @param tree 
	 */
	public void setTree2(ITreeNode tree)
	{
		tree2 = tree;
	}
}
/* 
 * (c) Copyright 2007 University of Zurich, Switzerland
 * All rights reserved.
 *
 * This file is licensed under a GNU Lesser General Public License.
 *
 * For more information about license terms see LICENSE.txt that comes with
 * the main iSPARQL-OptARQ distribution or visit the GNU website at
 * http://www.gnu.org/licenses/lgpl.html
 *
 */
