package ch.unizh.ifi.isparql.query.measure.vector;

import ch.unizh.ifi.isparql.query.api.impl.AbstractStrategy;
import ch.unizh.ifi.isparql.query.api.impl.AbstractMeasure;
import simpack.measure.vector.AlignmentComparison;
import java.util.*;

/**
 * SimPack wrapper for AlignmentComparison similarity measure.
 * The class extends the AbstractMeasure.
 *
 * @see			ch.unizh.ifi.isparql.query.api.impl.AbstractMeasure
 * @see 		simpack.measure.vector.AlignmentComparison
 * @author		Markus Stocker
 * @version 	$Id: MeasureAlignmentComparison.java,v 1.2 2006-11-21 18:31:46 ms Exp $
 */

public class MeasureAlignmentComparison extends AbstractMeasure 
{	
	private Map<Object, Object> argument1 = new HashMap<Object, Object>();
	private Map<Object, Object> argument2 = new HashMap<Object, Object>();
	
	public static final int complexity = 1;
	
	/**
	 * Constructor, set the URI and name for the measure.
	 * The strategy which invoques the wrapper is given
	 * as param, to get the strategy configuration which
	 * is relevant for the measure computation.
	 * 
	 * @param strategy
	 */
	public MeasureAlignmentComparison(AbstractStrategy strategy) 
	{
		this.strategy = strategy;
		
		setURI(urireg.get(stringTFIDF));
		setName(stringTFIDF);
	}
	
	/**
	 * Compute the similarity, using the SimPack AlignmentComparison class
	 * 
	 * @return Double
	 */
	public Double similarity()
	{
		return similarity(argument1, argument2);
	}
	
	/**
	 * Compute the similarity, using the SimPack AlignmentComparison class
	 * 
	 * @param argument1
	 * @param argument2
	 * @return Double
	 */
	public Double similarity(Map<Object, Object> argument1, Map<Object, Object> argument2)
	{	
		AlignmentComparison measure = new AlignmentComparison(argument1, argument2, "Levenshtein", 0.75d);
		
		return measure.getSimilarity();
	}
	
	/**
	 * Allows setting the first alignment map
	 * 
	 * @param value
	 */
	public void setArgument1(Map<Object, Object> value)
	{
		argument1 = value;
	}
	
	/**
	 * Allows setting the second alignment map
	 * 
	 * @param value
	 */
	public void setArgument2(Map<Object, Object> value)
	{
		argument2 = value;
	}
}
/* 
 * (c) Copyright 2007 University of Zurich, Switzerland
 * All rights reserved.
 *
 * This file is licensed under a GNU Lesser General Public License.
 *
 * For more information about license terms see LICENSE.txt that comes with
 * the main iSPARQL-OptARQ distribution or visit the GNU website at
 * http://www.gnu.org/licenses/lgpl.html
 *
 */
