package ch.unizh.ifi.isparql.query.optimizer;

import ch.unizh.ifi.isparql.query.api.impl.AbstractOptimizer;
import ch.unizh.ifi.isparql.query.api.impl.AbstractAggregator;
import java.util.*;

/**
 * iSPARQL aggregation scores optimizer. This class is not really used
 * and we never evaluated the performance of this optimization.
 * The class extends AbstractOptimizer.
 *
 * @see			ch.unizh.ifi.isparql.query.api.impl.AbstractOptimizer
 * @author		Markus Stocker
 * @version 	$Id: AggregatorOptimizer.java,v 1.2 2006-11-21 18:31:46 ms Exp $
 */

public class AggregatorOptimizer extends AbstractOptimizer
{
	private String name;
	private double defaultValue;
	private List<Double> scores;
	
	/**
	 * Interface implementation. Perform the optimizations for 
	 * specific aggregators (only a few have a optimization 
	 * characteristic. The optimization is executed only
	 * if the optimizer is activated (AbstractOptimizer).
	 * Before this method can be executed, the aggregation method
	 * name (setAggregatorName()), the default value (setAggregatorDefaultValue())
	 * and the scores (setAggregatorScores()) must be set to the 
	 * aggregator optimizer (ie, this class).
	 * 
	 * @see ch.unizh.ifi.isparql.query.api.impl.AbstractOptimizer
	 * @return Double
	 */
	public Double lookup()
	{
		if (optimizer)
		{
			if (name.equals(AbstractAggregator.geometric))
				return geometric();
			if (name.equals(AbstractAggregator.max))
				return max();
			if (name.equals(AbstractAggregator.min))
				return min();
			if (name.equals(AbstractAggregator.noisyand))
				return noisyand();
		}
		
		return defaultValue;
	}
	
	// Optimization for geometric aggregation method
	private double geometric()
	{		
		if (scores.contains(0d))
			return 0d;
		
		return defaultValue;
	}
	
	// Optimization for max aggregation method
	private double max()
	{
		Collections.sort(scores);
		
		return scores.get(scores.size() - 1);
	}
	
	// Optimization for min aggregation method
	private double min()
	{
		Collections.sort(scores);
		
		return scores.get(0);
	}
	
	// Optimization for noisy-and aggregation method
	private double noisyand()
	{
		if (scores.contains(0d))
			return 0d;
		
		return defaultValue;
	}
	
	/**
	 * The method allows setting the aggregation method name.
	 * This is one defined in AbstractAggregator. Set the 
	 * aggregation method name before executing the lookup().
	 * 
	 * @see ch.unizh.ifi.isparql.query.api.impl.AbstractAggregator
	 * @param value
	 */
	public void setAggregatorName(String value)
	{
		name = value;
	}
	
	/**
	 * The method allows setting the aggregation method default
	 * value. The specific aggregator implements a method which
	 * specifies the default value.
	 * 
	 * @see ch.unizh.ifi.isparql.query.aggregator
	 * @param value
	 */
	public void setAggregatorDefaultValue(double value)
	{
		defaultValue = value;
	}
	
	/**
	 * The method allows setting the list of scores which
	 * are aggregated. This list is required to check
	 * whether we may return the optimized value or not.
	 * 
	 * @param value
	 */
	public void setAggregatorScores(List<Double> value)
	{
		scores = value;
	}
}/* 
 * (c) Copyright 2007 University of Zurich, Switzerland
 * All rights reserved.
 *
 * This file is licensed under a GNU Lesser General Public License.
 *
 * For more information about license terms see LICENSE.txt that comes with
 * the main iSPARQL-OptARQ distribution or visit the GNU website at
 * http://www.gnu.org/licenses/lgpl.html
 *
 */
