package ch.unizh.ifi.isparql.query.optimizer;

import ch.unizh.ifi.isparql.query.api.impl.AbstractOptimizer;
import java.util.*;

/**
 * iSPARQL similarity index. Similarity index hash (in-memory)
 *
 * @author		Markus Stocker
 * @version 	$Id: SimilarityIndexMem.java,v 1.2 2006-11-21 18:31:46 ms Exp $
 */

public class SimilarityIndexMem extends AbstractOptimizer
{
	private Double similarity = null;
	/** The index, a hash map of hashCodes with similarity values */
	private Map<Integer, Double> index = new HashMap<Integer, Double>();

	/**
	 * Interface implementation.
	 * 
	 * @see ch.unizh.ifi.isparql.query.api.IOptimizer
	 * @return Double
	 */
	public Double lookup()
	{
		return lookup(null, null, null);
	}
	
	/**
	 * Interface implementation.
	 * 
	 * @see ch.unizh.ifi.isparql.query.api.IOptimizer
	 * @param argument1
	 * @param argument2
	 * @param strategy
	 * @return Double
	 */
	public Double lookup(String argument1, String argument2, String strategy)
	{	
		if (!optimizer || argument1 == null || argument2 == null)
			return similarity;
		
		Integer key = (argument1 + argument2 + strategy).toString().hashCode();
		
		return index.get(key);
	}
	
	/**
	 * The method allows adding a new similarity score to the index.
	 * The relevant arguments (isparql:arguments) and the strategy (isparql:name)
	 * used to compare the arguments are used to create a hash code,
	 * which is used as key in the index hash map.
	 * 
	 * @param argument1
	 * @param argument2
	 * @param strategy
	 * @param value
	 */
	public void addIndex(String argument1, String argument2, String strategy, Double value)
	{
		Integer key = (argument1 + argument2 + strategy).toString().hashCode();
		
		index.put(key, value);
	}
	
	/**
	 * The method returns the size of the index.
	 * 
	 * @return int
	 */
	public int size()
	{
		return index.size();
	}
}/* 
 * (c) Copyright 2007 University of Zurich, Switzerland
 * All rights reserved.
 *
 * This file is licensed under a GNU Lesser General Public License.
 *
 * For more information about license terms see LICENSE.txt that comes with
 * the main iSPARQL-OptARQ distribution or visit the GNU website at
 * http://www.gnu.org/licenses/lgpl.html
 *
 */
