package ch.unizh.ifi.isparql.query.property;

import ch.unizh.ifi.isparql.query.Constants;
import ch.unizh.ifi.isparql.query.StrategyRegistry;
import ch.unizh.ifi.isparql.query.api.impl.AbstractStrategy;
import ch.unizh.ifi.isparql.query.exception.ObjectIsNotList;
import ch.unizh.ifi.isparql.query.exception.ArgListSizeError;
import com.hp.hpl.jena.graph.Node;
import com.hp.hpl.jena.query.core.Binding;
import com.hp.hpl.jena.query.engine.QueryIterator;
import com.hp.hpl.jena.query.engine1.ExecutionContext;
import com.hp.hpl.jena.query.engine1.iterator.QueryIterPlainWrapper;
import com.hp.hpl.jena.query.util.Symbol;
import com.hp.hpl.jena.query.pfunction.PropFuncArg;
import com.hp.hpl.jena.query.pfunction.PropertyFunctionBase;
import java.util.*;

/**
 * Between property (isparql:between)
 *
 * @author		Markus Stocker
 * @version 	$Id: between.java,v 1.2 2006-11-21 18:31:46 ms Exp $
 */

public class between extends PropertyFunctionBase 
{	
	/**
	 * Override the ARQ method. We execute the logic
	 * when a triple pattern containing isparql:between
	 * is identified in iSPARQL query.
	 * 
	 * @param binding
	 * @param subject
	 * @param predicate
	 * @param object
	 * @param execCxt
	 * @return QueryIterator
	 */
	public QueryIterator exec(Binding binding, PropFuncArg subject, Node predicate, PropFuncArg object, ExecutionContext execCxt)
	{	
		if (! object.isList()) 
			throw new ObjectIsNotList("Between property needs a list as object");
		if (object.getArgList().size() != 2)
			throw new ArgListSizeError("Between property needs two arguments as object");
		
		List<Binding> bindings = new ArrayList<Binding>();
		StrategyRegistry resourcereg = (StrategyRegistry)execCxt.getContext().get(new Symbol(Constants.STRATEGY_REGISTRY));
		
		// Get the strategy from the registry
		subject = subject.evalIfExists(binding);
		String uri = subject.getArg().toString();
		AbstractStrategy strategy = (AbstractStrategy)resourcereg.get(uri);
		
		// Set bounds to strategy
		object = object.evalIfExists(binding);
		String lowerBound = object.getArg(0).toString(false);
		String upperBound = object.getArg(1).toString(false);
		
		// In case the object is written with (0.6 0.8) instead of ("0.6" "0.8")
		// we need to strip of the XMLSchema#decimal type
		if (lowerBound.indexOf("^^") > 0)
			lowerBound = lowerBound.substring(0, lowerBound.indexOf("^^"));
		if (upperBound.indexOf("^^") > 0)
			upperBound = upperBound.substring(0, upperBound.indexOf("^^"));
		
		Double lb = new Double(lowerBound);
		Double ub = new Double(upperBound);
		
		if (lb <= ub)
		{
			strategy.setLowerBound(lb);
			strategy.setUpperBound(ub);
		} else 
		{
			strategy.setLowerBound(ub);
			strategy.setUpperBound(lb);
		}
		
		// For between property, there is nothing to bind, add old bindings to list
		bindings.add(binding);
	 
		return new QueryIterPlainWrapper(bindings.iterator(), execCxt);
	}
}
/* 
 * (c) Copyright 2007 University of Zurich, Switzerland
 * All rights reserved.
 *
 * This file is licensed under a GNU Lesser General Public License.
 *
 * For more information about license terms see LICENSE.txt that comes with
 * the main iSPARQL-OptARQ distribution or visit the GNU website at
 * http://www.gnu.org/licenses/lgpl.html
 *
 */
