package ch.unizh.ifi.isparql.query.property;

import java.util.*;
import ch.unizh.ifi.isparql.query.Constants;
import ch.unizh.ifi.isparql.query.StrategyFactory;
import ch.unizh.ifi.isparql.query.StrategyProperty;
import ch.unizh.ifi.isparql.query.StrategyRegistry;
import ch.unizh.ifi.isparql.query.URIRegistry;
import ch.unizh.ifi.isparql.query.api.impl.AbstractStrategy;
import com.hp.hpl.jena.query.util.Symbol;
import com.hp.hpl.jena.query.core.Binding;
import com.hp.hpl.jena.query.engine.QueryIterator;
import com.hp.hpl.jena.query.engine1.ExecutionContext;
import com.hp.hpl.jena.query.engine1.iterator.QueryIterPlainWrapper;
import com.hp.hpl.jena.query.pfunction.PFuncSimple;
import com.hp.hpl.jena.graph.Node;

/**
 * Name property (isparql:name)
 *
 * @author		Markus Stocker
 * @version 	$Id: name.java,v 1.2 2006-11-21 18:31:46 ms Exp $
 */

public class name extends PFuncSimple 
{	
	/**
	 * Override the ARQ method. We execute the logic
	 * when a triple pattern containing isparql:name
	 * is identified in iSPARQL query.
	 * 
	 * @param binding
	 * @param subject
	 * @param predicate
	 * @param object
	 * @param execCxt
	 * @return QueryIterator
	 */
	public QueryIterator execEvaluated(Binding binding, Node subject, Node predicate, Node object, ExecutionContext execCxt)
	{	
		URIRegistry urireg = new URIRegistry();
		StrategyRegistry strategyreg = (StrategyRegistry)execCxt.getContext().get(new Symbol(Constants.STRATEGY_REGISTRY));
		
		
		// If the strategy registry is not found in the execution context, create it
		if (strategyreg == null) {
			strategyreg = new StrategyRegistry();
			execCxt.getContext().set(new Symbol(Constants.STRATEGY_REGISTRY), strategyreg);
		}
		
		// Create a URI to identify the new strategy
		// (ie http://www.ifi.unizh.ch/isparql/measure/sequence/levenshtein#ID1)
		String uri = urireg.get(object.toString(false)) + strategyreg.nextMeasureId();
		
		// Create a strategy
		AbstractStrategy strategy = StrategyFactory.create(binding, subject, predicate, object, execCxt);

		// Set the URI to the strategy
		strategy.setURI(uri);
		
		// Set the name to the strategy
		strategy.setName(object.toString(false));
		
		// Add the new strategy to the registry
		strategyreg.put(uri, strategy);
		
		// Bind the container variable 
		// ie ?levenshtein = http://www.ifi.unizh.ch/isparql/measure/sequence/levenshtein#ID1
		// The container has a ? as first char, remove this to not have two ?? after binding
		List<Binding> bindings = StrategyProperty.bind(binding, subject, Node.create(uri));
		
		// Return the binding to the engine
		return new QueryIterPlainWrapper(bindings.iterator(), execCxt);
	}
}/* 
 * (c) Copyright 2007 University of Zurich, Switzerland
 * All rights reserved.
 *
 * This file is licensed under a GNU Lesser General Public License.
 *
 * For more information about license terms see LICENSE.txt that comes with
 * the main iSPARQL-OptARQ distribution or visit the GNU website at
 * http://www.gnu.org/licenses/lgpl.html
 *
 */
