package ch.unizh.ifi.isparql.query.property;

import java.util.*;
import ch.unizh.ifi.isparql.query.Constants;
import ch.unizh.ifi.isparql.query.StrategyRegistry;
import ch.unizh.ifi.isparql.query.api.impl.AbstractStrategy;
import com.hp.hpl.jena.graph.Node;
import com.hp.hpl.jena.query.core.Binding;
import com.hp.hpl.jena.query.engine.QueryIterator;
import com.hp.hpl.jena.query.engine1.ExecutionContext;
import com.hp.hpl.jena.query.engine1.iterator.QueryIterPlainWrapper;
import com.hp.hpl.jena.query.pfunction.PropFuncArg;
import com.hp.hpl.jena.query.pfunction.PropertyFunctionBase;
import com.hp.hpl.jena.query.util.Symbol;

/**
 * Ontology URI property (isparql:ontologyURI)
 *
 * @author		Markus Stocker
 * @version 	$Id: ontologyURI.java,v 1.2 2006-11-21 18:31:46 ms Exp $
 */

public class ontologyURI extends PropertyFunctionBase 
{	
	/**
	 * Override the ARQ method. We execute the logic
	 * when a triple pattern containing isparql:ontologyURI
	 * is identified in iSPARQL query.
	 * 
	 * @param binding
	 * @param subject
	 * @param predicate
	 * @param object
	 * @param execCxt
	 * @return QueryIterator
	 */
	public QueryIterator exec(Binding binding, PropFuncArg subject, Node predicate, PropFuncArg object, ExecutionContext execCxt)
	{	
		String ontologyURI1, ontologyURI2;
		
		// For subgraph isomorphism we have two URIs
		if (object.isList() && object.getArgList().size() == 2) 
		{
			ontologyURI1 = ((Node)object.getArg(0)).toString(false);
			ontologyURI2 = ((Node)object.getArg(1)).toString(false);
		}	
		else
		{
			ontologyURI1 = object.getArg().toString(false);
			ontologyURI2 = null;
		}
		
		List<Binding> bindings = new ArrayList<Binding>();
		StrategyRegistry resourcereg = (StrategyRegistry)execCxt.getContext().get(new Symbol(Constants.STRATEGY_REGISTRY));
		
		// Get subject binding
		subject = subject.evalIfExists(binding);
		
		// Get the strategy from the registry
		String uri = subject.getArg().toString(false);
		AbstractStrategy strategy = (AbstractStrategy)resourcereg.get(uri);
		
		// Set ontology URI value to strategy
		strategy.setOntologyURI1(ontologyURI1);
		strategy.setOntologyURI2(ontologyURI2);
		
		// For ontology URI, there is nothing to bind, add old bindings to list
		bindings.add(binding);
		
		return new QueryIterPlainWrapper(bindings.iterator(), execCxt);
	}
	
	/*
	public QueryIterator execEvaluated(Binding binding, Node subject, Node predicate, Node object, ExecutionContext execCxt)
	{	
		List<Binding> bindings = new ArrayList<Binding>();
		StrategyRegistry resourcereg = (StrategyRegistry)execCxt.getContext().get(new Symbol(Constants.STRATEGY_REGISTRY));
		
		// Get the strategy from the registry
		String uri = subject.toString(false);
		AbstractStrategy strategy = (AbstractStrategy)resourcereg.get(uri);
		
		// Set ontology URI value to strategy
		strategy.setOntologyURI(object.toString(false));
		
		// For ontology URI, there is nothing to bind, add old bindings to list
		bindings.add(binding);
		
		return new QueryIterPlainWrapper(bindings.iterator(), execCxt);
	}
	*/
}/* 
 * (c) Copyright 2007 University of Zurich, Switzerland
 * All rights reserved.
 *
 * This file is licensed under a GNU Lesser General Public License.
 *
 * For more information about license terms see LICENSE.txt that comes with
 * the main iSPARQL-OptARQ distribution or visit the GNU website at
 * http://www.gnu.org/licenses/lgpl.html
 *
 */
