package ch.unizh.ifi.isparql.query.property;

import ch.unizh.ifi.isparql.query.Constants;
import ch.unizh.ifi.isparql.query.StrategyProperty;
import ch.unizh.ifi.isparql.query.StrategyRegistry;
import ch.unizh.ifi.isparql.query.api.impl.AbstractStrategy;
import ch.unizh.ifi.isparql.query.api.impl.AbstractOptimizer;
import ch.unizh.ifi.isparql.query.optimizer.SimilarityIndexMem;
import com.hp.hpl.jena.graph.Node;
import com.hp.hpl.jena.query.core.Binding;
import com.hp.hpl.jena.query.engine.QueryIterator;
import com.hp.hpl.jena.query.engine1.ExecutionContext;
import com.hp.hpl.jena.query.engine1.iterator.QueryIterPlainWrapper;
import com.hp.hpl.jena.query.util.Symbol;
import com.hp.hpl.jena.query.pfunction.PFuncSimple;
import java.util.*;

/**
 * Similarity property (isparql:similarity)
 *
 * @author		Markus Stocker
 * @version 	$Id: similarity.java,v 1.2 2006-11-21 18:31:46 ms Exp $
 */

public class similarity extends PFuncSimple 
{	
	/**
	 * Override the ARQ method. We execute the logic
	 * when a triple pattern containing isparql:similarity
	 * is identified in iSPARQL query.
	 * 
	 * @param binding
	 * @param subject
	 * @param predicate
	 * @param object
	 * @param execCxt
	 * @return QueryIterator
	 */
	public QueryIterator execEvaluated(Binding binding, Node subject, Node predicate, Node object, ExecutionContext execCxt)
	{			
		Double similarity = null;
		String uri = subject.toString(false);
		List<Binding> bindings = new ArrayList<Binding>();
		StrategyRegistry strategyreg = (StrategyRegistry)execCxt.getContext().get(new Symbol(Constants.STRATEGY_REGISTRY));
		SimilarityIndexMem si = (SimilarityIndexMem)execCxt.getContext().get(new Symbol(Constants.SIMILARITY_INDEX));
		AbstractStrategy strategy = (AbstractStrategy)strategyreg.get(uri);
		
		// Check if the similarity index is null, then create it
		/*
		if (si == null)
		{
			si = new SimilarityIndex();
			execCxt.getContext().set(new Symbol(Constants.SIMILARITY_INDEX), si);
		}
		*/
		
		/* If optimizer is activated, lookup for the similarity */
		if (AbstractOptimizer.optimizer && si != null)
		{
			String argument1, argument2;
			Node node1 = strategy.getArgument1();
			Node node2 = strategy.getArgument2();
			
			if (node1 != null && node2 != null)
			{
				if (node1.isURI())
					argument1 = node1.getURI();
				else 
					argument1 = node1.getLiteralValue().toString();
			
				if (node2.isURI())
					argument2 = node2.getURI();
				else 
					argument2 = node2.getLiteralValue().toString();
		
				similarity = si.lookup(argument1, argument2, strategy.getName());
			}
		}
		
		/* Overall similarity */
		if (similarity == null)
			similarity = strategy.similarity();
		
		/*
		 * Optimize: check if the calculated similarity out of the strategy satisfies
		 * the threshold and both lower and upper bound of between properties
		 * If this is not the case, we discard the record from the results by 
		 * returning an empty list (binding)
		 */
		if (similarity >= strategy.getThreshold() 
			&& similarity >= strategy.getLowerBound() 
			&& similarity <= strategy.getUpperBound())
			// Bind the similarity value to the results
			bindings = StrategyProperty.bind(binding, object, similarity);
		
		// Clean up registry
		strategyreg.clear();
		
		return new QueryIterPlainWrapper(bindings.iterator(), execCxt);
	}
}
/* 
 * (c) Copyright 2007 University of Zurich, Switzerland
 * All rights reserved.
 *
 * This file is licensed under a GNU Lesser General Public License.
 *
 * For more information about license terms see LICENSE.txt that comes with
 * the main iSPARQL-OptARQ distribution or visit the GNU website at
 * http://www.gnu.org/licenses/lgpl.html
 *
 */
