package ch.unizh.ifi.isparql.query.property;

import ch.unizh.ifi.isparql.query.Constants;
import ch.unizh.ifi.isparql.query.StrategyRegistry;
import ch.unizh.ifi.isparql.query.api.impl.AbstractStrategy;
import com.hp.hpl.jena.graph.Node;
import com.hp.hpl.jena.query.core.Binding;
import com.hp.hpl.jena.query.engine.QueryIterator;
import com.hp.hpl.jena.query.engine1.ExecutionContext;
import com.hp.hpl.jena.query.engine1.iterator.QueryIterPlainWrapper;
import com.hp.hpl.jena.query.util.Symbol;
import com.hp.hpl.jena.query.pfunction.PropFuncArg;
import com.hp.hpl.jena.query.pfunction.PropertyFunctionBase;
import java.util.*;

/**
 * Weights property (isparql:weights)
 *
 * @author		Markus Stocker
 * @version 	$Id: weights.java,v 1.2 2006-11-21 18:31:46 ms Exp $
 */

public class weights extends PropertyFunctionBase 
{	
	/**
	 * Override the ARQ method. We execute the logic
	 * when a triple pattern containing isparql:argument1
	 * is identified in iSPARQL query.
	 * 
	 * @param binding
	 * @param subject
	 * @param predicate
	 * @param object
	 * @param execCxt
	 * @return QueryIterator
	 */
	public QueryIterator exec(Binding binding, PropFuncArg subject, Node predicate, PropFuncArg object, ExecutionContext execCxt)
	{	
		List<Binding> bindings = new ArrayList<Binding>();
		StrategyRegistry resourcereg = (StrategyRegistry)execCxt.getContext().get(new Symbol(Constants.STRATEGY_REGISTRY));
		
		// Get the strategy from the registry
		subject = subject.evalIfExists(binding);
		String uri = subject.getArg().toString();
		AbstractStrategy strategy = (AbstractStrategy)resourcereg.get(uri);
		
		for (Iterator iter = object.getArgList().iterator(); iter.hasNext(); )
			strategy.addWeight(new Double(((Node)iter.next()).getLiteralValue().toString()));
		
		// For scores, there is nothing to bind, add old bindings to list
		bindings.add(binding);
	 
		return new QueryIterPlainWrapper(bindings.iterator(), execCxt);
	}
}
/* 
 * (c) Copyright 2007 University of Zurich, Switzerland
 * All rights reserved.
 *
 * This file is licensed under a GNU Lesser General Public License.
 *
 * For more information about license terms see LICENSE.txt that comes with
 * the main iSPARQL-OptARQ distribution or visit the GNU website at
 * http://www.gnu.org/licenses/lgpl.html
 *
 */
