package ch.unizh.ifi.isparql.query.strategy;

import ch.unizh.ifi.isparql.query.api.impl.AbstractStrategy;
import ch.unizh.ifi.isparql.query.measure.sequence.MeasureLOL;
import com.hp.hpl.jena.graph.Graph;
import com.hp.hpl.jena.graph.Node;
import com.hp.hpl.jena.query.core.Binding;
import com.hp.hpl.jena.query.engine1.ExecutionContext;

/**
 * Levenshtein of Levenshtein string comparison strategy
 *
 * @author		Markus Stocker
 * @version 	$Id: LOLStrCmp.java,v 1.2 2006-11-21 18:31:46 ms Exp $
 */

public class LOLStrCmp extends AbstractStrategy 
{
	/** The framework SimPack wrapper */
	private MeasureLOL measure = new MeasureLOL(this);
	
	/**
	 * Constructor
	 * 
	 * @see ch.unizh.ifi.isparql.query.StrategyFactory
	 * @param binding
	 * @param subject
	 * @param predicate
	 * @param object
	 * @param execCxt
	 */
	public LOLStrCmp(Binding binding, Node subject, Node predicate, Node object, ExecutionContext execCxt)
	{
		super(binding, subject, predicate, object, execCxt);
		
		setName(AbstractStrategy.LOLStrCmp);
	}
	
	/** 
	 * Constructor, used for similarity indexing
	 * 
	 * @see ch.unizh.ifi.isparql.query.StrategyFactory
	 * @param graph
	 */
	public LOLStrCmp(Graph graph)
	{
		super(graph);
		
		setName(AbstractStrategy.LOLStrCmp);
	}
	
	/**
	 * Interface implementation
	 * 
	 * @see ch.unizh.ifi.isparql.query.strategy.LOLStrCmp
	 * @return Double
	 */
	public Double similarity()
	{		
		return measure.similarity();
	}
}
/* 
 * (c) Copyright 2007 University of Zurich, Switzerland
 * All rights reserved.
 *
 * This file is licensed under a GNU Lesser General Public License.
 *
 * For more information about license terms see LICENSE.txt that comes with
 * the main iSPARQL-OptARQ distribution or visit the GNU website at
 * http://www.gnu.org/licenses/lgpl.html
 *
 */
