package ch.unizh.ifi.isparql.query.strategy;

import ch.unizh.ifi.isparql.query.api.impl.AbstractStrategy;
import ch.unizh.ifi.isparql.query.measure.tree.MeasureTreeEditDistance;
import com.hp.hpl.jena.graph.Node;
import com.hp.hpl.jena.query.core.Binding;
import com.hp.hpl.jena.query.engine1.ExecutionContext;
import simpack.api.ITreeNode;
import simpack.util.tree.TreeNode;
import java.util.Iterator;

/**
 * TreeEditDistance resource comparison for MIT Process Handbook
 *
 * @author		Markus Stocker
 * @version 	$Id: MITPHTreeEditDistanceResCmp.java,v 1.2 2006-11-21 18:31:46 ms Exp $
 */

public class MITPHTreeEditDistanceResCmp extends AbstractStrategy 
{
	/** The framework SimPack wrapper */
	private MeasureTreeEditDistance measure = new MeasureTreeEditDistance(this);
	private String baseURI = "http://www.ifi.unizh.ch/ddis/ph/2006/08/ProcessHandbook.owl#";
	
	/**
	 * Constructor
	 * 
	 * @see ch.unizh.ifi.isparql.query.StrategyFactory
	 * @param binding
	 * @param subject
	 * @param predicate
	 * @param object
	 * @param execCxt
	 */
	public MITPHTreeEditDistanceResCmp(Binding binding, Node subject, Node predicate, Node object, ExecutionContext execCxt)
	{
		super(binding, subject, predicate, object, execCxt);
		
		setName(AbstractStrategy.MITPHTreeEditDistanceResCmp);
	}
	
	/**
	 * Interface implementation
	 * 
	 * @see ch.unizh.ifi.isparql.query.measure.tree.MeasureTreeEditDistance
	 * @return Double
	 */
	public Double similarity()
	{	
		int level = 0;
		ITreeNode tree1 = new TreeNode(getArgument1().getURI());
		ITreeNode tree2 = new TreeNode(getArgument2().getURI());
	
		tree1 = getTreeNodeFor(tree1, getArgument1(), level);
		tree2 = getTreeNodeFor(tree2, getArgument2(), level);
		
		measure.setTree1(tree1);
		measure.setTree2(tree2);
		
		return measure.similarity();
	}
	
	// Create trees recursively until level 10
	private ITreeNode getTreeNodeFor(ITreeNode tree, Node uri, int level)
	{	
		level++;
		
		// We create trees of recurse level 10
		if (level == 10)
			return tree;
		
		Iterator hasPartPredicates = graph.queryHandler().objectsFor(uri, Node.createURI(baseURI + "hasPart"));
		
		/*
		 * Step through all hasPart predicates for this uri 
		 * and create a TreeNode for each one
		 * Add the hasPart TreeNodes to the parent TreeNode
		 */
		while (hasPartPredicates.hasNext())
		{
			// Get the URI for the new hasPart predicate
			uri = (Node)hasPartPredicates.next();
			tree.add(getTreeNodeFor(new TreeNode(uri.getURI()), uri, level));
		}
		
		return tree;
	}
}
/* 
 * (c) Copyright 2007 University of Zurich, Switzerland
 * All rights reserved.
 *
 * This file is licensed under a GNU Lesser General Public License.
 *
 * For more information about license terms see LICENSE.txt that comes with
 * the main iSPARQL-OptARQ distribution or visit the GNU website at
 * http://www.gnu.org/licenses/lgpl.html
 *
 */
