package ch.unizh.ifi.isparql.query.strategy;

import ch.unizh.ifi.isparql.query.AggregatorFactory;
import ch.unizh.ifi.isparql.query.api.impl.AbstractAggregator;
import ch.unizh.ifi.isparql.query.api.impl.AbstractStrategy;
import com.hp.hpl.jena.graph.Node;
import com.hp.hpl.jena.query.core.Binding;
import com.hp.hpl.jena.query.engine1.ExecutionContext;
import java.util.*;

/**
 * Aggregator strategy.
 * This strategy takes similarity scores and (optionally) weights to create
 * an overall similarity value based on different strategies defined in a query
 *
 * @author		Markus Stocker
 * @version 	$Id: ScoreAggregator.java,v 1.2 2006-11-21 18:31:46 ms Exp $
 */

public class ScoreAggregator extends AbstractStrategy 
{
	/**
	 * Constructor
	 * 
	 * @see ch.unizh.ifi.isparql.query.StrategyFactory
	 * @param binding
	 * @param subject
	 * @param predicate
	 * @param object
	 * @param execCxt
	 */
	public ScoreAggregator(Binding binding, Node subject, Node predicate, Node object, ExecutionContext execCxt)
	{
		super(binding, subject, predicate, object, execCxt);
		
		setName(AbstractStrategy.scoreAggregator);
	}
	
	/**
	 * Interface implementation. 
	 * There is no SimPack measure wrapper involved.
	 * 
	 * @return Double
	 */
	public Double similarity()
	{	
		// Take the informations from strategy
		List<Double> scores = getScores();
		List<Double> weights = getWeights();
		
		AbstractAggregator aggregator = AggregatorFactory.create(getAggregator());
		aggregator.addScores(scores);
		
		if (scores.size() == weights.size())
			aggregator.addWeights(weights);
		
		return aggregator.aggregate();
	}
}
/* 
 * (c) Copyright 2007 University of Zurich, Switzerland
 * All rights reserved.
 *
 * This file is licensed under a GNU Lesser General Public License.
 *
 * For more information about license terms see LICENSE.txt that comes with
 * the main iSPARQL-OptARQ distribution or visit the GNU website at
 * http://www.gnu.org/licenses/lgpl.html
 *
 */
