package ch.unizh.ifi.isparql.tools;

import com.hp.hpl.jena.rdf.model.Model;
import com.hp.hpl.jena.rdf.model.ModelFactory;
import com.hp.hpl.jena.util.FileManager;
import java.io.*;

/**
 * This tool allows the aggregation of ontologies to a single model.
 * The tool takes an input directory, where *only* ontology files
 * which should be aggregated are stored.
 * 
 * After reading the files which can be found in the input directory
 * to a single model, the model is stored to the file which is
 * given as output.
 * 
 * @author		Markus Stocker
 * @version 	$Id: OntologyAggregator.java,v 1.1 2006-11-20 18:48:55 ms Exp $
 */

public class OntologyAggregator 
{
	private static String inDirectory, outFileName;
	/** The Jena model */
	private static Model model = ModelFactory.createDefaultModel();
	
	public static void main(String[] args)
	{	
		if (args.length > 0)
		{
			for (int i = 0; i < args.length; i++)
			{
				if (args[i].equals("--dir"))
					inDirectory = args[i+1];
				else if (args[i].equals("--out"))
					outFileName = args[i+1];
				else if (args[i].equals("--help"))
					usage();
			}
		} 
		else
			usage();
		
		File dir = new File(inDirectory);
			
		String[] files = dir.list();
		if (files == null)
		{
			System.out.println("Error on reading " + inDirectory);
			System.exit(0);
		}
		
		for (int i = 0; i < files.length; i++) {
			System.out.println(files[i]);
			InputStream in = FileManager.get().open(inDirectory + files[i]);
			model.read(in, "");
		}
		
		try {
	        BufferedWriter out = new BufferedWriter(new FileWriter(outFileName));
	        model.write(out);
	        out.close();
	    } catch (IOException e) {
	    }
	}
	
	private static void usage()
	{
		System.out.println("ch.unizh.ifi.isparql.tools.OntologyAggregator [options]");
		System.out.println("Options:");
		System.out.println("--dir [input directory]");
		System.out.println("--out [ouput ontology file name]");
		
		System.exit(0);
	}
}
/* 
 * (c) Copyright 2007 University of Zurich, Switzerland
 * All rights reserved.
 *
 * This file is licensed under a GNU Lesser General Public License.
 *
 * For more information about license terms see LICENSE.txt that comes with
 * the main iSPARQL-OptARQ distribution or visit the GNU website at
 * http://www.gnu.org/licenses/lgpl.html
 *
 */
