package ch.unizh.ifi.isparql.tools;

import com.hp.hpl.jena.rdf.model.Model;
import com.hp.hpl.jena.rdf.model.ModelFactory;
import com.hp.hpl.jena.rdf.model.ResIterator;
import com.hp.hpl.jena.util.FileManager;
import java.io.*;

/**
 * This tool allows counting resources of an underlying ontology
 *
 * @author		Markus Stocker
 * @version 	$Id: ResourceCounter.java,v 1.1 2006-11-20 18:48:55 ms Exp $
 */

public class ResourceCounter 
{
	/** The Jena model */
	private static Model model = ModelFactory.createNonreifyingModel();
	
	/**
	 * The main application entry point 
	 * 
	 * @param args
	 */
	public static void main(String[] args)
	{
		String inGraphFileName = null;
		
		// Eval arguments
		if (args.length > 0)
		{
			for (int i = 0; i < args.length; i++)
			{				
				if (args[i].equals("--graph"))
					inGraphFileName = args[i + 1];
				else if (args[i].equals("--help"))
					usage();
			}
		} else
			usage();
		
		InputStream in = FileManager.get().open(inGraphFileName);
		
		if (in == null)
			throw new IllegalArgumentException("File: " + inGraphFileName + " not found");
	
		// Load the model
		model.read(in, "");
		
		ResIterator resIter = model.listSubjects();
		
		int count = 0;
		
		while (resIter.hasNext())
		{
			resIter.nextResource();
			count++;
		}
		
		System.out.println("Number of resources: " + count);
	}
	
	private static void usage()
	{
		System.out.println("ch.unizh.ifi.isparql.tool.ResourceCounter [options]");
		System.out.println("Options:");
		System.out.println("--graph [filename]");
		
		System.exit(0);
	}
}
/* 
 * (c) Copyright 2007 University of Zurich, Switzerland
 * All rights reserved.
 *
 * This file is licensed under a GNU Lesser General Public License.
 *
 * For more information about license terms see LICENSE.txt that comes with
 * the main iSPARQL-OptARQ distribution or visit the GNU website at
 * http://www.gnu.org/licenses/lgpl.html
 *
 */
