package ch.unizh.ifi.isparql.tools;

import java.io.*;
import com.hp.hpl.jena.query.Query;
import com.hp.hpl.jena.query.QueryExecution;
import com.hp.hpl.jena.query.QueryExecutionFactory;
import com.hp.hpl.jena.query.QueryFactory;
import com.hp.hpl.jena.query.ResultSet;
import com.hp.hpl.jena.rdf.model.Model;
import com.hp.hpl.jena.rdf.model.ModelFactory;
import com.hp.hpl.jena.util.FileManager;

/**
 * This tool counts the number of rows returned by a 
 * specific query which is executed over an underlying
 * ontology.
 * 
 * @author		Markus Stocker
 * @version 	$Id: TupleCounter.java,v 1.1 2006-11-20 18:48:55 ms Exp $
 */

public class TupleCounter 
{
	/** The Jena model */
	private static Model model = ModelFactory.createDefaultModel();
	private static String inFileName, inQueryFileName;
	
	/**
	 * Main entry point to the application
	 * 
	 * @param args
	 */
	public static void main(String[] args)
	{
		if (args.length > 0)
		{
			for (int i = 0; i < args.length; i++)
			{
				if (args[i].equals("--graph"))
					inFileName = args[i+1];
				else if (args[i].equals("--query"))
					inQueryFileName = args[i+1];
				else if (args[i].equals("--help"))
					usage();
			}
		} else 
			usage();
		
		System.out.println("=================================");
		System.out.println("Start evaluation");
		System.out.println("Graph file: " + inFileName);
		System.out.println("Execute query");
		
		InputStream in = FileManager.get().open( inFileName );
		model.read(in, "");
		
		// Read the query from file
		String inQueryString = inQueryString(inQueryFileName);
		
		System.out.println(inQueryString);
		System.out.println("==================================\n\n");
	
		Query query = QueryFactory.create(inQueryString);
		QueryExecution qe = QueryExecutionFactory.create(query, model);
		ResultSet results = qe.execSelect();
			
		int c = 0;
		
		for ( ; results.hasNext() ; )
		{
			results.nextSolution();
			c++;
		}
		
		System.out.println("Number of records: " + c);
	
		qe.close();
		
		model.close();
	}
	
	/** Read the query from file */
	private static String inQueryString(String inQueryFileName)
	{
		String line = "";
		BufferedReader br = null;
		String inQueryString = "";
		
		try
		{
			br = new BufferedReader(new InputStreamReader(new FileInputStream(inQueryFileName)));
		} catch (FileNotFoundException e) { System.out.println(e); }
		
		try
		{
			while((line = br.readLine()) != null) {
				inQueryString = inQueryString + line + "\n";
			}
		} catch (IOException e) {}
		
		return inQueryString;
	}
	
	private static void usage()
	{
		System.out.println("ch.unizh.ifi.isparql.tools.TupleCounter [options]");
		System.out.println("Options:");
		System.out.println("--graph [filename]");
		System.out.println("--query [filename]");
		
		System.exit(0);
	}
}
/* 
 * (c) Copyright 2007 University of Zurich, Switzerland
 * All rights reserved.
 *
 * This file is licensed under a GNU Lesser General Public License.
 *
 * For more information about license terms see LICENSE.txt that comes with
 * the main iSPARQL-OptARQ distribution or visit the GNU website at
 * http://www.gnu.org/licenses/lgpl.html
 *
 */
