package ch.unizh.ifi.isparql.tests;

import java.io.ByteArrayInputStream;

import com.hp.hpl.jena.rdf.model.Model;
import com.hp.hpl.jena.rdf.model.ModelFactory;

/**
 * Sample RDF cost model. It corresponds to the default model used for tests.
 * Test class for reuse in multiple test cases.
 *
 * @see			ch.unizh.ifi.isparql.tests.DefaultModel
 * @author		Markus Stocker
 * @version 	$Id: DefaultCostModel.java,v 1.1 2006-11-22 12:32:52 ms Exp $
 */

public class DefaultCostModel 
{
	public static final String NS = "http://localhost/";
	public static final String NSRP = NS + "resource/person/";
	public static final String NSRA = NS + "resource/address/";
	public static final String NSP = NS + "property/";
	private Model model = ModelFactory.createDefaultModel();

	public DefaultCostModel()
	{
		String defaultCostModel = defaultCostModel();
		
		// Convert to string to an input stream, which is given to Jena
		ByteArrayInputStream in = new ByteArrayInputStream(defaultCostModel.getBytes());
		
		model.read(in, "");
	}
	
	public Model getModel()
	{
		return model;
	}
	
	private String defaultCostModel()
	{
		return "<?xml version=\"1.0\"?> " +
			   "<rdf:RDF " +
			   "xmlns:j.0=\"http://localhost/\" " +
			   "xmlns:j.1=\"http://localhost/histogram/class/\" " +
			   "xmlns:rdf=\"http://www.w3.org/1999/02/22-rdf-syntax-ns#\" > " +
			   "<rdf:Description rdf:about=\"http://localhost/histogram/class/#C7\">" +
			   "<j.1:label>-1.324753371E9</j.1:label>" +
			   "<j.1:items>1</j.1:items>" +
			   "</rdf:Description>" +
			   "<rdf:Description rdf:about=\"http://localhost/histogram/class/#C6\">" +
			   "<j.1:label>1.896154529E9</j.1:label>" +
			   "<j.1:items>1</j.1:items>" +
			   "</rdf:Description>" +
			   "<rdf:Description rdf:about=\"http://localhost/histogram/class/#C3\">" +
			   "<j.1:label>806.0</j.1:label>" +
			   "<j.1:items>2</j.1:items>" +
			   "</rdf:Description>" +
			   "<rdf:Description rdf:about=\"http://localhost/property/firstname\">" +
			   "<j.0:histogram rdf:resource=\"http://localhost/histogram/#H5\"/>" +
			   "<j.0:frequency>3</j.0:frequency>" +
			   "</rdf:Description>" +
			   "<rdf:Description rdf:about=\"http://localhost/histogram/#H7\">" +
			   "<rdf:_2 rdf:resource=\"http://localhost/histogram/class/#C14\"/>" +
			   "<rdf:_1 rdf:resource=\"http://localhost/histogram/class/#C13\"/>" +
			   "<rdf:type rdf:resource=\"http://www.w3.org/1999/02/22-rdf-syntax-ns#Seq\"/>" +
			   "</rdf:Description>" +
			   "<rdf:Description rdf:about=\"http://localhost/property/address\">" +
			   "<j.0:histogram rdf:resource=\"http://localhost/histogram/#H1\"/>" +
			   "<j.0:frequency>3</j.0:frequency>" +
			   "</rdf:Description>" +
			   "<rdf:Description rdf:about=\"http://localhost/histogram/class/#C8\">" +
			   "<j.1:label>-1.083382381E9</j.1:label>" +
			   "<j.1:items>2</j.1:items>" +
			   "</rdf:Description>" +
			   "<rdf:Description rdf:about=\"http://localhost/histogram/#H2\">" +
			   "<rdf:_2 rdf:resource=\"http://localhost/histogram/class/#C4\"/>" +
			   "<rdf:_1 rdf:resource=\"http://localhost/histogram/class/#C3\"/>" +
			   "<rdf:type rdf:resource=\"http://www.w3.org/1999/02/22-rdf-syntax-ns#Seq\"/>" +
			   "</rdf:Description>" +
			   "<rdf:Description rdf:about=\"http://localhost/histogram/class/#C9\">" +
			   "<j.1:label>-1.791047915E9</j.1:label>" +
			   "<j.1:items>2</j.1:items>" +
			   "</rdf:Description>" +
			   "<rdf:Description rdf:about=\"http://localhost/statistics\">" +
			   "<j.0:avgNrOfPredicates>3.3333333333333335</j.0:avgNrOfPredicates>" +
			   "<j.0:nrOfTriples>20</j.0:nrOfTriples>" +
			   "</rdf:Description>" +
			   "<rdf:Description rdf:about=\"http://localhost/histogram/#H1\">" +
			   "<rdf:_2 rdf:resource=\"http://localhost/histogram/class/#C2\"/>" +
			   "<rdf:_1 rdf:resource=\"http://localhost/histogram/class/#C1\"/>" +
			   "<rdf:type rdf:resource=\"http://www.w3.org/1999/02/22-rdf-syntax-ns#Seq\"/>" +
			   "</rdf:Description>" +
			   "<rdf:Description rdf:about=\"http://localhost/property/lastname\">" +
			   "<j.0:histogram rdf:resource=\"http://localhost/histogram/#H3\"/>" +
			   "<j.0:frequency>3</j.0:frequency>" +
			   "</rdf:Description>" +
			   "<rdf:Description rdf:about=\"http://localhost/histogram/class/#C1\">" +
			   "<j.1:label>-2.009287513E9</j.1:label>" +
			   "<j.1:items>2</j.1:items>" +
			   "</rdf:Description>" +
			   "<rdf:Description rdf:about=\"http://localhost/property/age\">" +
			   "<j.0:histogram rdf:resource=\"http://localhost/histogram/#H2\"/>" +
			   "<j.0:frequency>3</j.0:frequency>" +
			   "</rdf:Description>" +
			   "<rdf:Description rdf:about=\"http://localhost/property/city\">" +
			   "<j.0:histogram rdf:resource=\"http://localhost/histogram/#H6\"/>" +
			   "<j.0:frequency>2</j.0:frequency>" +
			   "</rdf:Description>" +
			   "<rdf:Description rdf:about=\"http://localhost/histogram/#H4\">" +
			   "<rdf:_2 rdf:resource=\"http://localhost/histogram/class/#C8\"/>" +
			   "<rdf:_1 rdf:resource=\"http://localhost/histogram/class/#C7\"/>" +
			   "<rdf:type rdf:resource=\"http://www.w3.org/1999/02/22-rdf-syntax-ns#Seq\"/>" +
			   "</rdf:Description>" +
			   "<rdf:Description rdf:about=\"http://localhost/histogram/class/#C14\">" +
			   "<j.1:label>227276.5</j.1:label>" +
			   "<j.1:items>2</j.1:items>" +
			   "</rdf:Description>" +
			   "<rdf:Description rdf:about=\"http://localhost/predicates\">" +
			   "<rdf:_7>http://localhost/property/zip</rdf:_7>" +
			   "<rdf:_6>http://localhost/property/city</rdf:_6>" +
			   "<rdf:_5>http://localhost/property/firstname</rdf:_5>" +
			   "<rdf:_4>http://localhost/property/street</rdf:_4>" +
			   "<rdf:_3>http://localhost/property/lastname</rdf:_3>" +
			   "<rdf:_2>http://localhost/property/age</rdf:_2>" +
			   "<rdf:_1>http://localhost/property/address</rdf:_1>" +
			   "<rdf:type rdf:resource=\"http://www.w3.org/1999/02/22-rdf-syntax-ns#Seq\"/>" +
			   "</rdf:Description>" +
			   "<rdf:Description rdf:about=\"http://localhost/histogram/#H3\">" +
			   "<rdf:_2 rdf:resource=\"http://localhost/histogram/class/#C6\"/>" +
			   "<rdf:_1 rdf:resource=\"http://localhost/histogram/class/#C5\"/>" +
			   "<rdf:type rdf:resource=\"http://www.w3.org/1999/02/22-rdf-syntax-ns#Seq\"/>" +
			   "</rdf:Description>" +
			   "<rdf:Description rdf:about=\"http://localhost/histogram/class/#C13\">" +
			   "<j.1:label>204972.0</j.1:label>" +
			   "<j.1:items>1</j.1:items>" +
			   "</rdf:Description>" +
			   "<rdf:Description rdf:about=\"http://localhost/histogram/class/#C2\">" +
			   "<j.1:label>-3.9943434E8</j.1:label>" +
			   "<j.1:items>1</j.1:items>" +
			   "</rdf:Description>" +
			   "<rdf:Description rdf:about=\"http://localhost/histogram/class/#C12\">" +
			   "<j.1:label>1.337232364E9</j.1:label>" +
			   "<j.1:items>1</j.1:items>" +
			   "</rdf:Description>" +
			   "<rdf:Description rdf:about=\"http://localhost/histogram/class/#C5\">" +
			   "<j.1:label>1.768134205E9</j.1:label>" +
			   "<j.1:items>2</j.1:items>" +
			   "</rdf:Description>" +
			   "<rdf:Description rdf:about=\"http://localhost/histogram/#H6\">" +
			   "<rdf:_2 rdf:resource=\"http://localhost/histogram/class/#C12\"/>" +
			   "<rdf:_1 rdf:resource=\"http://localhost/histogram/class/#C11\"/>" +
			   "<rdf:type rdf:resource=\"http://www.w3.org/1999/02/22-rdf-syntax-ns#Seq\"/>" +
			   "</rdf:Description>" +
			   "<rdf:Description rdf:about=\"http://localhost/property/zip\">" +
			   "<j.0:histogram rdf:resource=\"http://localhost/histogram/#H7\"/>" +
			   "<j.0:frequency>3</j.0:frequency>" +
			   "</rdf:Description>" +
			   "<rdf:Description rdf:about=\"http://localhost/histogram/class/#C11\">" +
			   "<j.1:label>9.45393583E8</j.1:label>" +
			   "<j.1:items>1</j.1:items>" +
			   "</rdf:Description>" +
			   "<rdf:Description rdf:about=\"http://localhost/histogram/class/#C4\">" +
			   "<j.1:label>1240.0</j.1:label>" +
			   "<j.1:items>1</j.1:items>" +
			   "</rdf:Description>" +
			   "<rdf:Description rdf:about=\"http://localhost/property/street\">" +
			   "<j.0:histogram rdf:resource=\"http://localhost/histogram/#H4\"/>" +
			   "<j.0:frequency>3</j.0:frequency>" +
			   "</rdf:Description>" +
			   "<rdf:Description rdf:about=\"http://localhost/histogram/#H5\">" +
			   "<rdf:_2 rdf:resource=\"http://localhost/histogram/class/#C10\"/>" +
			   "<rdf:_1 rdf:resource=\"http://localhost/histogram/class/#C9\"/>" +
			   "<rdf:type rdf:resource=\"http://www.w3.org/1999/02/22-rdf-syntax-ns#Seq\"/>" +
			   "</rdf:Description>" +
			   "<rdf:Description rdf:about=\"http://localhost/histogram/class/#C10\">" +
			   "<j.1:label>-1.202568068E9</j.1:label>" +
			   "<j.1:items>1</j.1:items>" +
			   "</rdf:Description>" +
			   "</rdf:RDF>";
	}
}
/* 
 * (c) Copyright 2007 University of Zurich, Switzerland
 * All rights reserved.
 *
 * This file is licensed under a GNU Lesser General Public License.
 *
 * For more information about license terms see LICENSE.txt that comes with
 * the main iSPARQL-OptARQ distribution or visit the GNU website at
 * http://www.gnu.org/licenses/lgpl.html
 *
 */
