package ch.unizh.ifi.isparql.tests;

import com.hp.hpl.jena.rdf.model.Model;
import com.hp.hpl.jena.rdf.model.Resource;
import com.hp.hpl.jena.rdf.model.ModelFactory;
import com.hp.hpl.jena.rdf.model.Property;
import com.hp.hpl.jena.datatypes.xsd.XSDDatatype;

/**
 * Sample RDF model. Test class for reuse in multiple test cases.
 * 
 * @author Markus Stocker
 * @version $Id: DefaultModel.java,v 1.4 2006-11-22 12:32:52 ms Exp $
 */

public class DefaultModel {
	public static final String NS = "http://localhost/";
	public static final String NSRP = NS + "resource/person/";
	public static final String NSRA = NS + "resource/address/";
	public static final String NSP = NS + "property/";
	private Model model = ModelFactory.createDefaultModel();

	public DefaultModel() {
		Property firstnameP = model.createProperty(NSP + "firstname");
		Property lastnameP = model.createProperty(NSP + "lastname");
		Property ageP = model.createProperty(NSP + "age");
		Property addressP = model.createProperty(NSP + "address");
		Property streetP = model.createProperty(NSP + "street");
		Property zipP = model.createProperty(NSP + "zip");
		Property cityP = model.createProperty(NSP + "city");

		Resource markusR = model.createResource(NSRP + "markus");
		Resource melanieR = model.createResource(NSRP + "melanie");
		Resource francescaR = model.createResource(NSRP + "francesca");

		Resource markusAddressR = model.createResource(NSRA + "markus");
		Resource melanieAddressR = model.createResource(NSRA + "melanie");
		Resource francescaAddressR = model.createResource(NSRA + "francesca");

		markusAddressR.addProperty(streetP, "Heidwiesen 19");
		markusAddressR.addProperty(zipP, model.createTypedLiteral(8051,
				XSDDatatype.XSDinteger));
		markusAddressR.addProperty(cityP, "Zurich");

		melanieAddressR.addProperty(streetP, "Via Pancaldi Moda 4B");
		melanieAddressR.addProperty(zipP, model.createTypedLiteral(6612,
				XSDDatatype.XSDinteger));
		melanieAddressR.addProperty(cityP, "Ascona");

		francescaAddressR.addProperty(streetP, "Saatlenstrasse 51");
		francescaAddressR.addProperty(zipP, model.createTypedLiteral(8050,
				XSDDatatype.XSDinteger));
		francescaAddressR.addProperty(cityP, "Zurich");

		markusR.addProperty(firstnameP, "Markus");
		markusR.addProperty(lastnameP, "Stocker");
		markusR.addProperty(ageP, model.createTypedLiteral(27,
				XSDDatatype.XSDinteger));
		markusR.addProperty(addressP, markusAddressR);

		melanieR.addProperty(firstnameP, "Melanie");
		melanieR.addProperty(lastnameP, "Stocker");
		melanieR.addProperty(ageP, model.createTypedLiteral(54,
				XSDDatatype.XSDinteger));
		melanieR.addProperty(addressP, melanieAddressR);

		francescaR.addProperty(firstnameP, "Francesca");
		francescaR.addProperty(lastnameP, "Corti");
		francescaR.addProperty(ageP, model.createTypedLiteral(26,
				XSDDatatype.XSDinteger));
		francescaR.addProperty(addressP, francescaAddressR);
	}

	public Model getModel() {
		return model;
	}
}
/* 
 * (c) Copyright 2007 University of Zurich, Switzerland
 * All rights reserved.
 *
 * This file is licensed under a GNU Lesser General Public License.
 *
 * For more information about license terms see LICENSE.txt that comes with
 * the main iSPARQL-OptARQ distribution or visit the GNU website at
 * http://www.gnu.org/licenses/lgpl.html
 *
 */
