package ch.unizh.ifi.isparql.tests.optimizer;

import ch.unizh.ifi.isparql.tests.DefaultModel;
import ch.unizh.ifi.isparql.tests.DefaultCostModel;
import ch.unizh.ifi.isparql.optimizer.ModOptimizer;
import ch.unizh.ifi.isparql.optimizer.CostModel;
import com.hp.hpl.jena.rdf.model.Model;
import com.hp.hpl.jena.query.Query;
import com.hp.hpl.jena.query.QueryExecution;
import com.hp.hpl.jena.query.QueryExecutionFactory;
import com.hp.hpl.jena.query.QueryFactory;
import com.hp.hpl.jena.query.ResultSet;
import com.hp.hpl.jena.query.QuerySolution;
import junit.framework.TestCase;

/**
 * Test the optimizer
 *
 * @author		Markus Stocker
 * @version 	$Id: Rule1TestA.java,v 1.2 2006-11-22 12:32:52 ms Exp $
 */

public class Rule1TestA extends TestCase 
{	
	private CostModel cm;
	private Model defaultModel;
	private Model defaultCostModel;
	private DefaultModel dm = new DefaultModel();
	private DefaultCostModel dcm = new DefaultCostModel();
	private ModOptimizer optimizer = new ModOptimizer();
	
	public void testSimilarity()
	{	
		defaultModel = dm.getModel();
		defaultCostModel = dcm.getModel();
		
		// Load the optimization cost model by assigning 
		// the Jena model of the statistical ontology
		cm = new CostModel(defaultCostModel);
		
		if (! optimizer.isOn())
			optimizer.on();
		
		assertEquals(optimizer.isOn(), true);
		
		query1();
	}
	
	private void query1()
	{
		String qstr = "BASE					<" + DefaultModel.NS + "> " +
				      "PREFIX	person:		<" + DefaultModel.NSP + "> " +
				      "PREFIX	property:	<java:ch.unizh.ifi.isparql.query.property.> " +
				      "SELECT ?person " + 
				      "WHERE { " +
				      "?person person:firstname ?firstname . " +
				      "?person person:lastname ?lastname . " +
				      "FILTER (?firstname = \"Markus\" && ?lastname = \"Stocker\") " +
				      "} ";
		
		String oqstr = "BASE				<" + DefaultModel.NS + "> " +
				       "PREFIX	person:		<" + DefaultModel.NSP + "> " +
				       "PREFIX	property:	<java:ch.unizh.ifi.isparql.query.property.> " +
				       "SELECT ?person " + 
				       "WHERE { " +
				       "?person person:firstname \"Markus\" . " +
				       "?person person:lastname \"Stocker\" . " +
				       "} ";
		
		Query query = QueryFactory.create(qstr);
		Query oquery = QueryFactory.create(oqstr);
		
		query = optimizer.optimize(query, cm);
		
		// Check if the optimizer turned the the qstr into the optimized qstr
		assertEquals(query.getQueryPattern().toString().hashCode(), 
					 oquery.getQueryPattern().toString().hashCode());
		
		QueryExecution qe = QueryExecutionFactory.create(query, defaultModel);
		ResultSet results = qe.execSelect();
		
		assertEquals(results.hasNext(), true);
		
		while (results.hasNext())
		{
			QuerySolution qs = results.nextSolution();

			for (java.util.Iterator iter = qs.varNames(); iter.hasNext(); )
			{
				String key = (String)iter.next();
				
				String value = qs.get(key).asNode().getURI();
				assertEquals(value, DefaultModel.NSRP + "markus");
			}
		}
	}
}

/* 
 * (c) Copyright 2007 University of Zurich, Switzerland
 * All rights reserved.
 *
 * This file is licensed under a GNU Lesser General Public License.
 *
 * For more information about license terms see LICENSE.txt that comes with
 * the main iSPARQL-OptARQ distribution or visit the GNU website at
 * http://www.gnu.org/licenses/lgpl.html
 *
 */
