package ch.unizh.ifi.isparql.tests.query.property;

import ch.unizh.ifi.isparql.tests.DefaultModel;
import com.hp.hpl.jena.rdf.model.Model;
import com.hp.hpl.jena.query.Query;
import com.hp.hpl.jena.query.QueryExecution;
import com.hp.hpl.jena.query.QueryExecutionFactory;
import com.hp.hpl.jena.query.QueryFactory;
import com.hp.hpl.jena.query.ResultSet;
import com.hp.hpl.jena.query.QuerySolution;
import junit.framework.TestCase;

/**
 * Test isparql:aggregator
 *
 * @author		Markus Stocker
 * @version 	$Id: AggregatorTest.java,v 1.2 2006-11-22 12:32:52 ms Exp $
 */

public class AggregatorTest extends TestCase 
{	
	private Model model;
	private DefaultModel dm = new DefaultModel();
	
	public void testSimilarity()
	{		
		model = dm.getModel();
		
		query1();
	}
	
	private void query1()
	{	
		String qstr = "BASE					<" + DefaultModel.NS + "> " +
				      "PREFIX	person:		<" + DefaultModel.NSP + "> " +
				      "PREFIX	property:	<java:ch.unizh.ifi.isparql.query.property.> " +
				      "SELECT ?person1 ?person2 ?similarity " + 
				      "WHERE { " +
				      "?person1 person:firstname \"Markus\" . " +
				      "?person2 person:firstname \"Melanie\" . " +
				      "?strategy property:name \"LevenshteinResCmp\" . " +
				      "?strategy property:aggregator \"avg\" . " +
				      "?strategy property:arguments (?person1 ?person2) . " +
				      "?strategy property:similarity ?similarity " +
				      "} " +
				      "ORDER BY DESC(?similarity) " +
				      "LIMIT 1 ";
		
		Query query = QueryFactory.create(qstr);
		QueryExecution qe = QueryExecutionFactory.create(query, model);
		ResultSet results = qe.execSelect();
		
		assertEquals(results.hasNext(), true);
		
		while (results.hasNext())
		{
			QuerySolution qs = results.nextSolution();

			for (java.util.Iterator iter = qs.varNames(); iter.hasNext(); )
			{
				String key = (String)iter.next();
				
				if (key.equals("person1"))
				{
					String value = qs.get(key).asNode().getURI();
					assertEquals(value, DefaultModel.NSRP + "markus");
				}
				else if (key.equals("person2"))
				{
					String value = qs.get(key).asNode().getURI();
					assertEquals(value, DefaultModel.NSRP + "melanie");
				}
				else if (key.equals("similarity"))
				{
					// Using default avg aggregator (measure + structure similarity)
					String value = qs.get(key).asNode().getLiteralValue().toString();
					assertEquals(new Double(value), new Double(0.7592038396386223));
				}
			}
		}
	}
}

/* 
 * (c) Copyright 2007 University of Zurich, Switzerland
 * All rights reserved.
 *
 * This file is licensed under a GNU Lesser General Public License.
 *
 * For more information about license terms see LICENSE.txt that comes with
 * the main iSPARQL-OptARQ distribution or visit the GNU website at
 * http://www.gnu.org/licenses/lgpl.html
 *
 */
