package ch.unizh.ifi.isparql.tests.query.property;

import ch.unizh.ifi.isparql.tests.DefaultModel;
import com.hp.hpl.jena.rdf.model.Model;
import com.hp.hpl.jena.query.Query;
import com.hp.hpl.jena.query.QueryExecution;
import com.hp.hpl.jena.query.QueryExecutionFactory;
import com.hp.hpl.jena.query.QueryFactory;
import com.hp.hpl.jena.query.ResultSet;
import com.hp.hpl.jena.query.QuerySolution;
import junit.framework.TestCase;

/**
 * Test isparql:scores and isparql:weights, ignoring weights
 *
 * @author		Markus Stocker
 * @version 	$Id: ScoresAndWeightsTestC.java,v 1.2 2006-11-22 12:32:52 ms Exp $
 */

public class ScoresAndWeightsTestC extends TestCase 
{	
	private Model model;
	private DefaultModel dm = new DefaultModel();
	
	public void testSimilarity()
	{		
		model = dm.getModel();
		
		query1();
	}
	
	private void query1()
	{	
		// Ignore weights, since not the same size of scores
		String qstr = "BASE					<" + DefaultModel.NS + "> " +
				      "PREFIX	person:		<" + DefaultModel.NSP + "> " +
				      "PREFIX	property:	<java:ch.unizh.ifi.isparql.query.property.> " +
				      "SELECT ?similarity " + 
				      "WHERE { " +
				      "?strategy property:name \"ScoreAggregator\" . " +
				      "?strategy property:scores (0.2222222222222222 0.14285714285714285 0.5) . " +
				      "?strategy property:weights (0.1 0.1) . " +
				      "?strategy property:aggregator \"sum\" . " +
				      "?strategy property:similarity ?similarity " +
				      "} " +
				      "ORDER BY DESC(?similarity) " +
				      "LIMIT 1 ";
		
		Query query = QueryFactory.create(qstr);
		QueryExecution qe = QueryExecutionFactory.create(query, model);
		ResultSet results = qe.execSelect();
		
		assertEquals(results.hasNext(), true);
		
		while (results.hasNext())
		{
			QuerySolution qs = results.nextSolution();

			for (java.util.Iterator iter = qs.varNames(); iter.hasNext(); )
			{
				String key = (String)iter.next();

				String value = qs.get(key).asNode().getLiteralValue().toString();
				assertEquals(new Double(value), new Double(0.8650793650793651));
			}
		}
	}
}

/* 
 * (c) Copyright 2007 University of Zurich, Switzerland
 * All rights reserved.
 *
 * This file is licensed under a GNU Lesser General Public License.
 *
 * For more information about license terms see LICENSE.txt that comes with
 * the main iSPARQL-OptARQ distribution or visit the GNU website at
 * http://www.gnu.org/licenses/lgpl.html
 *
 */
