package ch.unizh.ifi.isparql.tests.query.property;

import ch.unizh.ifi.isparql.tests.DefaultModel;
import com.hp.hpl.jena.rdf.model.Model;
import com.hp.hpl.jena.query.Query;
import com.hp.hpl.jena.query.QueryExecution;
import com.hp.hpl.jena.query.QueryExecutionFactory;
import com.hp.hpl.jena.query.QueryFactory;
import com.hp.hpl.jena.query.ResultSet;
import com.hp.hpl.jena.query.QuerySolution;
import junit.framework.TestCase;

/**
 * Test isparql:scores, sum, variables, without quotes
 * 
 * @author Markus Stocker
 * @version $Id: ScoresTestB.java,v 1.2 2006-11-22 12:32:52 ms Exp $
 */

public class ScoresTestB extends TestCase {
	private Model model;
	private DefaultModel dm = new DefaultModel();

	public void testSimilarity() {
		model = dm.getModel();

		query1();
	}

	private void query1() {
		String qstr = "BASE					<"
				+ DefaultModel.NS
				+ "> "
				+ "PREFIX	person:		<"
				+ DefaultModel.NSP
				+ "> "
				+ "PREFIX	property:	<java:ch.unizh.ifi.isparql.query.property.> "
				+ "SELECT ?similarity1 ?similarity2 ?similarity3 ?similarity4 "
				+ "WHERE { "
				+ "?person1 person:firstname ?firstname1 . "
				+ "?person2 person:firstname ?firstname2 . "
				+ "?person1 person:lastname ?lastname1 . "
				+ "?person2 person:lastname ?lastname2 . "
				+ "?person1 person:age ?age1 . "
				+ "?person2 person:age ?age2 . "
				+ "?strategy1 property:name \"LevenshteinStrCmp\" . "
				+ "?strategy1 property:arguments (?firstname1 ?firstname2) . "
				+ "?strategy1 property:similarity ?similarity1 . "
				+ "?strategy2 property:name \"LevenshteinStrCmp\" . "
				+ "?strategy2 property:arguments (?lastname1 ?lastname2) . "
				+ "?strategy2 property:similarity ?similarity2 . "
				+ "?strategy3 property:name \"LevenshteinStrCmp\" . "
				+ "?strategy3 property:arguments (?age1 ?age2) . "
				+ "?strategy3 property:similarity ?similarity3 . "
				+ "?strategy4 property:name \"ScoreAggregator\" . "
				+ "?strategy4 property:aggregator \"sum\" . "
				+ "?strategy4 property:scores (?similarity1 ?similarity2 ?similarity3) . "
				+ "?strategy4 property:similarity ?similarity4 " + "} "
				+ "ORDER BY DESC(?similarity) " + "LIMIT 1 ";

		Query query = QueryFactory.create(qstr);
		QueryExecution qe = QueryExecutionFactory.create(query, model);
		ResultSet results = qe.execSelect();

		assertEquals(results.hasNext(), true);

		while (results.hasNext()) {
			QuerySolution qs = results.nextSolution();

			for (java.util.Iterator iter = qs.varNames(); iter.hasNext();) {
				String key = (String) iter.next();

				if (key.equals("similarity1")) {
					String value = qs.get(key).asNode().getLiteralValue()
							.toString();
					assertEquals(new Double(value), new Double(1.0));
				} else if (key.equals("similarity2")) {
					String value = qs.get(key).asNode().getLiteralValue()
							.toString();
					assertEquals(new Double(value), new Double(1.0));
				} else if (key.equals("similarity3")) {
					String value = qs.get(key).asNode().getLiteralValue()
							.toString();
					assertEquals(new Double(value), new Double(1.0));
				} else if (key.equals("similarity4")) {
					String value = qs.get(key).asNode().getLiteralValue()
							.toString();
					assertEquals(new Double(value), new Double(3.0));
				}
			}
		}
	}
}
/* 
 * (c) Copyright 2007 University of Zurich, Switzerland
 * All rights reserved.
 *
 * This file is licensed under a GNU Lesser General Public License.
 *
 * For more information about license terms see LICENSE.txt that comes with
 * the main iSPARQL-OptARQ distribution or visit the GNU website at
 * http://www.gnu.org/licenses/lgpl.html
 *
 */
