package ch.unizh.ifi.isparql.tests.query.strategy;

import ch.unizh.ifi.isparql.tests.DefaultModel;
import com.hp.hpl.jena.rdf.model.Model;
import com.hp.hpl.jena.query.Query;
import com.hp.hpl.jena.query.QueryExecution;
import com.hp.hpl.jena.query.QueryExecutionFactory;
import com.hp.hpl.jena.query.QueryFactory;
import com.hp.hpl.jena.query.ResultSet;
import com.hp.hpl.jena.query.QuerySolution;
import junit.framework.TestCase;

/**
 * Test Levenshtein string comparison strategy
 *
 * @author		Markus Stocker
 * @version 	$Id: LevenshteinStrCmpTest.java,v 1.3 2006-11-22 12:32:53 ms Exp $
 */

public class LevenshteinStrCmpTest extends TestCase 
{	
	private Model model;
	private DefaultModel dm = new DefaultModel();
	
	public void testSimilarity()
	{		
		model = dm.getModel();
		
		query1();
	}
	
	private void query1()
	{	
		String qstr = "BASE					<" + DefaultModel.NS + "> " +
				      "PREFIX	person:		<" + DefaultModel.NSP + "> " +
				      "PREFIX	property:	<java:ch.unizh.ifi.isparql.query.property.> " +
				      "SELECT ?firstname ?lastname ?similarity " + 
				      "WHERE { " +
				      "?person person:firstname ?firstname . " +
				      "?person person:lastname ?lastname . " +
				      "?strategy property:name \"LevenshteinStrCmp\" . " +
				      "?strategy property:arguments (?firstname ?lastname) . " +
				      "?strategy property:similarity ?similarity " +
				      "} " +
				      "ORDER BY DESC(?similarity) " +
				      "LIMIT 1 ";
		
		Query query = QueryFactory.create(qstr);
		QueryExecution qe = QueryExecutionFactory.create(query, model);
		ResultSet results = qe.execSelect();
		
		assertEquals(results.hasNext(), true);
		
		while (results.hasNext())
		{
			QuerySolution qs = results.nextSolution();

			for (java.util.Iterator iter = qs.varNames(); iter.hasNext(); )
			{
				String key = (String)iter.next();
				String value = qs.get(key).asNode().getLiteralValue().toString();
				
				if (key.equals("firstname"))
					assertEquals(value, "Markus");
				else if (key.equals("lastname"))
					assertEquals(value, "Stocker");
				else if (key.equals("similarity"))
					assertEquals(new Double(value), new Double(0.14285714285714285));
			}
		}
	}
}

/* 
 * (c) Copyright 2007 University of Zurich, Switzerland
 * All rights reserved.
 *
 * This file is licensed under a GNU Lesser General Public License.
 *
 * For more information about license terms see LICENSE.txt that comes with
 * the main iSPARQL-OptARQ distribution or visit the GNU website at
 * http://www.gnu.org/licenses/lgpl.html
 *
 */
