/*
 * Created on 30.10.2004
 * OWL-S Matchmaker
 * 
 * COPYRIGHT NOTICE
 * 
 * Copyright (C) 2005 DFKI GmbH, Germany
 * Developed by Benedikt Fries, Matthias Klusch
 * 
 * The code is free for non-commercial use only.
 * You can redistribute it and/or modify it under the terms
 * of the Mozilla Public License version 1.1  as
 * published by the Mozilla Foundation at
 * http://www.mozilla.org/MPL/MPL-1.1.txt
 */
package owlsmx;

/**
 * Interface for generic matchmakers
 * 
 * @author bEn
 *
 */
public interface Matchmaker{
    
    /**
     * Adds a service to the matchmaker with the given ID,
     * loads it in advance from the given URI
     * 
     * @param integer		ID of the service
     * @param profileURI	URI from which the service will be loaded
     */
    public void addService(Integer integer, java.net.URI profileURI);
	
    /**
     * Removes a service with the given ID from the matchmaker
     * 
     * @param integer		ID of the service
     */
    public void removeService(Integer integer);
    
    /**
     * Matches the service request which will be loaded from the given URI
     * against all services registered at the matchmaker
     * 
     * @param profileURI     
     * @return SortedSet of (MatchedService) services 
     * @throws java.net.URISyntaxException
     * @throws owlsmx.exceptions.MatchingException
     */
    public java.util.SortedSet matchRequest(java.net.URI profileURI) throws java.net.URISyntaxException, owlsmx.exceptions.MatchingException;
    
    /**
     * Loads the last saved state of the matchmaker
     * 
     * @return If the matchmaker was loaded successfully
     */
    public boolean load();
    
    /**
     * Saves the current state of the matchmaker
     * 
     * @return If the matchmaker was saved successfully
     */
    public boolean save();
    
    /**
     * Clears the matchmaker
     */
    public void clear();
}
