/*
 * Created on 07.02.2005
 * OWL-S Matchmaker
 * 
 * COPYRIGHT NOTICE
 * 
 * Copyright (C) 2005 DFKI GmbH, Germany
 * Developed by Benedikt Fries, Matthias Klusch
 * 
 * The code is free for non-commercial use only.
 * You can redistribute it and/or modify it under the terms
 * of the Mozilla Public License version 1.1  as
 * published by the Mozilla Foundation at
 * http://www.mozilla.org/MPL/MPL-1.1.txt
 */
package owlsmx.data;

import java.net.URI;

import owlsmx.utils.MatchmakerUtils;


/**
 * Final representation of a matching service, which will be returned as result
 * 
 * @author bEn
 *
 */
public class MatchedService implements Comparable {
    public URI serviceURI;
    public int serviceID;
    public int degreeOfMatch;
    public double similarity;
    
    MatchedService(int serviceID, URI serviceURI, int degreeOfMatch, double similarity) {
        this.serviceID=serviceID;
        this.serviceURI=serviceURI;
        this.degreeOfMatch=degreeOfMatch;
        this.similarity=similarity;
    }
    
    MatchedService(ExtendedServiceInformation exInfo) {
        this.serviceID=exInfo.serviceID;
        this.degreeOfMatch=exInfo.degreeOfMatch;
        this.similarity=exInfo.similarity;
    }
 
    MatchedService(MatchingResult exInfo) {
        this.serviceID=exInfo.serviceID;
        this.degreeOfMatch=exInfo.degreeOfMatch;
        this.similarity=exInfo.similarity;
    }
    
    public MatchedService(MatchingResult exInfo, URI serviceURI) {
        this.serviceID=exInfo.serviceID;
        this.degreeOfMatch=exInfo.degreeOfMatch;
        this.similarity=exInfo.similarity;
        this.serviceURI = serviceURI;
    }
    
    MatchedService(ExtendedServiceInformation exInfo, URI serviceURI) {
        this.serviceID=exInfo.serviceID;
        this.degreeOfMatch=exInfo.degreeOfMatch;
        this.similarity=exInfo.similarity;
        this.serviceURI=serviceURI;
    }
    
    MatchedService(ServiceEntry entry, URI serviceURI) {
        this.serviceID=entry.ID;
        this.degreeOfMatch=entry.degreeOfMatch;
        this.similarity=entry.similarity;
        this.serviceURI=entry.uri;
    }
    
    
    public void setURI(URI serviceURI) {
        this.serviceURI=serviceURI;
    }
    
    public String toString() {
        return "(S-ID (" + serviceID + "), " +  MatchmakerUtils.degreeOfMatchIntToString(degreeOfMatch) + ", SIM(" + similarity + ", " +serviceURI + " )";
    }
    
    public int compareTo(Object obj2) {
        final int BEFORE = -1;
        final int EQUAL = 0;
        final int AFTER = 1;
        if (this == obj2) {
          return EQUAL;
        } 
        MatchedService service = (MatchedService) obj2;
        if ( (this.serviceID==service.serviceID) && (this.degreeOfMatch==service.degreeOfMatch) && (this.similarity==service.similarity) )
            return EQUAL;
        else if ( (this.degreeOfMatch < service.degreeOfMatch) || 
                    ( (this.degreeOfMatch==service.degreeOfMatch) && (this.similarity>service.similarity) ) )
            return BEFORE;
        return AFTER;
    }
}
